@@ -35,5 +35,5 @@
 int main(int argc, char **argv)
 {
     using ProblemTypeTag = Opm::Properties::TTag::FractureProblem;
-    return Opm::start<ProblemTypeTag>(argc, argv);
+    return Opm::start<ProblemTypeTag>(argc, argv, true);
 }
