@@ -15,6 +15,9 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "config.h"
+
+#include <opm/models/blackoil/blackoillocalresidualtpfa.hh>
+#include <opm/models/discretization/common/tpfalinearizer.hh>
 #include <opm/simulators/flow/Main.hpp>
 
 namespace Opm {
@@ -24,14 +27,26 @@ struct FlowBrineEnergyProblem {
     using InheritsFrom = std::tuple<FlowProblem>;
 };
 }
+
 template<class TypeTag>
 struct EnableBrine<TypeTag, TTag::FlowBrineEnergyProblem> {
     static constexpr bool value = true;
 };
+
 template<class TypeTag>
 struct EnableEnergy<TypeTag, TTag::FlowBrineEnergyProblem> {
     static constexpr bool value = true;
 };
+
+template<class TypeTag>
+struct Linearizer<TypeTag, TTag::FlowBrineEnergyProblem> {
+    using type = TpfaLinearizer<TypeTag>;
+};
+
+template<class TypeTag>
+struct LocalResidual<TypeTag, TTag::FlowBrineEnergyProblem> {
+    using type = BlackOilLocalResidualTPFA<TypeTag>;
+};
 }
 
 int flowBrineEnergyMain(int argc, char** argv)
