@@ -31,7 +31,7 @@
 #include <opm/simulators/linalg/parallelbicgstabbackend.hh>
 
 #include <flowexperimental/comp/EmptyModel.hpp>
-#include <flowexperimental/comp/wells/CompositionalWellModel.hpp>
+#include <flowexperimental/comp/wells/CompWellModel.hpp>
 
 // // the current code use eclnewtonmethod adding other conditions to proceed_ should do the trick for KA
 // // adding linearshe sould be chaning the update_ function in the same class with condition that the error is reduced.
@@ -123,7 +123,7 @@ struct AquiferModel<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
 
 template<class TypeTag, int NumComp, bool EnableWater>
 struct WellModel<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
-    using type = CompositionalWellModel<TypeTag>;
+    using type = CompWellModel<TypeTag>;
 };
 
 template<class TypeTag, int NumComp, bool EnableWater>
@@ -187,8 +187,8 @@ struct EnableMech<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
 };
 
 template<class TypeTag, int NumComp, bool EnableWater>
-struct EnableDisgasInWater<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> { 
-    static constexpr bool value = false; 
+struct EnableDisgasInWater<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
+    static constexpr bool value = false;
 };
 
 template<class TypeTag, int NumComp, bool EnableWater>
@@ -257,7 +257,7 @@ struct EnableExtbo<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
     static constexpr bool value = false;
 };
 template<class TypeTag, int NumComp, bool EnableWater>
-struct EnableMICP<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
+struct EnableBioeffects<TypeTag, TTag::FlowExpCompProblem<NumComp, EnableWater>> {
     static constexpr bool value = false;
 };
 
