@@ -28,9 +28,9 @@
 namespace Opm {
 
 template<>
-int dispatchFlowExpComp<2>(int argc, char** argv)
+int dispatchFlowExpComp<2, true>(int argc, char** argv)
 {
-    return start<Properties::TTag::FlowExpCompProblem<2>>(argc, argv, false);
+    return start<Properties::TTag::FlowExpCompProblem<2, true>>(argc, argv, false);
 }
 
 }
