@@ -30,14 +30,18 @@
 
 #include <opm/common/Exceptions.hpp>
 
-#include <opm/models/blackoil/blackoilproperties.hh>
-#include <opm/models/blackoil/blackoilmicpmodules.hh>
+#include <opm/models/blackoil/blackoilbioeffectsmodules.hh>
 #include <opm/models/blackoil/blackoilnewtonmethodparams.hpp>
+#include <opm/models/blackoil/blackoilproperties.hh>
 
 #include <opm/models/nonlinear/newtonmethod.hh>
 
 #include <opm/models/utils/signum.hh>
 
+#include <algorithm>
+#include <cmath>
+#include <vector>
+
 namespace Opm::Properties {
 
 template <class TypeTag, class MyTypeTag>
@@ -65,7 +69,7 @@ class BlackOilNewtonMethod : public GetPropType<TypeTag, Properties::DiscNewtonM
     using FluidSystem = GetPropType<TypeTag, Properties::FluidSystem>;
     using Scalar = GetPropType<TypeTag, Properties::Scalar>;
     using Linearizer = GetPropType<TypeTag, Properties::Linearizer>;
-    using MICPModule = BlackOilMICPModule<TypeTag>;
+    using BioeffectsModule = BlackOilBioeffectsModule<TypeTag>;
 
     static const unsigned numEq = getPropValue<TypeTag, Properties::NumEq>();
     static constexpr bool enableSaltPrecipitation = getPropValue<TypeTag, Properties::EnableSaltPrecipitation>();
@@ -117,6 +121,9 @@ protected:
 
     /*!
      * \copydoc FvBaseNewtonMethod::endIteration_
+     *
+     * \param uCurrentIter Current solution iterator
+     * \param uLastIter Last solution iterator
      */
     void endIteration_(SolutionVector& uCurrentIter,
                        const SolutionVector& uLastIter)
@@ -124,7 +131,7 @@ protected:
 #if HAVE_MPI
         // in the MPI enabled case we need to add up the number of DOF
         // for which the interpretation changed over all processes.
-        int localSwitched = numPriVarsSwitched_;
+        const int localSwitched = numPriVarsSwitched_;
         MPI_Allreduce(&localSwitched,
                       &numPriVarsSwitched_,
                       /*num=*/1,
@@ -204,7 +211,8 @@ protected:
         static constexpr bool enableEnergy = Indices::temperatureIdx >= 0;
         static constexpr bool enableFoam = Indices::foamConcentrationIdx >= 0;
         static constexpr bool enableBrine = Indices::saltConcentrationIdx >= 0;
-        static constexpr bool enableMICP = Indices::microbialConcentrationIdx >= 0;
+        static constexpr bool enableBioeffects = Indices::biofilmVolumeFractionIdx >= 0;
+        static constexpr bool enableMICP = Indices::enableMICP;
 
         currentValue.checkDefined();
         Valgrind::CheckDefined(update);
@@ -279,19 +287,22 @@ protected:
                     delta *= satAlpha;
                 }
                 else {
-                    //Ensure Rv and Rs factor does not become negative
-                    if (delta > currentValue[Indices::compositionSwitchIdx])
+                    // Ensure Rv and Rs factor does not become negative
+                    if (delta > currentValue[Indices::compositionSwitchIdx]) {
                         delta = currentValue[Indices::compositionSwitchIdx];
+                    }
                 }
             }
             else if (enableSolvent && pvIdx == Indices::solventSaturationIdx) {
                 // solvent saturation updates are also subject to the Appleyard chop
                 if (currentValue.primaryVarsMeaningSolvent() == PrimaryVariables::SolventMeaning::Ss) {
                     delta *= satAlpha;
-                } else {
-                    //Ensure Rssolw factor does not become negative
-                    if (delta > currentValue[Indices::solventSaturationIdx])
+                }
+                else {
+                    // Ensure Rssolw factor does not become negative
+                    if (delta > currentValue[Indices::solventSaturationIdx]) {
                         delta = currentValue[Indices::solventSaturationIdx];
+                    }
                 }
             }
             else if (enableExtbo && pvIdx == Indices::zFractionIdx) {
@@ -313,7 +324,8 @@ protected:
             }
             else if (enableBrine && pvIdx == Indices::saltConcentrationIdx &&
                      enableSaltPrecipitation &&
-                     currentValue.primaryVarsMeaningBrine() == PrimaryVariables::BrineMeaning::Sp) {
+                     currentValue.primaryVarsMeaningBrine() == PrimaryVariables::BrineMeaning::Sp)
+            {
                 const Scalar maxSaltSaturationChange = 0.1;
                 const Scalar sign = delta >= 0. ? 1. : -1.;
                 delta = sign * std::min(std::abs(delta), maxSaltSaturationChange);
@@ -354,11 +366,15 @@ protected:
 
             if (enableBrine && pvIdx == Indices::saltConcentrationIdx) {
                // keep the salt concentration above 0
-                if (!enableSaltPrecipitation || (enableSaltPrecipitation && currentValue.primaryVarsMeaningBrine() == PrimaryVariables::BrineMeaning::Cs)) {
+                if (!enableSaltPrecipitation ||
+                    currentValue.primaryVarsMeaningBrine() == PrimaryVariables::BrineMeaning::Cs)
+               {
                    nextValue[pvIdx] = std::max(nextValue[pvIdx], Scalar{0.0});
                 }
                // keep the salt saturation below upperlimit
-                if ((enableSaltPrecipitation && currentValue.primaryVarsMeaningBrine() == PrimaryVariables::BrineMeaning::Sp)) {
+                if (enableSaltPrecipitation &&
+                    currentValue.primaryVarsMeaningBrine() == PrimaryVariables::BrineMeaning::Sp)
+                {
                    nextValue[pvIdx] = std::min(nextValue[pvIdx], Scalar{1.0-1.e-8});
                 }
             }
@@ -374,26 +390,29 @@ protected:
 
             // keep the values above 0
             // for the biofilm and calcite, we set an upper limit equal to the initial porosity
-            // minus 1e-8. This prevents singularities (e.g., one of the calcite source term is 
+            // minus 1e-8. This prevents singularities (e.g., one of the calcite source term is
             // evaluated at 1/(iniPoro - calcite)). The value 1e-8 is taken from the salt precipitation
-            // clapping above. 
-            if constexpr (enableMICP) {
+            // clapping above.
+            if constexpr (enableBioeffects) {
                 if (pvIdx == Indices::microbialConcentrationIdx) {
                     nextValue[pvIdx] = std::max(nextValue[pvIdx], Scalar{0.0});
                 }
-                if (pvIdx == Indices::oxygenConcentrationIdx) {
-                    nextValue[pvIdx] = std::max(nextValue[pvIdx], Scalar{0.0});
+                if (pvIdx == Indices::biofilmVolumeFractionIdx) {
+                    nextValue[pvIdx] = std::clamp(nextValue[pvIdx],
+                                                  Scalar{0.0},
+                                                  this->problem().referencePorosity(globalDofIdx, 0) - 1e-8);
                 }
-                if (pvIdx == Indices::ureaConcentrationIdx) {
-                    nextValue[pvIdx] = std::max(nextValue[pvIdx], Scalar{0.0});
-                }
-                if (pvIdx == Indices::biofilmConcentrationIdx) {
-                    nextValue[pvIdx] = std::clamp(nextValue[pvIdx], Scalar{0.0},
-                                                                    this->problem().referencePorosity(globalDofIdx, 0) - 1e-8);
-                }
-                if (pvIdx == Indices::calciteConcentrationIdx) {
-                    nextValue[pvIdx] = std::clamp(nextValue[pvIdx], Scalar{0.0},
-                                                                    this->problem().referencePorosity(globalDofIdx, 0) - 1e-8);
+                if constexpr (enableMICP) {
+                    if (pvIdx == Indices::oxygenConcentrationIdx) {
+                        nextValue[pvIdx] = std::max(nextValue[pvIdx], Scalar{0.0});
+                    }
+                    if (pvIdx == Indices::ureaConcentrationIdx) {
+                        nextValue[pvIdx] = std::max(nextValue[pvIdx], Scalar{0.0});
+                    }
+                    if (pvIdx == Indices::calciteVolumeFractionIdx) {
+                        nextValue[pvIdx] = std::clamp(nextValue[pvIdx], Scalar{0.0},
+                                                                        this->problem().referencePorosity(globalDofIdx, 0) - 1e-8);
+                    }
                 }
             }
         }
