@@ -60,11 +60,12 @@ class VtkBlackOilSolventModule : public BaseOutputModule<TypeTag>
     using Evaluation = GetPropType<TypeTag, Properties::Evaluation>;
     using ElementContext = GetPropType<TypeTag, Properties::ElementContext>;
 
-    static const int vtkFormat = getPropValue<TypeTag, Properties::VtkOutputFormat>();
+    static constexpr auto vtkFormat = getPropValue<TypeTag, Properties::VtkOutputFormat>();
     using VtkMultiWriter = ::Opm::VtkMultiWriter<GridView, vtkFormat>;
 
     enum { enableSolvent = getPropValue<TypeTag, Properties::EnableSolvent>() };
 
+    using BufferType = typename ParentType::BufferType;
     using ScalarBuffer = typename ParentType::ScalarBuffer;
 
 public:
@@ -99,19 +100,19 @@ public:
             }
 
             if (params_.solventSaturationOutput_) {
-                this->resizeScalarBuffer_(solventSaturation_);
+                this->resizeScalarBuffer_(solventSaturation_, BufferType::Dof);
             }
             if (params_.solventRswOutput_) {
-                this->resizeScalarBuffer_(solventRsw_);
+                this->resizeScalarBuffer_(solventRsw_, BufferType::Dof);
             }
             if (params_.solventDensityOutput_) {
-                this->resizeScalarBuffer_(solventDensity_);
+                this->resizeScalarBuffer_(solventDensity_, BufferType::Dof);
             }
             if (params_.solventViscosityOutput_) {
-                this->resizeScalarBuffer_(solventViscosity_);
+                this->resizeScalarBuffer_(solventViscosity_, BufferType::Dof);
             }
             if (params_.solventMobilityOutput_) {
-                this->resizeScalarBuffer_(solventMobility_);
+                this->resizeScalarBuffer_(solventMobility_, BufferType::Dof);
             }
         }
     }
@@ -130,7 +131,7 @@ public:
             using Toolbox = MathToolbox<Evaluation>;
             for (unsigned dofIdx = 0; dofIdx < elemCtx.numPrimaryDof(/*timeIdx=*/0); ++dofIdx) {
                 const auto& intQuants = elemCtx.intensiveQuantities(dofIdx, /*timeIdx=*/0);
-                unsigned globalDofIdx = elemCtx.globalSpaceIndex(dofIdx, /*timeIdx=*/0);
+                const unsigned globalDofIdx = elemCtx.globalSpaceIndex(dofIdx, /*timeIdx=*/0);
 
                 if (params_.solventSaturationOutput_) {
                     solventSaturation_[globalDofIdx] =
@@ -166,29 +167,33 @@ public:
     void commitBuffers(BaseOutputWriter& baseWriter) override
     {
         if constexpr (enableSolvent) {
-            VtkMultiWriter* vtkWriter = dynamic_cast<VtkMultiWriter*>(&baseWriter);
-            if (!vtkWriter) {
+            if (!dynamic_cast<VtkMultiWriter*>(&baseWriter)) {
                 return;
             }
 
             if (params_.solventSaturationOutput_) {
-                this->commitScalarBuffer_(baseWriter, "saturation_solvent", solventSaturation_);
+                this->commitScalarBuffer_(baseWriter, "saturation_solvent",
+                                          solventSaturation_, BufferType::Dof);
             }
 
             if (params_.solventRswOutput_) {
-                this->commitScalarBuffer_(baseWriter, "dissolved_solvent", solventRsw_);
+                this->commitScalarBuffer_(baseWriter, "dissolved_solvent",
+                                          solventRsw_, BufferType::Dof);
             }
 
             if (params_.solventDensityOutput_) {
-                this->commitScalarBuffer_(baseWriter, "density_solvent", solventDensity_);
+                this->commitScalarBuffer_(baseWriter, "density_solvent",
+                                          solventDensity_, BufferType::Dof);
             }
 
             if (params_.solventViscosityOutput_) {
-                this->commitScalarBuffer_(baseWriter, "viscosity_solvent", solventViscosity_);
+                this->commitScalarBuffer_(baseWriter, "viscosity_solvent",
+                                          solventViscosity_, BufferType::Dof);
             }
 
             if (params_.solventMobilityOutput_) {
-                this->commitScalarBuffer_(baseWriter, "mobility_solvent", solventMobility_);
+                this->commitScalarBuffer_(baseWriter, "mobility_solvent",
+                                          solventMobility_, BufferType::Dof);
             }
         }
     }
