@@ -22,6 +22,8 @@
 
 #include <opm/models/discretization/common/fvbaseparameters.hh>
 
+#include <opm/models/nonlinear/newtonmethodparams.hpp>
+
 #include <opm/models/utils/parametersystem.hpp>
 
 #include <algorithm>
@@ -62,6 +64,8 @@ BlackoilModelParameters<Scalar>::BlackoilModelParameters()
     max_niter_inner_well_iter_ = Parameters::Get<Parameters::MaxNewtonIterationsWithInnerWellIterations>();
     shut_unsolvable_wells_ = Parameters::Get<Parameters::ShutUnsolvableWells>();
     max_inner_iter_wells_ = Parameters::Get<Parameters::MaxInnerIterWells>();
+    max_well_status_switch_inner_iter_ = Parameters::Get<Parameters::MaxWellStatusSwitchInInnerIterWells>();
+    max_well_status_switch_ = Parameters::Get<Parameters::MaxWellStatusSwitchForWells>();
     maxSinglePrecisionTimeStep_ = Parameters::Get<Parameters::MaxSinglePrecisionDays<Scalar>>() * 24 * 60 * 60;
     min_strict_cnv_iter_ = Parameters::Get<Parameters::MinStrictCnvIter>();
     min_strict_mb_iter_ = Parameters::Get<Parameters::MinStrictMbIter>();
@@ -91,7 +95,10 @@ BlackoilModelParameters<Scalar>::BlackoilModelParameters()
     max_local_solve_iterations_ = Parameters::Get<Parameters::MaxLocalSolveIterations>();
     local_tolerance_scaling_mb_ = Parameters::Get<Parameters::LocalToleranceScalingMb<Scalar>>();
     local_tolerance_scaling_cnv_ = Parameters::Get<Parameters::LocalToleranceScalingCnv<Scalar>>();
+    newton_max_iter_ = Parameters::Get<Parameters::NewtonMaxIterations>();
+    newton_min_iter_ = Parameters::Get<Parameters::NewtonMinIterations>();
     nldd_num_initial_newton_iter_ = Parameters::Get<Parameters::NlddNumInitialNewtonIter>();
+    nldd_relative_mobility_change_tol_ = Parameters::Get<Parameters::NlddRelativeMobilityChangeTol<Scalar>>();
     num_local_domains_ = Parameters::Get<Parameters::NumLocalDomains>();
     local_domains_partition_imbalance_ = std::max(Scalar{1.0}, Parameters::Get<Parameters::LocalDomainsPartitioningImbalance<Scalar>>());
     local_domains_partition_method_ = Parameters::Get<Parameters::LocalDomainsPartitioningMethod>();
@@ -110,6 +117,7 @@ BlackoilModelParameters<Scalar>::BlackoilModelParameters()
     monitor_params_.decay_factor_ = Parameters::Get<Parameters::ConvergenceMonitoringDecayFactor<Scalar>>();
 
     nupcol_group_rate_tolerance_ = Parameters::Get<Parameters::NupcolGroupRateTolerance<Scalar>>();
+    well_group_constraints_max_iterations_ = Parameters::Get<Parameters::WellGroupConstraintsMaxIterations>();
 }
 
 template<class Scalar>
@@ -129,7 +137,7 @@ void BlackoilModelParameters<Scalar>::registerParameters()
         ("Absolute maximum tolerated for residuals without cutting the time step size");
     Parameters::Register<Parameters::RelaxedMaxPvFraction<Scalar>>
         ("The fraction of the pore volume of the reservoir "
-         "where the volumetric error (CNV) may be voilated "
+         "where the volumetric error (CNV) may be violated "
          "during strict Newton iterations.");
     Parameters::Register<Parameters::ToleranceMb<Scalar>>
         ("Tolerated mass balance error relative to total mass present");
@@ -181,6 +189,10 @@ void BlackoilModelParameters<Scalar>::registerParameters()
         ("Shut unsolvable wells");
     Parameters::Register<Parameters::MaxInnerIterWells>
         ("Maximum number of inner iterations for standard wells");
+    Parameters::Register<Parameters::MaxWellStatusSwitchInInnerIterWells>
+        ("Maximum number of status switching (stop<->open) for a well during inner iterations.");
+    Parameters::Register<Parameters::MaxWellStatusSwitchForWells>
+        ("Maximum number of status switching (stop<->open) for a well during a time-step.");
     Parameters::Register<Parameters::AlternativeWellRateInit>
         ("Use alternative well rate initialization procedure");
     Parameters::Register<Parameters::RegularizationFactorWells<Scalar>>
@@ -222,7 +234,7 @@ void BlackoilModelParameters<Scalar>::registerParameters()
     Parameters::Register<Parameters::UseImplicitIpr>
         ("Compute implict IPR for stability checks and stable solution search");
     Parameters::Register<Parameters::CheckGroupConstraintsInnerWellIterations>
-        ("Allow checking of group constraints during inner well iterations");        
+        ("Allow checking of group constraints during inner well iterations");
     Parameters::Register<Parameters::NetworkMaxStrictOuterIterations>
         ("Maximum outer iterations in network solver before relaxing tolerance");
     Parameters::Register<Parameters::NetworkMaxOuterIterations>
@@ -237,6 +249,9 @@ void BlackoilModelParameters<Scalar>::registerParameters()
         ("Choose nonlinear solver. Valid choices are newton or nldd.");
     Parameters::Register<Parameters::LocalSolveApproach>
         ("Choose local solve approach. Valid choices are jacobi and gauss-seidel");
+    Parameters::SetDefault<Parameters::NewtonMaxIterations>(20);
+    Parameters::Register<Parameters::NewtonMinIterations>
+        ("The minimum number of Newton iterations per time step");
     Parameters::Register<Parameters::MaxLocalSolveIterations>
         ("Max iterations for local solves with NLDD nonlinear solver.");
     Parameters::Register<Parameters::LocalToleranceScalingMb<Scalar>>
@@ -245,6 +260,8 @@ void BlackoilModelParameters<Scalar>::registerParameters()
         ("Set lower than 1.0 to use stricter convergence tolerance for local solves.");
     Parameters::Register<Parameters::NlddNumInitialNewtonIter>
         ("Number of initial global Newton iterations when running the NLDD nonlinear solver.");
+    Parameters::Register<Parameters::NlddRelativeMobilityChangeTol<Scalar>>
+        ("Threshold for single cell relative mobility change in the NLDD solver");
     Parameters::Register<Parameters::NumLocalDomains>
         ("Number of local domains for NLDD nonlinear solver.");
     Parameters::Register<Parameters::LocalDomainsPartitioningImbalance<Scalar>>
@@ -276,6 +293,9 @@ void BlackoilModelParameters<Scalar>::registerParameters()
 
     Parameters::Hide<Parameters::DebugEmitCellPartition>();
 
+    Parameters::Register<Parameters::WellGroupConstraintsMaxIterations>
+    ("Maximum number of iterations in the well/group switching algorithm");
+
     // if openMP is available, use two threads per mpi rank by default
 #if _OPENMP
     Parameters::SetDefault<Parameters::ThreadsPerProcess>(2);
