@@ -103,6 +103,8 @@ struct MaxPressureChangeMsWells { static constexpr Scalar value = 10*1e5; };
 struct MaxNewtonIterationsWithInnerWellIterations { static constexpr int value = 8; };
 struct MaxInnerIterMsWells { static constexpr int value = 100; };
 struct MaxInnerIterWells { static constexpr int value = 50; };
+struct MaxWellStatusSwitchInInnerIterWells { static constexpr int value = 99; };
+struct MaxWellStatusSwitchForWells { static constexpr int value = 99; };
 struct ShutUnsolvableWells { static constexpr bool value = true; };
 struct AlternativeWellRateInit { static constexpr bool value = true; };
 struct StrictOuterIterWells { static constexpr int value = 6; };
@@ -139,13 +141,17 @@ struct NetworkMaxPressureUpdateInBars { static constexpr Scalar value = 5.0; };
 struct NonlinearSolver { static constexpr auto value = "newton"; };
 struct LocalSolveApproach { static constexpr auto value = "gauss-seidel"; };
 struct MaxLocalSolveIterations { static constexpr int value = 20; };
+struct NewtonMinIterations { static constexpr int value = 2; };
 
+struct WellGroupConstraintsMaxIterations { static constexpr int value = 1; };
 template<class Scalar>
 struct LocalToleranceScalingMb { static constexpr Scalar value = 1.0; };
 
 template<class Scalar>
 struct LocalToleranceScalingCnv { static constexpr Scalar value = 0.1; };
 struct NlddNumInitialNewtonIter { static constexpr int value = 1; };
+template<class Scalar>
+struct NlddRelativeMobilityChangeTol { static constexpr Scalar value = 0.1; };
 struct NumLocalDomains { static constexpr int value = 0; };
 
 template<class Scalar>
@@ -263,7 +269,7 @@ public:
     /// Update scaling factors for mass balance equations
     bool update_equations_scaling_;
 
-    /// Try to detect oscillation or stagnation.
+    /// Try to detect oscillation or stagnation
     bool use_update_stabilization_;
 
     /// Whether to use MultisegmentWell to handle multisegment wells
@@ -300,7 +306,7 @@ public:
     bool use_implicit_ipr_;
 
     /// Whether to allow checking/changing to group controls during inner well iterations
-    bool check_group_constraints_inner_well_iterations_; 
+    bool check_group_constraints_inner_well_iterations_;
 
     /// Maximum number of iterations in the network solver before relaxing tolerance
     int network_max_strict_outer_iterations_;
@@ -317,17 +323,35 @@ public:
     /// Maximum pressure update in the inner network pressure update iterations
     Scalar network_max_pressure_update_in_bars_;
 
-    /// Nonlinear solver type: newton or nldd.
+    /// Maximum number of iterations in the well/group switch algorithm
+    int well_group_constraints_max_iterations_;
+
+    /// Maximum number of status switches (open<->stop> in local well iterations
+    int max_well_status_switch_inner_iter_;
+
+    /// Maximum number of status switches (open<->stop> during a time step
+    int max_well_status_switch_;
+
+    /// Nonlinear solver type: newton or nldd
     std::string nonlinear_solver_;
-    /// 'jacobi' and 'gauss-seidel' supported.
+
+    /// 'jacobi' and 'gauss-seidel' supported
     DomainSolveApproach local_solve_approach_{DomainSolveApproach::Jacobi};
 
+    /// Maximum number of Newton iterations per time step
+    int newton_max_iter_;
+
+    /// Minimum number of Newton iterations per time step
+    int newton_min_iter_;
+
     int max_local_solve_iterations_;
 
     Scalar local_tolerance_scaling_mb_;
     Scalar local_tolerance_scaling_cnv_;
 
     int nldd_num_initial_newton_iter_{1};
+    /// Threshold for single cell relative mobility change in NLDD
+    Scalar nldd_relative_mobility_change_tol_;
     int num_local_domains_{0};
     Scalar local_domains_partition_imbalance_{1.03};
     std::string local_domains_partition_method_;
