@@ -77,6 +77,16 @@ struct EnableApiTracking { using type = UndefinedProperty; };
 template<class TypeTag, class MyTypeTag>
 struct EnableDebuggingChecks { using type = Properties::UndefinedProperty; };
 
+template<class TypeTag, class MyTypeTag>
+struct EnableHysteresis { using type = Properties::UndefinedProperty; };
+
+template<class TypeTag, class MyTypeTag>
+struct EnableEndpointScaling { using type = Properties::UndefinedProperty; };
+
+// Avoid using ElementContext-based code if possible.
+template<class TypeTag, class MyTypeTag>
+struct AvoidElementContext { using type = Properties::UndefinedProperty; };
+
 // if thermal flux boundaries are enabled an effort is made to preserve the initial
 // thermal gradient specified via the TEMPVD keyword
 template<class TypeTag, class MyTypeTag>
@@ -222,7 +232,7 @@ struct EnableExtbo<TypeTag, TTag::FlowBaseProblem>
 { static constexpr bool value = false; };
 
 template<class TypeTag>
-struct EnableMICP<TypeTag, TTag::FlowBaseProblem>
+struct EnableBioeffects<TypeTag, TTag::FlowBaseProblem>
 { static constexpr bool value = false; };
 
 // disable thermal flux boundaries by default
@@ -241,6 +251,20 @@ template<class TypeTag>
 struct EnableDebuggingChecks<TypeTag, TTag::FlowBaseProblem>
 { static constexpr bool value = true; };
 
+template<class TypeTag>
+struct EnableHysteresis<TypeTag, TTag::FlowBaseProblem>
+{ static constexpr bool value = true; };
+
+template<class TypeTag>
+struct EnableEndpointScaling<TypeTag, TTag::FlowBaseProblem>
+{ static constexpr bool value = true; };
+
+// Most modules are implemented only in terms of element contexts,
+// so this must default to false.
+template<class TypeTag>
+struct AvoidElementContext<TypeTag, TTag::FlowBaseProblem>
+{ static constexpr bool value = false; };
+
 } // namespace Opm::Properties
 
 #endif // OPM_BASE_FLOW_PROBLEM_PROPERTIES_HPP
