@@ -47,8 +47,9 @@
 #include <opm/input/eclipse/Schedule/Action/State.hpp>
 #include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
-#include <opm/input/eclipse/Schedule/Group/GroupEconProductionLimits.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
+#include <opm/input/eclipse/Schedule/Group/GroupEconProductionLimits.hpp>
+#include <opm/input/eclipse/Schedule/Group/GroupSatelliteInjection.hpp>
 #include <opm/input/eclipse/Schedule/Group/GSatProd.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
 #include <opm/input/eclipse/Schedule/Network/Balance.hpp>
@@ -72,6 +73,7 @@
 #include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEconProductionLimits.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellFoamProperties.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellFractureSeeds.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellMICPProperties.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellPolymerProperties.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
@@ -92,9 +94,12 @@
 #include <mpi.h>
 #endif // HAVE_MPI
 
+#include <algorithm>
 #include <filesystem>
 #include <stdexcept>
 
+#include <fmt/format.h>
+
 namespace Opm {
 
 std::unique_ptr<Parallel::Communication> FlowGenericVanguard::comm_;
@@ -110,18 +115,49 @@ FlowGenericVanguard::FlowGenericVanguard(SimulationModelParams&& params)
     defineSimulationModel(std::move(params));
 
     fileName_ = Parameters::Get<Parameters::EclDeckFileName>();
-    edgeWeightsMethod_   = Dune::EdgeWeightMethod(Parameters::Get<Parameters::EdgeWeightsMethod>());
+    const std::string ewm = Parameters::Get<Parameters::EdgeWeightsMethod>();
+    if (ewm == "uniform") {
+        edgeWeightsMethod_ = Dune::EdgeWeightMethod::uniformEdgeWgt;
+    } else if (ewm == "transmissibility") {
+        edgeWeightsMethod_ = Dune::EdgeWeightMethod::defaultTransEdgeWgt;
+    } else if (ewm == "logtrans") {
+        edgeWeightsMethod_ = Dune::EdgeWeightMethod::logTransEdgeWgt;
+    } else {
+        std::string msg = fmt::format("Unknown value for --edge-weights-method parameter: '{}'. "
+                                      "Accepted values are 'uniform', 'transmissibility' and 'logtrans'.",
+                                      ewm);
+        OpmLog::error(msg);
+        throw std::runtime_error(msg);
+    }
 
 #if HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
     numJacobiBlocks_ = Parameters::Get<Parameters::NumJacobiBlocks>();
 #endif // HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
 
     ownersFirst_ = Parameters::Get<Parameters::OwnerCellsFirst>();
+    edgeConformal_ = Parameters::Get<Parameters::EdgeConformal>();
 
 #if HAVE_MPI
     numOverlap_ = Parameters::Get<Parameters::NumOverlap>();
     addCorners_ = Parameters::Get<Parameters::AddCorners>();
-    partitionMethod_ = Dune::PartitionMethod(Parameters::Get<Parameters::PartitionMethod>());
+
+    const std::string pm = Parameters::Get<Parameters::PartitionMethod>();
+    if (pm == "simple") {
+        partitionMethod_ = Dune::PartitionMethod::simple;
+    } else if (pm == "zoltan") {
+        partitionMethod_ = Dune::PartitionMethod::zoltan;
+    } else if (pm == "metis") {
+        partitionMethod_ = Dune::PartitionMethod::metis;
+    } else if (pm == "zoltanwell") {
+        partitionMethod_ = Dune::PartitionMethod::zoltanGoG;
+    } else {
+        std::string msg = fmt::format("Unknown value for --partition-method parameter: '{}'. "
+                                      "Accepted values are 'simple', 'zoltan', 'metis', and 'zoltanwell'.",
+                                      pm);
+        OpmLog::error(msg);
+        throw std::runtime_error(msg);
+    }
+
     serialPartitioning_ = Parameters::Get<Parameters::SerialPartitioning>();
     zoltanParams_ = Parameters::Get<Parameters::ZoltanParams>();
     zoltanPhgEdgeSizeThreshold_ = Parameters::Get<Parameters::ZoltanPhgEdgeSizeThreshold>();
@@ -306,7 +342,7 @@ void FlowGenericVanguard::init()
     if(!comm_){
         FlowGenericVanguard::setCommunication(std::make_unique<Parallel::Communication>());
     }
-    
+
     // set eclState if not already set as in opm flow
     // it means that setParams is called
     if(!eclState_){
@@ -314,7 +350,7 @@ void FlowGenericVanguard::init()
         this->defineSimulationModel(std::move(this->modelParams_));
     }
 
-    
+
     if (!this->summaryState_) {
         this->summaryState_ = std::make_unique<SummaryState>
             (TimeService::from_time_t(this->eclSchedule_->getStartTime()),
@@ -342,10 +378,8 @@ void FlowGenericVanguard::init()
             if (comm.rank() == 0)
             {
                 const auto& wells = this->schedule().getWellsatEnd();
-                for (const auto& well : wells)
-                {
-                    hasMsWell = hasMsWell || well.isMultiSegment();
-                }
+                hasMsWell = std::any_of(wells.begin(), wells.end(),
+                                        [](const auto& well) { return well.isMultiSegment(); });
             }
         }
 
@@ -367,14 +401,12 @@ void FlowGenericVanguard::init()
 
 bool FlowGenericVanguard::drsdtconEnabled() const
 {
-  for (const auto& schIt : this->schedule()) {
-      const auto& oilVaporizationControl = schIt.oilvap();
-      if (oilVaporizationControl.getType() == OilVaporizationProperties::OilVaporization::DRSDTCON) {
-          return true;
-      }
-  }
-
-  return false;
+    return std::any_of(this->schedule().begin(), this->schedule().end(),
+                       [](const auto& schIt)
+                       {
+                           return schIt.oilvap().getType() ==
+                              OilVaporizationProperties::OilVaporization::DRSDTCON;
+                       });
 }
 
 std::unordered_map<size_t, const NumericalAquiferCell*>
@@ -448,7 +480,7 @@ void FlowGenericVanguard::registerParameters_()
         ("When restarting: should we try to initialize wells and "
          "groups from historical SCHEDULE section.");
     Parameters::Register<Parameters::EdgeWeightsMethod>
-        ("Choose edge-weighing strategy: 0=uniform, 1=trans, 2=log(trans).");
+        ("Choose edge-weighing strategy: 'uniform', 'transmissibility', or 'logtrans' (logarithm of transmissibility).");
 
 #if HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
     Parameters::Register<Parameters::NumJacobiBlocks>
@@ -457,14 +489,17 @@ void FlowGenericVanguard::registerParameters_()
 
     Parameters::Register<Parameters::OwnerCellsFirst>
         ("Order cells owned by rank before ghost/overlap cells.");
+    Parameters::Register<Parameters::EdgeConformal>
+        ("Edge conformal cornerpoint processing.");
+
 #if HAVE_MPI
     Parameters::Register<Parameters::AddCorners>
         ("Add corners to partition.");
     Parameters::Register<Parameters::NumOverlap>
         ("Numbers of layers overlap in parallel partition");
     Parameters::Register<Parameters::PartitionMethod>
-        ("Choose partitioning strategy: 0=simple, 1=Zoltan, 2=METIS, "
-         "3=Zoltan with all cells of well represented by one vertex.");
+        ("Choose partitioning method: 'simple', 'zoltan', 'metis', or "
+         "'zoltanwell' (Zoltan with all cells perforated by a well represented by a single vertex).");
     Parameters::Register<Parameters::SerialPartitioning>
         ("Perform partitioning for parallel runs on a single process.");
     Parameters::Register<Parameters::ZoltanImbalanceTol<Scalar>>
