@@ -51,7 +51,7 @@ struct AllowDistributedWells { static constexpr bool value = false; };
 struct AllowSplittingInactiveWells { static constexpr bool value = true; };
 
 struct EclOutputInterval { static constexpr int value = -1; };
-struct EdgeWeightsMethod  { static constexpr int value = 1; };
+struct EdgeWeightsMethod  { static constexpr auto value = "transmissibility"; };
 struct EnableDryRun { static constexpr auto value = "auto"; };
 struct EnableEclOutput { static constexpr auto value = true; };
 struct EnableOpmRstFile { static constexpr bool value = false; };
@@ -74,9 +74,10 @@ struct ActionParsingStrictness { static constexpr auto value = "normal"; };
 
 /// 0: simple, 1: Zoltan, 2: METIS, 3: Zoltan with a all cells of a well
 /// represented by one vertex in the graph, see GridEnums.hpp
-struct PartitionMethod { static constexpr int value = 3; };
+struct PartitionMethod { static constexpr auto value = "zoltanwell"; };
 struct AddCorners { static constexpr bool value = false; };
 struct NumOverlap { static constexpr int value = 1; };
+struct EdgeConformal { static constexpr bool value = false; };
 
 struct SchedRestart{ static constexpr bool value = false; };
 struct SerialPartitioning{ static constexpr bool value = false; };
@@ -259,6 +260,9 @@ public:
     bool ownersFirst() const
     { return ownersFirst_; }
 
+    bool edgeConformal() const
+    { return edgeConformal_; }
+
 #if HAVE_MPI
     bool addCorners() const
     { return addCorners_; }
@@ -372,6 +376,8 @@ protected:
 #endif // HAVE_OPENCL || HAVE_ROCSPARSE || HAVE_CUDA
 
     bool ownersFirst_;
+    bool edgeConformal_;
+
 #if HAVE_MPI
     bool addCorners_;
     int numOverlap_;
