@@ -2,7 +2,7 @@
 // vi: set et ts=4 sw=4 sts=4:
 /*
   Copyright 2023 INRIA
-  
+
   This file is part of the Open Porous Media project (OPM).
 
   OPM is free software: you can redistribute it and/or modify
@@ -105,6 +105,7 @@ protected:
     using GlobalEqVector = GetPropType<TypeTag, Properties::GlobalEqVector>;
     using EqVector = GetPropType<TypeTag, Properties::EqVector>;
     using Vanguard = GetPropType<TypeTag, Properties::Vanguard>;
+    using Indices = GetPropType<TypeTag, Properties::Indices>;
 
     // Grid and world dimension
     enum { dim = GridView::dimension };
@@ -115,15 +116,16 @@ protected:
     enum { numPhases = FluidSystem::numPhases };
     enum { numComponents = FluidSystem::numComponents };
 
-    enum { enableConvectiveMixing = getPropValue<TypeTag, Properties::EnableConvectiveMixing>() };
+    enum { enableBioeffects = getPropValue<TypeTag, Properties::EnableBioeffects>() };
     enum { enableBrine = getPropValue<TypeTag, Properties::EnableBrine>() };
+    enum { enableConvectiveMixing = getPropValue<TypeTag, Properties::EnableConvectiveMixing>() };
     enum { enableDiffusion = getPropValue<TypeTag, Properties::EnableDiffusion>() };
     enum { enableDispersion = getPropValue<TypeTag, Properties::EnableDispersion>() };
     enum { enableEnergy = getPropValue<TypeTag, Properties::EnableEnergy>() };
     enum { enableExperiments = getPropValue<TypeTag, Properties::EnableExperiments>() };
     enum { enableExtbo = getPropValue<TypeTag, Properties::EnableExtbo>() };
     enum { enableFoam = getPropValue<TypeTag, Properties::EnableFoam>() };
-    enum { enableMICP = getPropValue<TypeTag, Properties::EnableMICP>() };
+    enum { enableMICP = Indices::enableMICP };
     enum { enablePolymer = getPropValue<TypeTag, Properties::EnablePolymer>() };
     enum { enablePolymerMolarWeight = getPropValue<TypeTag, Properties::EnablePolymerMW>() };
     enum { enableSaltPrecipitation = getPropValue<TypeTag, Properties::EnableSaltPrecipitation>() };
@@ -154,7 +156,6 @@ protected:
     using MaterialLaw = GetPropType<TypeTag, Properties::MaterialLaw>;
     using DofMapper = GetPropType<TypeTag, Properties::DofMapper>;
     using Evaluation = GetPropType<TypeTag, Properties::Evaluation>;
-    using Indices = GetPropType<TypeTag, Properties::Indices>;
     using IntensiveQuantities = GetPropType<TypeTag, Properties::IntensiveQuantities>;
     using WellModel = GetPropType<TypeTag, Properties::WellModel>;
     using AquiferModel = GetPropType<TypeTag, Properties::AquiferModel>;
@@ -164,7 +165,7 @@ protected:
 
 
     using TracerModel = GetPropType<TypeTag, Properties::TracerModel>;
-    using DirectionalMobilityPtr = Utility::CopyablePtr<DirectionalMobility<TypeTag, Evaluation>>;
+    using DirectionalMobilityPtr = Utility::CopyablePtr<DirectionalMobility<TypeTag>>;
 
 public:
     using BaseType::briefDescription;
@@ -186,6 +187,12 @@ public:
 
     /*!
      * \copydoc FvBaseProblem::handlePositionalParameter
+     *
+     * \param addKey Callback for adding a key-value pair
+     * \param seenParams Already seen parameters
+     * \param errorMsg Error message
+     * \param argv Command line parameters
+     * \param paramIdx Index of parameter to handle
      */
     static int handlePositionalParameter(std::function<void(const std::string&,
                                                             const std::string&)> addKey,
@@ -342,7 +349,7 @@ public:
     /*!
      * \brief Called by the simulator before each time integration.
      */
-    void beginTimeStep()
+    virtual void beginTimeStep()
     {
         OPM_TIMEBLOCK(beginTimeStep);
         const int episodeIdx = this->episodeIndex();
@@ -369,7 +376,6 @@ public:
         wellModel_.beginTimeStep();
         aquiferModel_.beginTimeStep();
         tracerModel_.beginTimeStep();
-
     }
 
     /*!
@@ -473,10 +479,11 @@ public:
     virtual void writeOutput(bool verbose)
     {
         OPM_TIMEBLOCK(problemWriteOutput);
-        // use the generic code to prepare the output fields and to
-        // write the desired VTK files.
+
         if (Parameters::Get<Parameters::EnableWriteAllSolutions>() ||
-            this->simulator().episodeWillBeOver()) {
+            this->episodeWillBeOver())
+        {
+            // Create VTK output as needed.
             ParentType::writeOutput(verbose);
         }
     }
@@ -774,19 +781,20 @@ public:
     std::shared_ptr<const EclMaterialLawManager> materialLawManager() const
     { return materialLawManager_; }
 
-    template <class FluidState>
+    template <class FluidState, class ...Args>
     void updateRelperms(
         std::array<Evaluation,numPhases> &mobility,
         DirectionalMobilityPtr &dirMob,
         FluidState &fluidState,
         unsigned globalSpaceIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(updateRelperms);
+        using ContainerT = std::array<Evaluation, numPhases>;
+        OPM_TIMEBLOCK_LOCAL(updateRelperms, Subsystem::SatProps);
         {
             // calculate relative permeabilities. note that we store the result into the
             // mobility_ class attribute. the division by the phase viscosity happens later.
             const auto& materialParams = materialLawParams(globalSpaceIdx);
-            MaterialLaw::relativePermeabilities(mobility, materialParams, fluidState);
+            MaterialLaw::template relativePermeabilities<ContainerT, FluidState, Args...>(mobility, materialParams, fluidState);
             Valgrind::CheckDefined(mobility);
         }
         if (materialLawManager_->hasDirectionalRelperms()
@@ -794,12 +802,12 @@ public:
         {
             using Dir = FaceDir::DirEnum;
             constexpr int ndim = 3;
-            dirMob = std::make_unique<DirectionalMobility<TypeTag, Evaluation>>();
+            dirMob = std::make_unique<DirectionalMobility<TypeTag>>();
             Dir facedirs[ndim] = {Dir::XPlus, Dir::YPlus, Dir::ZPlus};
             for (int i = 0; i<ndim; i++) {
                 const auto& materialParams = materialLawParams(globalSpaceIdx, facedirs[i]);
                 auto& mob_array = dirMob->getArray(i);
-                MaterialLaw::relativePermeabilities(mob_array, materialParams, fluidState);
+                MaterialLaw::template relativePermeabilities<ContainerT, FluidState, Args...>(mob_array, materialParams, fluidState);
             }
         }
     }
@@ -973,7 +981,7 @@ public:
                 unsigned globalDofIdx,
                 unsigned timeIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(eclProblemSource);
+        OPM_TIMEBLOCK_LOCAL(eclProblemSource, Subsystem::Assembly);
         rate = 0.0;
 
         // Add well contribution to source here.
@@ -1051,7 +1059,7 @@ public:
     template <class LhsEval>
     LhsEval rockCompPoroMultiplier(const IntensiveQuantities& intQuants, unsigned elementIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(rockCompPoroMultiplier);
+        OPM_TIMEBLOCK_LOCAL(rockCompPoroMultiplier, Subsystem::PvtProps);
         if (this->rockCompPoroMult_.empty() && this->rockCompPoroMultWc_.empty())
             return 1.0;
 
@@ -1107,19 +1115,19 @@ public:
     template <class LhsEval>
     LhsEval wellTransMultiplier(const IntensiveQuantities& intQuants, unsigned elementIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(wellTransMultiplier);
-        
+        OPM_TIMEBLOCK_LOCAL(wellTransMultiplier, Subsystem::Wells);
+
         const bool implicit = !this->explicitRockCompaction_;
         double trans_mult = implicit ? this->simulator().problem().template computeRockCompTransMultiplier_<double>(intQuants, elementIdx)
                                      : this->simulator().problem().getRockCompTransMultVal(elementIdx);
         trans_mult *= this->simulator().problem().template permFactTransMultiplier<double>(intQuants, elementIdx);
-    
+
         return trans_mult;
     }
 
     std::pair<BCType, RateVector> boundaryCondition(const unsigned int globalSpaceIdx, const int directionId) const
     {
-        OPM_TIMEBLOCK_LOCAL(boundaryCondition);
+        OPM_TIMEBLOCK_LOCAL(boundaryCondition, Subsystem::Assembly);
         if (!nonTrivialBoundaryConditions_) {
             return { BCType::NONE, RateVector(0.0) };
         }
@@ -1139,13 +1147,13 @@ public:
         RateVector rate = 0.0;
         switch (bc.component) {
         case BCComponent::OIL:
-            rate[Indices::canonicalToActiveComponentIndex(oilCompIdx)] = bc.rate;
+            rate[FluidSystem::canonicalToActiveCompIdx(oilCompIdx)] = bc.rate;
             break;
         case BCComponent::GAS:
-            rate[Indices::canonicalToActiveComponentIndex(gasCompIdx)] = bc.rate;
+            rate[FluidSystem::canonicalToActiveCompIdx(gasCompIdx)] = bc.rate;
             break;
         case BCComponent::WATER:
-            rate[Indices::canonicalToActiveComponentIndex(waterCompIdx)] = bc.rate;
+            rate[FluidSystem::canonicalToActiveCompIdx(waterCompIdx)] = bc.rate;
             break;
         case BCComponent::SOLVENT:
             this->handleSolventBC(bc, rate);
@@ -1230,7 +1238,7 @@ protected:
 
     bool updateMaxOilSaturation_(unsigned compressedDofIdx, const IntensiveQuantities& iq)
     {
-        OPM_TIMEBLOCK_LOCAL(updateMaxOilSaturation);
+        OPM_TIMEBLOCK_LOCAL(updateMaxOilSaturation, Subsystem::SatProps);
         const auto& fs = iq.fluidState();
         const Scalar So = decay<Scalar>(fs.saturation(refPressurePhaseIdx_()));
         auto& mos = this->maxOilSaturation_;
@@ -1261,7 +1269,7 @@ protected:
 
     bool updateMaxWaterSaturation_(unsigned compressedDofIdx, const IntensiveQuantities& iq)
     {
-        OPM_TIMEBLOCK_LOCAL(updateMaxWaterSaturation);
+        OPM_TIMEBLOCK_LOCAL(updateMaxWaterSaturation, Subsystem::SatProps);
         const auto& fs = iq.fluidState();
         const Scalar Sw = decay<Scalar>(fs.saturation(waterPhaseIdx));
         auto& mow = this->maxWaterSaturation_;
@@ -1289,7 +1297,7 @@ protected:
     }
 
     bool updateMinPressure_(unsigned compressedDofIdx, const IntensiveQuantities& iq){
-        OPM_TIMEBLOCK_LOCAL(updateMinPressure);
+        OPM_TIMEBLOCK_LOCAL(updateMinPressure, Subsystem::PvtProps);
         const auto& fs = iq.fluidState();
         const Scalar min_pressure = getValue(fs.pressure(refPressurePhaseIdx_()));
         auto& min_pressures = this->minRefPressure_;
@@ -1452,7 +1460,7 @@ protected:
 
     bool updateHysteresis_(unsigned compressedDofIdx, const IntensiveQuantities& iq)
     {
-        OPM_TIMEBLOCK_LOCAL(updateHysteresis_);
+        OPM_TIMEBLOCK_LOCAL(updateHysteresis_, Subsystem::SatProps);
         materialLawManager_->updateHysteresis(iq.fluidState(), compressedDofIdx);
         //TODO change materials to give a bool
         return true;
@@ -1629,7 +1637,7 @@ protected:
     template <class LhsEval>
     LhsEval computeRockCompTransMultiplier_(const IntensiveQuantities& intQuants, unsigned elementIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(computeRockCompTransMultiplier);
+        OPM_TIMEBLOCK_LOCAL(computeRockCompTransMultiplier, Subsystem::PvtProps);
         if (this->rockCompTransMult_.empty() && this->rockCompTransMultWc_.empty())
             return 1.0;
 
@@ -1648,7 +1656,7 @@ protected:
 
         if (!this->overburdenPressure_.empty())
             effectivePressure -= this->overburdenPressure_[elementIdx];
-        
+
         if (rock_config.store()) {
             effectivePressure -= asImp_().initialFluidState(elementIdx).pressure(refPressurePhaseIdx_());
         }
@@ -1719,6 +1727,13 @@ protected:
     BCData<int> bcindex_;
     bool nonTrivialBoundaryConditions_ = false;
     bool first_step_ = true;
+
+    /// Whether or not the current episode will end at the end of the
+    /// current time step.
+    virtual bool episodeWillBeOver() const
+    {
+        return this->simulator().episodeWillBeOver();
+    }
 };
 
 } // namespace Opm
