@@ -3,7 +3,7 @@
 /*
   Copyright 2023 INRIA
   Copyright 2024 SINTEF Digital
-  
+
   This file is part of the Open Porous Media project (OPM).
 
   OPM is free software: you can redistribute it and/or modify
@@ -52,6 +52,8 @@
 #include <opm/simulators/flow/MixingRateControls.hpp>
 #include <opm/simulators/flow/OutputBlackoilModule.hpp>
 #include <opm/simulators/flow/VtkTracerModule.hpp>
+#include <opm/simulators/flow/HybridNewton.hpp> 
+#include <opm/simulators/flow/HybridNewtonConfig.hpp> 
 
 #include <opm/simulators/utils/satfunc/SatfuncConsistencyCheckManager.hpp>
 
@@ -62,7 +64,8 @@
 #include <algorithm>
 #include <cstddef>
 #include <functional>
-#include <set>
+#include <limits>
+#include <memory>
 #include <stdexcept>
 #include <string>
 #include <string_view>
@@ -98,8 +101,9 @@ private:
     using FlowProblemType::numComponents;
 
     // TODO: potentially some cleaning up depending on the usage later here
-    using FlowProblemType::enableConvectiveMixing;
+    using FlowProblemType::enableBioeffects;
     using FlowProblemType::enableBrine;
+    using FlowProblemType::enableConvectiveMixing;
     using FlowProblemType::enableDiffusion;
     using FlowProblemType::enableDispersion;
     using FlowProblemType::enableEnergy;
@@ -132,19 +136,25 @@ private:
     using typename FlowProblemType::MaterialLaw;
     using typename FlowProblemType::DimMatrix;
 
+    enum { enableDissolvedGas = Indices::compositionSwitchIdx >= 0 };
+    enum { enableVapwat = getPropValue<TypeTag, Properties::EnableVapwat>() };
+    enum { enableDisgasInWater = getPropValue<TypeTag, Properties::EnableDisgasInWater>() };
+
     using SolventModule = BlackOilSolventModule<TypeTag>;
     using PolymerModule = BlackOilPolymerModule<TypeTag>;
     using FoamModule = BlackOilFoamModule<TypeTag>;
     using BrineModule = BlackOilBrineModule<TypeTag>;
     using ExtboModule = BlackOilExtboModule<TypeTag>;
-    using MICPModule = BlackOilMICPModule<TypeTag>;
+    using BioeffectsModule = BlackOilBioeffectsModule<TypeTag>;
     using DispersionModule = BlackOilDispersionModule<TypeTag, enableDispersion>;
     using DiffusionModule = BlackOilDiffusionModule<TypeTag, enableDiffusion>;
     using ConvectiveMixingModule = BlackOilConvectiveMixingModule<TypeTag, enableConvectiveMixing>;
     using ModuleParams = typename BlackOilLocalResidualTPFA<TypeTag>::ModuleParams;
+    using HybridNewton = BlackOilHybridNewton<TypeTag>;
 
     using InitialFluidState = typename EquilInitializer<TypeTag>::ScalarFluidState;
     using EclWriterType = EclWriter<TypeTag, OutputBlackOilModule<TypeTag> >;
+    using IndexTraits = typename FluidSystem::IndexTraitsType;
 #if HAVE_DAMARIS
     using DamarisWriterType = DamarisWriter<TypeTag>;
 #endif
@@ -181,8 +191,9 @@ public:
                          simulator.vanguard().summaryState(),
                          this->wellModel_,
                          simulator.vanguard().grid().comm())
+        , hybridNewton_(simulator)
     {
-        this->model().addOutputModule(new VtkTracerModule<TypeTag>(simulator));
+        this->model().addOutputModule(std::make_unique<VtkTracerModule<TypeTag>>(simulator));
 
         // Tell the black-oil extensions to initialize their internal data structures
         const auto& vanguard = simulator.vanguard();
@@ -203,9 +214,9 @@ public:
         foamParams.template initFromState<enableFoam>(vanguard.eclState());
         FoamModule::setParams(std::move(foamParams));
 
-        BlackOilMICPParams<Scalar> micpParams;
-        micpParams.template initFromState<enableMICP>(vanguard.eclState());
-        MICPModule::setParams(std::move(micpParams));
+        BlackOilBioeffectsParams<Scalar> bioeffectsParams;
+        bioeffectsParams.template initFromState<enableBioeffects, enableMICP>(vanguard.eclState());
+        BioeffectsModule::setParams(std::move(bioeffectsParams));
 
         BlackOilPolymerParams<Scalar> polymerParams;
         polymerParams.template initFromState<enablePolymer, enablePolymerMolarWeight>(vanguard.eclState());
@@ -257,6 +268,15 @@ public:
                                              this->moduleParams_.convectiveMixingModuleParam);
     }
 
+    /*!
+     * \brief Called by the simulator before each time integration.
+     */
+    void beginTimeStep() override
+    {
+        FlowProblemType::beginTimeStep();
+        hybridNewton_.tryApplyHybridNewton();
+    }
+
     /*!
      * \copydoc FvBaseProblem::finishInit
      */
@@ -334,8 +354,6 @@ public:
             this->maxTimeStepAfterWellEvent_ = tuning.TMAXWC;
         }
 
-        this->initFluidSystem_();
-
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) &&
             FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
             this->maxOilSaturation_.resize(this->model().numGridDof(), 0.0);
@@ -346,9 +364,8 @@ public:
                                   {
                                       std::array<int,dim> coords;
                                       simulator.vanguard().cartesianCoordinate(idx, coords);
-                                      for (auto& c : coords) {
-                                          ++c;
-                                      }
+                                      std::transform(coords.begin(), coords.end(), coords.begin(),
+                                                     [](const auto c) { return c + 1; });
                                       return coords;
                                   });
 
@@ -431,6 +448,17 @@ public:
             simulator.model().applyInitialSolution();
             FlowProblemType::writeOutput(true);
         }
+
+        if (!eclState.getIOConfig().initOnly()) {
+            if (!this->enableTuning_ && eclState.getSimulationConfig().anyTUNING()) {
+                OpmLog::info("\nThe deck has TUNING in the SCHEDULE section, but "
+                             "it is ignored due\nto the flag --enable-tuning=false. "
+                             "Set this flag to true to activate it.\n"
+                             "Manually tuning the simulator with the TUNING keyword may "
+                             "increase run time.\nIt is recommended using the simulator's "
+                             "default tuning (--enable-tuning=false).");
+            }
+        }
     }
 
     /*!
@@ -454,9 +482,7 @@ public:
         using GridType = std::remove_cv_t<std::remove_reference_t<decltype(grid)>>;
         constexpr bool isCpGrid = std::is_same_v<GridType, Dune::CpGrid>;
         if (!isCpGrid || (grid.maxLevel() == 0)) {
-            const bool isSubStep = !this->simulator().episodeWillBeOver();
-
-            this->eclWriter_->evalSummaryState(isSubStep);
+            this->eclWriter_->evalSummaryState(!this->episodeWillBeOver());
         }
 
         {
@@ -523,22 +549,24 @@ public:
      * \brief Write the requested quantities of the current solution into the output
      *        files.
      */
-    void writeOutput(bool verbose) override
+    void writeOutput(const bool verbose) override
     {
         FlowProblemType::writeOutput(verbose);
 
-        bool isSubStep = !this->simulator().episodeWillBeOver();
+        const auto isSubStep = !this->episodeWillBeOver();
+
+        auto localCellData = data::Solution {};
 
-        data::Solution localCellData = {};
 #if HAVE_DAMARIS
         // N.B. the Damaris output has to be done before the ECL output as the ECL one
         // does all kinds of std::move() relocation of data
-        if (enableDamarisOutput_) {
-            damarisWriter_->writeOutput(localCellData, isSubStep) ;
+        if (this->enableDamarisOutput_ && (this->damarisWriter_ != nullptr)) {
+            this->damarisWriter_->writeOutput(localCellData, isSubStep);
         }
 #endif
-        if (enableEclOutput_){
-            eclWriter_->writeOutput(std::move(localCellData), isSubStep);
+
+        if (this->enableEclOutput_ && (this->eclWriter_ != nullptr)) {
+            this->eclWriter_->writeOutput(std::move(localCellData), isSubStep);
         }
     }
 
@@ -604,7 +632,7 @@ public:
                 if constexpr (getPropValue<TypeTag, Properties::BlackoilConserveSurfaceVolume>()) {
                     mass_rate /= FluidSystem::referenceDensity(phaseIdx, pvtRegionIdx);
                 }
-                rate[Indices::canonicalToActiveComponentIndex(compIdx)] += mass_rate;
+                rate[FluidSystem::canonicalToActiveCompIdx(compIdx)] += mass_rate;
             }
 
             if constexpr (enableSolvent) {
@@ -687,7 +715,7 @@ public:
     template <class LhsEval>
     LhsEval permFactTransMultiplier(const IntensiveQuantities& intQuants, unsigned elementIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(permFactTransMultiplier);
+        OPM_TIMEBLOCK_LOCAL(permFactTransMultiplier, Subsystem::PvtProps);
         if constexpr (enableSaltPrecipitation) {
             const auto& fs = intQuants.fluidState();
             unsigned tableIdx = this->simulator().problem().satnumRegionIndex(elementIdx);
@@ -696,7 +724,7 @@ public:
             const auto& permfactTable = BrineModule::permfactTable(tableIdx);
             return permfactTable.eval(porosityFactor, /*extrapolation=*/true);
         }
-        else if constexpr (enableMICP) {
+        else if constexpr (enableBioeffects) {
             return intQuants.permFactor().value();
         }
         else {
@@ -725,7 +753,7 @@ public:
 
     InitialFluidState boundaryFluidState(unsigned globalDofIdx, const int directionId) const
     {
-        OPM_TIMEBLOCK_LOCAL(boundaryFluidState);
+        OPM_TIMEBLOCK_LOCAL(boundaryFluidState, Subsystem::Assembly);
         const auto& bcprop = this->simulator().vanguard().schedule()[this->episodeIndex()].bcprop;
         if (bcprop.size() > 0) {
             FaceDir::DirEnum dir = FaceDir::FromIntersectionIndex(directionId);
@@ -783,8 +811,6 @@ public:
                 Valgrind::CheckDefined(pc);
                 for (unsigned activePhaseIdx = 0; activePhaseIdx < FluidSystem::numActivePhases(); ++activePhaseIdx) {
                     const auto phaseIdx = FluidSystem::activeToCanonicalPhaseIdx(activePhaseIdx);
-
-                    fluidState.setPc(phaseIdx, pc[phaseIdx]);
                     if (Indices::oilEnabled)
                         fluidState.setPressure(phaseIdx, pressure + (pc[phaseIdx] - pc[oilPhaseIdx]));
                     else if (Indices::gasEnabled)
@@ -800,15 +826,22 @@ public:
                     temperature = *temperature_input;
                 fluidState.setTemperature(temperature);
 
-                if (FluidSystem::enableDissolvedGas()) {
-                    fluidState.setRs(0.0);
-                    fluidState.setRv(0.0);
+                if constexpr (enableDissolvedGas) {
+                    if (FluidSystem::enableDissolvedGas()) {
+                        fluidState.setRs(0.0);
+                        fluidState.setRv(0.0);
+                    }
                 }
-                if (FluidSystem::enableDissolvedGasInWater()) {
-                    fluidState.setRsw(0.0);
+                if constexpr (enableDisgasInWater) {
+                    if (FluidSystem::enableDissolvedGasInWater()) {
+                        fluidState.setRsw(0.0);
+                    }
+                }
+                if constexpr (enableVapwat) {
+                    if (FluidSystem::enableVaporizedWater()) {
+                        fluidState.setRvw(0.0);
+                    }
                 }
-                if (FluidSystem::enableVaporizedWater())
-                    fluidState.setRvw(0.0);
 
                 for (unsigned activePhaseIdx = 0; activePhaseIdx < FluidSystem::numActivePhases(); ++activePhaseIdx) {
                     const auto phaseIdx = FluidSystem::activeToCanonicalPhaseIdx(activePhaseIdx);
@@ -818,7 +851,7 @@ public:
 
                     const auto& rho = FluidSystem::density(fluidState, phaseIdx, pvtRegionIdx);
                     fluidState.setDensity(phaseIdx, rho);
-                    if (enableEnergy) {
+                    if constexpr (enableEnergy) {
                         const auto& h = FluidSystem::enthalpy(fluidState, phaseIdx, pvtRegionIdx);
                         fluidState.setEnthalpy(phaseIdx, h);
                     }
@@ -909,12 +942,14 @@ public:
             }
         }
 
-        if constexpr (enableMICP){
-            values[Indices::microbialConcentrationIdx] = this->micp_.microbialConcentration[globalDofIdx];
-            values[Indices::oxygenConcentrationIdx]= this->micp_.oxygenConcentration[globalDofIdx];
-            values[Indices::ureaConcentrationIdx]= this->micp_.ureaConcentration[globalDofIdx];
-            values[Indices::calciteConcentrationIdx]= this->micp_.calciteConcentration[globalDofIdx];
-            values[Indices::biofilmConcentrationIdx]= this->micp_.biofilmConcentration[globalDofIdx];
+        if constexpr (enableBioeffects) {
+            values[Indices::microbialConcentrationIdx] = this->bioeffects_.microbialConcentration[globalDofIdx];
+            values[Indices::biofilmVolumeFractionIdx]= this->bioeffects_.biofilmVolumeFraction[globalDofIdx];
+            if constexpr (enableMICP) {
+                values[Indices::oxygenConcentrationIdx]= this->bioeffects_.oxygenConcentration[globalDofIdx];
+                values[Indices::ureaConcentrationIdx]= this->bioeffects_.ureaConcentration[globalDofIdx];
+                values[Indices::calciteVolumeFractionIdx]= this->bioeffects_.calciteVolumeFraction[globalDofIdx];
+            }
         }
 
         values.checkDefined();
@@ -943,7 +978,7 @@ public:
                   unsigned spaceIdx,
                   unsigned timeIdx) const
     {
-        OPM_TIMEBLOCK_LOCAL(eclProblemBoundary);
+        OPM_TIMEBLOCK_LOCAL(eclProblemBoundary, Subsystem::Assembly);
         if (!context.intersection(spaceIdx).boundary())
             return;
 
@@ -1002,15 +1037,15 @@ public:
             this->polymer_.moleWeight.resize(numElems, 0.0);
         }
 
-        if constexpr (enableMICP) {
-            this->micp_.resize(numElems);
+        if constexpr (enableBioeffects) {
+            this->bioeffects_.resize(numElems);
         }
 
         // Initialize mixing controls before trying to set any lastRx valuesx
         this->mixControls_.init(numElems, restart_step, eclState.runspec().tabdims().getNumPVTTables());
 
-        if constexpr (enableMICP) {
-            this->micp_ = this->eclWriter_->outputModule().getMICP().getSolution();
+        if constexpr (enableBioeffects) {
+            this->bioeffects_ = this->eclWriter_->outputModule().getBioeffects().getSolution();
         }
 
         for (std::size_t elemIdx = 0; elemIdx < numElems; ++elemIdx) {
@@ -1168,8 +1203,8 @@ protected:
                 continue;
 
             Scalar avgB = numTotalDof / sumInvB[phaseIdx];
-            unsigned solventCompIdx = FluidSystem::solventComponentIndex(phaseIdx);
-            unsigned activeSolventCompIdx = Indices::canonicalToActiveComponentIndex(solventCompIdx);
+            const unsigned solventCompIdx = FluidSystem::solventComponentIndex(phaseIdx);
+            const unsigned activeSolventCompIdx = FluidSystem::canonicalToActiveCompIdx(solventCompIdx);
             this->model().setEqWeight(activeSolventCompIdx, avgB);
         }
     }
@@ -1427,21 +1462,26 @@ protected:
                     dofFluidState.setPressure(phaseIdx, pressure);
             }
 
-            if (FluidSystem::enableDissolvedGas())
-                dofFluidState.setRs(rsData[dofIdx]);
-            else if (Indices::gasEnabled && Indices::oilEnabled)
-                dofFluidState.setRs(0.0);
+            if constexpr (enableDissolvedGas) {
+                if (FluidSystem::enableDissolvedGas())
+                    dofFluidState.setRs(rsData[dofIdx]);
+                else if (Indices::gasEnabled && Indices::oilEnabled)
+                    dofFluidState.setRs(0.0);
+                if (FluidSystem::enableVaporizedOil())
+                    dofFluidState.setRv(rvData[dofIdx]);
+                else if (Indices::gasEnabled && Indices::oilEnabled)
+                    dofFluidState.setRv(0.0);
+            }
 
-            if (FluidSystem::enableDissolvedGasInWater() && has_rsw)
-                dofFluidState.setRsw(rswData[dofIdx]);
+            if constexpr (enableDisgasInWater) {
+                if (FluidSystem::enableDissolvedGasInWater() && has_rsw)
+                    dofFluidState.setRsw(rswData[dofIdx]);
+            }
 
-            if (FluidSystem::enableVaporizedOil())
-                dofFluidState.setRv(rvData[dofIdx]);
-            else if (Indices::gasEnabled && Indices::oilEnabled)
-                dofFluidState.setRv(0.0);
-
-            if (FluidSystem::enableVaporizedWater())
-                dofFluidState.setRvw(rvwData[dofIdx]);
+            if constexpr (enableVapwat) {
+                if (FluidSystem::enableVaporizedWater())
+                    dofFluidState.setRvw(rvwData[dofIdx]);
+            }
 
             //////
             // set invB_
@@ -1499,11 +1539,12 @@ protected:
     {
         FlowProblemType::readInitialCondition_();
 
-        if constexpr (enableSolvent || enablePolymer || enablePolymerMolarWeight || enableMICP)
+        if constexpr (enableSolvent || enablePolymer || enablePolymerMolarWeight || enableBioeffects)
             this->readBlackoilExtentionsInitialConditions_(this->model().numGridDof(),
                                                            enableSolvent,
                                                            enablePolymer,
                                                            enablePolymerMolarWeight,
+                                                           enableBioeffects,
                                                            enableMICP);
 
     }
@@ -1601,8 +1642,11 @@ protected:
         const auto isIoRank = this->simulator().vanguard()
             .grid().comm().rank() == ioRank;
 
+        // Note: Run saturation function consistency checks on main grid
+        // only (i.e., levelGridView(0)).  These checks are not supported
+        // for LGRs at this time.
         sfuncConsistencyChecks.collectFailuresTo(ioRank)
-            .run(this->simulator().vanguard().grid().leafGridView(),
+            .run(this->simulator().vanguard().grid().levelGridView(0),
                  [&vg   = this->simulator().vanguard(),
                   &emap = this->simulator().model().elementMapper()]
                  (const auto& elem)
@@ -1660,9 +1704,37 @@ protected:
 #endif
     MixingRateControls<FluidSystem> mixControls_;
 
-    ActionHandler<Scalar> actionHandler_;
+    ActionHandler<Scalar, IndexTraits> actionHandler_;
 
     ModuleParams moduleParams_;
+
+    HybridNewton hybridNewton_;
+
+private:
+    /// Whether or not the current epsiode will end at the end of the
+    /// current time step.
+    ///
+    /// Custom implementation for black-oil cases.
+    ///
+    /// [2025-08-14] This is arguably a hack.  We intentionally do not use
+    /// the 'simulator()'s episodeWillBeOver() function, as that has been
+    /// shown to have non-trivial false negatives in the context of certain
+    /// regression tests.  It is likely that we will need to revisit this
+    /// predicate in the future.
+    bool episodeWillBeOver() const override
+    {
+        const auto currTime = this->simulator().time()
+            + this->simulator().timeStepSize();
+
+        const auto nextReportStep =
+            this->simulator().vanguard().schedule()
+            .seconds(this->simulator().episodeIndex() + 1);
+
+        const auto isSubStep = (nextReportStep - currTime)
+            > (2 * std::numeric_limits<float>::epsilon()) * nextReportStep;
+
+        return !isSubStep;
+    }
 };
 
 } // namespace Opm
