@@ -22,6 +22,7 @@
 */
 
 #include <config.h>
+#include <filesystem>
 #include <opm/simulators/flow/GenericCpGridVanguard.hpp>
 
 #include <dune/grid/common/mcmgmapper.hh>
@@ -32,6 +33,7 @@
 #include <opm/common/utility/ActiveGridCells.hpp>
 
 #include <opm/grid/cpgrid/GridHelpers.hpp>
+#include <opm/grid/cpgrid/LevelCartesianIndexMapper.hpp>
 
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
@@ -71,6 +73,12 @@ namespace details {
 std::vector<int>
 MPIPartitionFromFile::operator()(const Dune::CpGrid& grid) const
 {
+    // Check if the external partition file exists
+    if (!std::filesystem::exists(this->partitionFile_)) {
+        OPM_THROW(std::runtime_error,
+                  fmt::format("External partition file '{}' does not exist.", this->partitionFile_.string()));
+    }
+
     std::ifstream pfile { this->partitionFile_ };
 
     auto partition = std::vector<int> {
@@ -299,7 +307,13 @@ distributeFieldProps_(EclipseState& eclState1)
     {
         // Reset Cartesian index mapper for automatic creation of field
         // properties
-        parallelEclState->resetCartesianMapper(this->cartesianIndexMapper_.get());
+        // Note: the previous cartesianIndexMapper_ has been replaces by levelCartesianIndexMapper_
+        // to support also the case of a distributed level zero grid in a CpGrid with LGRs.
+        // This change allows access to the Cartesian indices of the distributed level zero grid,
+        // where the field properties are given - for now.
+        // In case of supporting LGR field properties, this need to be adapted, to access instead
+        // each local/level Cartesian index set.
+        parallelEclState->resetCartesianMapper(this->levelCartesianIndexMapper_.get());
         parallelEclState->switchToDistributedProps();
     }
     else {
@@ -435,7 +449,8 @@ distributeGrid(const Dune::EdgeWeightMethod                          edgeWeights
 #endif  // HAVE_MPI
 
 template<class ElementMapper, class GridView, class Scalar>
-void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(EclipseState& eclState)
+void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::
+doCreateGrids_(const bool edge_conformal, EclipseState& eclState)
 {
     const auto isRoot = this->mpiRank == 0;
 
@@ -465,22 +480,27 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
                                &eclState,
                                /* isPeriodic = */ false,
                                /* flipNormals = */ false,
-                               /* clipZ = */ false);
+                               /* clipZ = */ false,
+                               edge_conformal);
 
     if (isRoot) {
         const auto& active_porv = eclState.fieldProps().porv(false);
         const auto& unit_system = eclState.getUnits();
-        const auto& volume_unit = unit_system.name( UnitSystem::measure::volume);
-        double total_pore_volume = unit_system.from_si( UnitSystem::measure::volume, std::accumulate(active_porv.begin(), active_porv.end(), 0.0));
-        OpmLog::info(fmt::format("Total number of active cells: {} / total pore volume: {:0.0f} {}", grid_->numCells(), total_pore_volume , volume_unit));
+        const auto& volume_unit = unit_system.name(UnitSystem::measure::volume);
+        double total_pore_volume = unit_system.from_si(UnitSystem::measure::volume,
+                                                       std::accumulate(active_porv.begin(), active_porv.end(), 0.0));
+        OpmLog::info(fmt::format("Total number of active cells: {} / total pore volume: {:0.0f} {}",
+                                 grid_->numCells(), total_pore_volume , volume_unit));
 
-        double removed_pore_volume = 0;
-        for (const auto& global_index : removed_cells)
-            removed_pore_volume += active_porv[ eclState.getInputGrid().activeIndex(global_index) ];
+        double removed_pore_volume =
+            std::accumulate(removed_cells.begin(), removed_cells.end(), 0.0,
+                            [&active_porv, &eclState](const auto acc, const auto global_index)
+                            { return acc + active_porv[eclState.getInputGrid().activeIndex(global_index)]; });
 
         if (removed_pore_volume > 0) {
-            removed_pore_volume = unit_system.from_si( UnitSystem::measure::volume, removed_pore_volume );
-            OpmLog::info(fmt::format("Removed {} cells with a pore volume of {:0.0f} {} ({:5.3f} %) due to MINPV/MINPVV",
+            removed_pore_volume = unit_system.from_si(UnitSystem::measure::volume, removed_pore_volume);
+            OpmLog::info(fmt::format("Removed {} cells with a pore volume of {:0.0f} "
+                                     "{} ({:5.3f} %) due to MINPV/MINPVV",
                                      removed_cells.size(),
                                      removed_pore_volume,
                                      volume_unit,
@@ -489,6 +509,7 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
     }
 
     cartesianIndexMapper_ = std::make_unique<CartesianIndexMapper>(*grid_);
+    levelCartesianIndexMapper_ = std::make_unique<LevelCartesianIndexMapper>(*grid_);
 
 #if HAVE_MPI
     if (this->grid_->comm().size() > 1) {
@@ -498,7 +519,7 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
         if (eclState.aquifer().hasNumericalAquifer()) {
             auto nnc_input = eclState.getInputNNC();
             Parallel::MpiSerializer ser(this->grid_->comm());
-            ser.broadcast(nnc_input);
+            ser.broadcast(Parallel::RootRank{0}, nnc_input);
 
             if (! isRoot) {
                 eclState.setInputNNC(nnc_input);
@@ -511,7 +532,7 @@ void GenericCpGridVanguard<ElementMapper,GridView,Scalar>::doCreateGrids_(Eclips
         if (hasPinchNnc) {
             auto pinch_nnc = eclState.getPinchNNC();
             Parallel::MpiSerializer ser(this->grid_->comm());
-            ser.broadcast(pinch_nnc);
+            ser.broadcast(Parallel::RootRank{0}, pinch_nnc);
 
             if (! isRoot) {
                 eclState.setPinchNNC(std::move(pinch_nnc));
@@ -629,7 +650,7 @@ template<class ElementMapper, class GridView, class Scalar>
 const LevelCartesianIndexMapper<Dune::CpGrid>
 GenericCpGridVanguard<ElementMapper,GridView,Scalar>::levelCartesianIndexMapper() const
 {
-    return LevelCartesianIndexMapper(*grid_);
+    return *levelCartesianIndexMapper_;
 }
 
 template<class ElementMapper, class GridView, class Scalar>
