@@ -30,7 +30,7 @@
 
 #include <opm/material/fluidmatrixinteractions/EclHysteresisConfig.hpp>
 #include <opm/material/fluidsystems/BlackOilFluidSystem.hpp>
-#include <opm/material/fluidsystems/BlackOilDefaultIndexTraits.hpp>
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
 #include <opm/material/fluidsystems/GenericOilGasWaterFluidSystem.hpp>
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
@@ -41,6 +41,7 @@
 #include <opm/input/eclipse/Schedule/RPTConfig.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
+#include <opm/input/eclipse/Schedule/Well/NameOrder.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 
@@ -76,12 +77,13 @@ namespace {
     defineInterRegionFlowArrays(const Opm::EclipseState&  eclState,
                                 const Opm::SummaryConfig& summaryConfig)
     {
-        auto regions = std::vector<Opm::InterRegFlowMap::SingleRegion>{};
-
-        const auto& fprops = eclState.fieldProps();
-        for (const auto& arrayName : summaryConfig.fip_regions_interreg_flow()) {
-            regions.push_back({ arrayName, std::cref(fprops.get_int(arrayName)) });
-        }
+        using SRegion = Opm::InterRegFlowMap::SingleRegion;
+        auto regions = std::vector<SRegion>{};
+        const auto fip_regions = summaryConfig.fip_regions_interreg_flow();
+        std::transform(fip_regions.begin(), fip_regions.end(),
+                       std::back_inserter(regions),
+                       [&fprops = eclState.fieldProps()](const auto& arrayName)
+                       { return SRegion{arrayName, std::cref(fprops.get_int(arrayName))}; });
 
         return regions;
     }
@@ -134,7 +136,7 @@ GenericOutputBlackoilModule(const EclipseState& eclState,
                             bool enableBrine,
                             bool enableSaltPrecipitation,
                             bool enableExtbo,
-                            bool enableMICP)
+                            bool enableBioeffects)
     : eclState_(eclState)
     , schedule_(schedule)
     , summaryState_(summaryState)
@@ -152,11 +154,11 @@ GenericOutputBlackoilModule(const EclipseState& eclState,
     , enableBrine_(enableBrine)
     , enableSaltPrecipitation_(enableSaltPrecipitation)
     , enableExtbo_(enableExtbo)
-    , enableMICP_(enableMICP)
+    , enableBioeffects_(enableBioeffects)
     , flowsC_(schedule, summaryConfig)
     , rftC_(eclState_, schedule_,
-            [this](const std::string& wname)
-            { return !isDefunctParallelWell(wname); })
+            [this](const std::string& wname) { return this->isOwnedByCurrentRank(wname); },
+            [this](const std::string& wname) { return this->isOnCurrentRank(wname); })
     , rst_conv_(std::move(globalCell), comm)
     , local_data_valid_(false)
 {
@@ -278,7 +280,7 @@ calc_inplace(std::map<std::string, double>& miscSummaryData,
              const Parallel::Communication& comm)
 {
     auto inplace = this->accumulateRegionSums(comm);
-    
+
     if (comm.rank() != 0)
         return inplace;
 
@@ -286,7 +288,7 @@ calc_inplace(std::map<std::string, double>& miscSummaryData,
                               miscSummaryData,
                               regionData);
 
-    
+
     return inplace;
 }
 
@@ -364,7 +366,7 @@ assignToSolution(data::Solution& sol)
     addEntry(baseSolutionVector, "RSSAT",    UnitSystem::measure::gas_oil_ratio,                         gasDissolutionFactor_);
     addEntry(baseSolutionVector, "RV",       UnitSystem::measure::oil_gas_ratio,                         rv_);
     addEntry(baseSolutionVector, "RVSAT",    UnitSystem::measure::oil_gas_ratio,                         oilVaporizationFactor_);
-    addEntry(baseSolutionVector, "SALT",     UnitSystem::measure::salinity,                              cSalt_);
+    addEntry(baseSolutionVector, "SALT",     UnitSystem::measure::concentration,                         cSalt_);
     addEntry(baseSolutionVector, "SGMAX",    UnitSystem::measure::identity,                              sgmax_);
     addEntry(baseSolutionVector, "SHMAX",    UnitSystem::measure::identity,                              shmax_);
     addEntry(baseSolutionVector, "SOMAX",    UnitSystem::measure::identity,                              soMax_);
@@ -400,8 +402,9 @@ assignToSolution(data::Solution& sol)
 
     this->flowsC_.outputRestart(sol);
 
-    if (this->micpC_.allocated()) {
-        this->micpC_.outputRestart(sol);
+    if (this->bioeffectsC_.allocated()) {
+        // Biofilms for gas-water systems; MICP only for water systems
+        this->bioeffectsC_.outputRestart(sol, !FluidSystem::phaseIsActive(gasPhaseIdx));
     }
 
     for (auto& array : extendedSolutionArrays) {
@@ -569,8 +572,9 @@ setRestart(const data::Solution& sol,
                   [&assign](const auto& p)
                   { assign(p.first, *p.second); });
 
-    if (this->micpC_.allocated()) {
-        this->micpC_.readRestart(globalDofIndex, elemIdx, sol);
+    if (this->bioeffectsC_.allocated()) {
+        // Biofilms for gas-water systems; MICP only for water systems
+        this->bioeffectsC_.readRestart(globalDofIndex, elemIdx, sol, !FluidSystem::phaseIsActive(gasPhaseIdx));
     }
 }
 
@@ -629,7 +633,7 @@ doAllocBuffers(const unsigned bufferSize,
 
     // Output RESTART_OPM_EXTENDED only when explicitly requested by user.
     for (auto& [keyword, should_write] : rstKeywords) {
-        if (this->isOutputCreationDirective_(keyword)) {
+        if (this->isOutputCreationDirective_(keyword) || keyword == "WELLS") {
             // 'BASIC', 'FREQ' and similar.  Don't attempt to create
             // cell-based output for these keywords and don't warn about
             // not being able to create such cell-based result vectors.
@@ -760,7 +764,7 @@ doAllocBuffers(const unsigned bufferSize,
        Entry{&cFoam_,                             "", enableFoam_},
        Entry{&cSalt_,                             "", enableBrine_},
        Entry{&pSalt_,                             "", enableSaltPrecipitation_},
-       Entry{&permFact_,                          "", enableSaltPrecipitation_ || enableMICP_},
+       Entry{&permFact_,                          "", enableSaltPrecipitation_ || enableBioeffects_},
        Entry{&soMax_,                             "", oilvap.getType() == OilVapP::VAPPARS},
        Entry{&soMax_,                             "", hysteresisConfig &&
                                                       hysteresisConfig->enableNonWettingHysteresis() &&
@@ -934,8 +938,9 @@ doAllocBuffers(const unsigned bufferSize,
         extboC_.allocate(bufferSize);
     }
 
-    if (enableMICP_) {
-        this->micpC_.allocate(bufferSize);
+    if (enableBioeffects_) {
+        // Biofilms for gas-water systems; MICP only for water systems
+        this->bioeffectsC_.allocate(bufferSize, !FluidSystem::phaseIsActive(gasPhaseIdx));
     }
 
     // tracers
@@ -1181,7 +1186,10 @@ void GenericOutputBlackoilModule<FluidSystem>::
 setupExtraBlockData(const std::size_t        reportStepNum,
                     std::function<bool(int)> isCartIdxOnThisRank)
 {
-    for (const auto& well : schedule_.getWells(reportStepNum - 1)) {
+    const auto& sched = this->schedule_[reportStepNum - 1];
+
+    for (const auto& wname : sched.well_order()) {
+        const auto& well = sched.wells.get(wname);
         for (const auto& connection : well.getConnections()) {
             if (isCartIdxOnThisRank(connection.global_index())) {
                 this->extraBlockData_.emplace(std::piecewise_construct,
@@ -1200,7 +1208,7 @@ assignGlobalFieldsToSolution(data::Solution& sol)
     this->rst_conv_.outputRestart(sol);
 }
 
-template<class T> using FS = BlackOilFluidSystem<T,BlackOilDefaultIndexTraits>;
+template<class T> using FS = BlackOilFluidSystem<T, BlackOilDefaultFluidSystemIndices>;
 
 #define INSTANTIATE_TYPE(T) \
     template class GenericOutputBlackoilModule<FS<T>>;
