@@ -35,6 +35,7 @@
 #include <dune/istl/schwarz.hh>
 
 #include <opm/common/OpmLog/OpmLog.hpp>
+#include <opm/common/TimingMacros.hpp>
 
 #include <opm/grid/CpGrid.hpp>
 
@@ -115,8 +116,9 @@ template<class Grid, class GridView, class DofMapper, class Stencil, class Fluid
 Scalar GenericTracerModel<Grid,GridView,DofMapper,Stencil,FluidSystem,Scalar>::
 freeTracerConcentration(int tracerIdx, int globalDofIdx) const
 {
-    if (freeTracerConcentration_.empty())
+    if (freeTracerConcentration_.empty()) {
         return 0.0;
+    }
 
     return freeTracerConcentration_[tracerIdx][globalDofIdx];
 }
@@ -125,8 +127,9 @@ template<class Grid, class GridView, class DofMapper, class Stencil, class Fluid
 Scalar GenericTracerModel<Grid,GridView,DofMapper,Stencil,FluidSystem,Scalar>::
 solTracerConcentration(int tracerIdx, int globalDofIdx) const
 {
-    if (solTracerConcentration_.empty())
+    if (solTracerConcentration_.empty()) {
         return 0.0;
+    }
 
     return solTracerConcentration_[tracerIdx][globalDofIdx];
 }
@@ -205,9 +208,11 @@ enableSolTracers() const
 
 template<class Grid, class GridView, class DofMapper, class Stencil, class FluidSystem, class Scalar>
 Scalar GenericTracerModel<Grid,GridView,DofMapper,Stencil,FluidSystem,Scalar>::
-currentConcentration_(const Well& eclWell, const std::string& name) const
+currentConcentration_(const Well& eclWell, const std::string& trName, const SummaryState& summaryState) const
 {
-    return eclWell.getTracerProperties().getConcentration(name);
+    return eclWell.getTracerProperties().getConcentration(WellTracerProperties::Well { eclWell.name() },
+                                                          WellTracerProperties::Tracer { trName },
+                                                          summaryState);
 }
 
 template<class Grid, class GridView, class DofMapper, class Stencil, class FluidSystem, class Scalar>
@@ -224,8 +229,9 @@ doInit(bool rst, std::size_t numGridDof,
 {
     const auto& tracers = eclState_.tracer();
 
-    if (tracers.size() == 0)
+    if (tracers.size() == 0) {
         return; // tracer treatment is supposed to be disabled
+    }
 
     // retrieve the number of tracers from the deck
     const std::size_t numTracers = tracers.size();
@@ -278,7 +284,7 @@ doInit(bool rst, std::size_t numGridDof,
                     free_tvdp.evaluate("TRACER_CONCENTRATION",
                                        centroids_(globalDofIdx)[2]);
             }
-        } 
+        }
         else {
             OpmLog::warning(fmt::format("No TBLKF or TVDPF given for free tracer {}. "
                                         "Initial values set to zero. ", tracer.name));
@@ -289,8 +295,8 @@ doInit(bool rst, std::size_t numGridDof,
         }
 
         // Solution tracer initialization only needed for gas/oil tracers with DISGAS/VAPOIL active
-        if (tracer.phase != Phase::WATER && 
-            ((tracer.phase == Phase::GAS && FluidSystem::enableDissolvedGas()) || 
+        if (tracer.phase != Phase::WATER &&
+            ((tracer.phase == Phase::GAS && FluidSystem::enableDissolvedGas()) ||
              (tracer.phase == Phase::OIL && FluidSystem::enableVaporizedOil()))) {
             // TBLKS keyword
             if (tracer.solution_concentration.has_value()){
@@ -318,7 +324,7 @@ doInit(bool rst, std::size_t numGridDof,
                         solution_tvdp.evaluate("TRACER_CONCENTRATION",
                                             centroids_(globalDofIdx)[2]);
                 }
-            } 
+            }
             else {
                 // No solution tracers, default to zero
                 enableSolTracers_[tracerIdx] = false;
@@ -371,10 +377,8 @@ doInit(bool rst, std::size_t numGridDof,
     // all of its neighbors. (it also talks to itself since
     // degrees of freedom are sometimes quite egocentric.)
     for (unsigned dofIdx = 0; dofIdx < numGridDof; ++ dofIdx) {
-        typename NeighborSet::iterator nIt = neighbors[dofIdx].begin();
-        typename NeighborSet::iterator nEndIt = neighbors[dofIdx].end();
-        for (; nIt != nEndIt; ++nIt) {
-            tracerMatrix_->addindex(dofIdx, *nIt);
+        for (const auto& index : neighbors[dofIdx]) {
+            tracerMatrix_->addindex(dofIdx, index);
         }
     }
     tracerMatrix_->endindices();
@@ -394,7 +398,7 @@ linearSolve_(const TracerMatrix& M, TracerVector& x, TracerVector& b)
     prm.put("tol", tolerance);
     prm.put("verbosity", verbosity);
     prm.put("solver", std::string("bicgstab"));
-    prm.put("preconditioner.type", std::string("ParOverILU0"));
+    prm.put("preconditioner.type", std::string("paroverilu0"));
 
 #if HAVE_MPI
     if(gridView_.grid().comm().size() > 1)
@@ -439,20 +443,20 @@ template<class Grid, class GridView, class DofMapper, class Stencil, class Fluid
 bool GenericTracerModel<Grid,GridView,DofMapper,Stencil,FluidSystem,Scalar>::
 linearSolveBatchwise_(const TracerMatrix& M, std::vector<TracerVector>& x, std::vector<TracerVector>& b)
 {
-    Scalar tolerance = 1e-2;
-    int maxIter = 100;
-
-    int verbosity = 0;
-    PropertyTree prm;
-    prm.put("maxiter", maxIter);
-    prm.put("tol", tolerance);
-    prm.put("verbosity", verbosity);
-    prm.put("solver", std::string("bicgstab"));
-    prm.put("preconditioner.type", std::string("ParOverILU0"));
+    OPM_TIMEBLOCK(tracerSolve);
+    const Scalar tolerance = 1e-2;
+    const int maxIter = 100;
+    const int verbosity = 0;
 
 #if HAVE_MPI
-    if(gridView_.grid().comm().size() > 1)
+    if (gridView_.grid().comm().size() > 1)
     {
+        PropertyTree prm;
+        prm.put("maxiter", maxIter);
+        prm.put("tol", tolerance);
+        prm.put("verbosity", verbosity);
+        prm.put("solver", std::string("bicgstab"));
+        prm.put("preconditioner.type", std::string("paroverilu0"));
         auto [tracerOperator, solver] =
             createParallelFlexibleSolver<TracerVector>(gridView_.grid(), M, prm);
         (void) tracerOperator;
@@ -466,12 +470,18 @@ linearSolveBatchwise_(const TracerMatrix& M, std::vector<TracerVector>& x, std::
         return converged;
     }
     else
-    {
 #endif
+    {
         using TracerSolver = Dune::BiCGSTABSolver<TracerVector>;
         using TracerOperator = Dune::MatrixAdapter<TracerMatrix,TracerVector,TracerVector>;
         using TracerScalarProduct = Dune::SeqScalarProduct<TracerVector>;
-        using TracerPreconditioner = Dune::SeqILU< TracerMatrix,TracerVector,TracerVector>;
+        using TracerPreconditioner = Dune::SeqILU<TracerMatrix,TracerVector,TracerVector>;
+
+        if (std::all_of(b.begin(), b.end(),
+            [](const auto& v) { return v.infinity_norm() == 0.0; }))
+        {
+            return true;
+        }
 
         TracerOperator tracerOperator(M);
         TracerScalarProduct tracerScalarProduct;
@@ -491,9 +501,7 @@ linearSolveBatchwise_(const TracerMatrix& M, std::vector<TracerVector>& x, std::
 
         // return the result of the solver
         return converged;
-#if HAVE_MPI
     }
-#endif
 }
 
 } // namespace Opm
