@@ -65,6 +65,12 @@ public:
     //! \brief Write fluid-in-place reservoir reports to output.
     void fipResv(const Inplace& inplace, const std::string& name) const;
 
+    //! \brief Write fluid-in-place reservoir reports to csv file.
+    void fip_csv(std::ostringstream& ss,const Inplace& inplace, const std::string& name) const;
+
+    //! \brief Write fluid-in-place reservoir reports to csv file.
+    void csv_header(std::ostringstream& ss) const;
+
     //! \brief Write injection report to output.
     void injection(const std::size_t reportStepNum,
                    const std::map<std::pair<std::string,int>, double>& block_pressures) const;
@@ -93,7 +99,7 @@ public:
 private:
     struct ConnData
     {
-        ConnData(const Connection& conn);
+        explicit ConnData(const Connection& conn);
 
         int I, J, K;
         std::vector<Scalar> data;
