@@ -41,15 +41,19 @@
 
 namespace Opm {
 
-SimulatorSerializer::SimulatorSerializer(SerializableSim& simulator,
+SimulatorSerializer::SimulatorSerializer([[maybe_unused]] SerializableSim& simulator,
                                          Parallel::Communication& comm,
                                          const IOConfig& ioconfig,
                                          const std::string& saveSpec,
                                          int loadStep,
                                          const std::string& saveFile,
                                          const std::string& loadFile)
+#if HAVE_HDF5
     : simulator_(simulator)
     , comm_(comm)
+#else
+    : comm_(comm)
+#endif // HAVE_HDF5
     , loadStep_(loadStep)
     , saveFile_(saveFile)
     , loadFile_(loadFile)
@@ -219,8 +223,8 @@ void SimulatorSerializer::checkSerializedCmdLine(const std::string& current,
                                   curr_strings.begin(), curr_strings.end(),
                                   std::back_inserter(difference));
 
-    std::vector<std::string> only_stored, only_curr;
     if (!difference.empty()) {
+        std::vector<std::string> only_stored, only_curr;
         for (std::size_t i = 0; i < difference.size(); ) {
             auto stored_it = std::find(stored_strings.begin(),
                                        stored_strings.end(), difference[i]);
