@@ -47,11 +47,12 @@ struct VtkWriteTracerConcentration { static constexpr bool value = false; };
 } // namespace Opm::Parameters
 
 namespace Opm {
+
     /*!
- * \ingroup Vtk
- *
- * \brief VTK output module for the tracer model's parameters.
- */
+     * \ingroup Vtk
+     *
+     * \brief VTK output module for the tracer model's parameters.
+     */
     template <class TypeTag>
     class VtkTracerModule : public BaseOutputModule<TypeTag>
     {
@@ -65,21 +66,21 @@ namespace Opm {
         using GridView = GetPropType<TypeTag, Properties::GridView>;
         using FluidSystem = GetPropType<TypeTag, Properties::FluidSystem>;
 
-        static constexpr int vtkFormat = getPropValue<TypeTag, Properties::VtkOutputFormat>();
+        static constexpr auto vtkFormat = getPropValue<TypeTag, Properties::VtkOutputFormat>();
         using VtkMultiWriter = ::Opm::VtkMultiWriter<GridView, vtkFormat>;
 
-
+        using BufferType = typename ParentType::BufferType;
         using ScalarBuffer = typename ParentType::ScalarBuffer;
 
     public:
         explicit VtkTracerModule(const Simulator& simulator)
             : ParentType(simulator)
-        { }
+        {}
 
         /*!
-     * \brief Register all run-time parameters for the tracer VTK output
-     * module.
-     */
+         * \brief Register all run-time parameters for the tracer VTK output
+         * module.
+         */
         static void registerParameters()
         {
             Parameters::Register<Parameters::VtkWriteTracerConcentration>
@@ -92,19 +93,19 @@ namespace Opm {
      */
         void allocBuffers() override
         {
-            if (eclTracerConcentrationOutput_()){
+            if (eclTracerConcentrationOutput_()) {
                 const auto& tracerModel = this->simulator_.problem().tracerModel();
                 eclFreeTracerConcentration_.resize(tracerModel.numTracers());
                 eclSolTracerConcentration_.resize(tracerModel.numTracers());
                 const auto& enableSolTracers = tracerModel.enableSolTracers();
 
                 for (std::size_t tracerIdx = 0; tracerIdx < eclFreeTracerConcentration_.size(); ++tracerIdx) {
-                    this->resizeScalarBuffer_(eclFreeTracerConcentration_[tracerIdx]);
-                    if (enableSolTracers[tracerIdx])
-                        this->resizeScalarBuffer_(eclSolTracerConcentration_[tracerIdx]);
+                    this->resizeScalarBuffer_(eclFreeTracerConcentration_[tracerIdx], BufferType::Dof);
+                    if (enableSolTracers[tracerIdx]) {
+                        this->resizeScalarBuffer_(eclSolTracerConcentration_[tracerIdx], BufferType::Dof);
+                    }
                 }
             }
-
         }
 
         /*!
@@ -124,14 +125,16 @@ namespace Opm {
                 for (std::size_t tracerIdx  = 0; tracerIdx < eclFreeTracerConcentration_.size(); ++tracerIdx) {
                     // free tracer
                     for (unsigned dofIdx = 0; dofIdx < elemCtx.numPrimaryDof(/*timeIdx=*/0); ++dofIdx) {
-                        unsigned globalDofIdx = elemCtx.globalSpaceIndex(dofIdx, /*timeIdx=*/0);
-                        eclFreeTracerConcentration_[tracerIdx][globalDofIdx] = tracerModel.freeTracerConcentration(tracerIdx, globalDofIdx);
+                        const unsigned globalDofIdx = elemCtx.globalSpaceIndex(dofIdx, /*timeIdx=*/0);
+                        eclFreeTracerConcentration_[tracerIdx][globalDofIdx] =
+                            tracerModel.freeTracerConcentration(tracerIdx, globalDofIdx);
                     }
                     // solution tracer (only if it exist)
                     if (enableSolTracers[tracerIdx]) {
                         for (unsigned dofIdx = 0; dofIdx < elemCtx.numPrimaryDof(/*timeIdx=*/0); ++dofIdx) {
-                            unsigned globalDofIdx = elemCtx.globalSpaceIndex(dofIdx, /*timeIdx=*/0);
-                            eclSolTracerConcentration_[tracerIdx][globalDofIdx] = tracerModel.solTracerConcentration(tracerIdx, globalDofIdx);
+                            const unsigned globalDofIdx = elemCtx.globalSpaceIndex(dofIdx, /*timeIdx=*/0);
+                            eclSolTracerConcentration_[tracerIdx][globalDofIdx] =
+                                tracerModel.solTracerConcentration(tracerIdx, globalDofIdx);
                         }
                     }
                 }
@@ -143,8 +146,7 @@ namespace Opm {
      */
         void commitBuffers(BaseOutputWriter& baseWriter) override
         {
-            VtkMultiWriter *vtkWriter = dynamic_cast<VtkMultiWriter*>(&baseWriter);
-            if (!vtkWriter)
+            if (!dynamic_cast<VtkMultiWriter*>(&baseWriter))
                 return;
 
             if (eclTracerConcentrationOutput_()){
@@ -153,10 +155,12 @@ namespace Opm {
 
                 for (std::size_t tracerIdx = 0; tracerIdx < eclFreeTracerConcentration_.size(); ++tracerIdx) {
                     const std::string tmp = "freeTracerConcentration_" + tracerModel.name(tracerIdx);
-                    this->commitScalarBuffer_(baseWriter, tmp.c_str(), eclFreeTracerConcentration_[tracerIdx]);
+                    this->commitScalarBuffer_(baseWriter, tmp.c_str(),
+                                              eclFreeTracerConcentration_[tracerIdx], BufferType::Dof);
                     if (enableSolTracers[tracerIdx]) {
                         const std::string tmp2 = "solTracerConcentration_" + tracerModel.name(tracerIdx);
-                        this->commitScalarBuffer_(baseWriter, tmp2.c_str(), eclSolTracerConcentration_[tracerIdx]);
+                        this->commitScalarBuffer_(baseWriter, tmp2.c_str(),
+                                                  eclSolTracerConcentration_[tracerIdx], BufferType::Dof);
                     }
                 }
             }
