@@ -16,6 +16,14 @@
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
+#ifndef OPM_PY_MATERIAL_STATE_IMPL_HEADER_INCLUDED
+#define OPM_PY_MATERIAL_STATE_IMPL_HEADER_INCLUDED
+
+// Improve IDE experience
+#ifndef OPM_PY_MATERIAL_STATE_HEADER_INCLUDED
+#include <config.h>
+#include <opm/simulators/flow/python/PyMaterialState.hpp>
+#endif
 
 #include <fmt/format.h>
 
@@ -26,8 +34,8 @@ std::vector<double>
 PyMaterialState<TypeTag>::
 getCellVolumes()
 {
-    Model &model = this->simulator_->model();
-    auto size = model.numGridDof();
+    Model& model = this->simulator_->model();
+    const auto size = model.numGridDof();
     std::vector<double> array(size);
     for (unsigned dof_idx = 0; dof_idx < size; ++dof_idx) {
         array[dof_idx] = model.dofTotalVolume(dof_idx);
@@ -40,9 +48,9 @@ std::vector<double>
 PyMaterialState<TypeTag>::
 getPorosity()
 {
-    Problem &problem = this->simulator_->problem();
-    Model &model = this->simulator_->model();
-    auto size = model.numGridDof();
+    Problem& problem = this->simulator_->problem();
+    Model& model = this->simulator_->model();
+    const auto size = model.numGridDof();
     std::vector<double> array(size);
     for (unsigned dof_idx = 0; dof_idx < size; ++dof_idx) {
         array[dof_idx] = problem.referencePorosity(dof_idx, /*timeIdx*/0);
@@ -57,15 +65,19 @@ setPorosity(const double* poro, std::size_t size)
 {
     Problem& problem = this->simulator_->problem();
     Model& model = this->simulator_->model();
-    auto model_size = model.numGridDof();
+    const auto model_size = model.numGridDof();
     if (model_size != size) {
         const std::string msg = fmt::format(
             "Cannot set porosity. Expected array of size: {}, got array of size: ",
-            model_size, size);
+            model_size, size
+        );
         throw std::runtime_error(msg);
     }
     for (unsigned dof_idx = 0; dof_idx < size; ++dof_idx) {
         problem.setPorosity(poro[dof_idx], dof_idx);
     }
 }
-} //namespace Opm::Pybind
+
+} // namespace Opm::Pybind
+
+#endif // OPM_PY_MATERIAL_STATE_IMPL_HEADER_INCLUDED
