@@ -28,6 +28,8 @@
 
 #include <opm/simulators/linalg/linalgparameters.hh>
 #include <opm/simulators/linalg/linalgproperties.hh>
+#include <opm/simulators/linalg/LinearSolverAcceleratorType.hpp>
+#include <string>
 
 namespace Opm {
 
@@ -37,6 +39,9 @@ class ISTLSolverGpuBridge;
 template <class TypeTag>
 class ISTLSolver;
 
+template<class TypeTag>
+class ISTLSolverRuntimeOptionProxy;
+
 }
 
 namespace Opm::Properties {
@@ -51,11 +56,7 @@ struct FlowIstlSolverParams {};
 template<class TypeTag>
 struct LinearSolverBackend<TypeTag, TTag::FlowIstlSolverParams>
 {
-#if COMPILE_GPU_BRIDGE
-    using type = ISTLSolverGpuBridge<TypeTag>;
-#else
-    using type = ISTLSolver<TypeTag>;
-#endif
+    using type = ISTLSolverRuntimeOptionProxy<TypeTag>;
 };
 
 }
@@ -70,7 +71,7 @@ struct LinearSolverMaxIter { static constexpr int value = 200; };
 struct NlddLocalLinearSolverMaxIter { static constexpr int value = 200; };
 struct LinearSolverRestart { static constexpr int value = 40; };
 struct IluFillinLevel { static constexpr int value = 0; };
-struct MiluVariant { static constexpr auto value = "ILU"; };
+struct MiluVariant { static constexpr auto value = "ilu"; };
 struct IluRedblack { static constexpr bool value = false; };
 struct IluReorderSpheres { static constexpr bool value = false; };
 struct UseGmres { static constexpr bool value = false; };
@@ -85,7 +86,9 @@ struct AcceleratorMode { static constexpr auto value = "none"; };
 struct GpuDeviceId { static constexpr int value = 0; };
 struct OpenclPlatformId { static constexpr int value = 0; };
 struct OpenclIluParallel { static constexpr bool value = true; }; // note: false should only be used in debug
-
+struct GpuAwareMpi { static constexpr bool value = false; };
+struct VerifyGpuAwareMpi { static constexpr bool value = false; };
+struct CprWeightsThreadParallel { static constexpr bool value = false; };
 } // namespace Opm::Parameters
 
 namespace Opm {
@@ -115,6 +118,10 @@ struct FlowLinearSolverParameters
     int gpu_device_id_;
     int opencl_platform_id_;
     bool opencl_ilu_parallel_;
+    Parameters::LinearSolverAcceleratorType linear_solver_accelerator_;
+    bool gpu_aware_mpi_;
+    bool verify_gpu_aware_mpi_;
+    bool cpr_weights_thread_parallel_;
 
     FlowLinearSolverParameters() { reset(); }
 
