@@ -30,6 +30,8 @@
 #include <opm/common/Exceptions.hpp>
 #include <opm/common/TimingMacros.hpp>
 
+#include <opm/grid/utility/ElementChunks.hpp>
+
 #include <opm/models/discretization/common/fvbaseproperties.hh>
 #include <opm/models/common/multiphasebaseproperties.hh>
 #include <opm/models/utils/parametersystem.hpp>
@@ -47,6 +49,8 @@
 #include <opm/simulators/linalg/findOverlapRowsAndColumns.hpp>
 #include <opm/simulators/linalg/getQuasiImpesWeights.hpp>
 #include <opm/simulators/linalg/setupPropertyTree.hpp>
+#include <opm/simulators/linalg/AbstractISTLSolver.hpp>
+#include <opm/simulators/linalg/printlinearsolverparameter.hpp>
 
 #include <any>
 #include <cstddef>
@@ -105,7 +109,7 @@ struct FlexibleSolverInfo
                 std::size_t pressureIndex,
                 std::function<Vector()> weightCalculator,
                 const bool forceSerial,
-                Comm& comm);
+                Comm* comm);
 
     std::unique_ptr<AbstractSolverType> solver_;
     std::unique_ptr<AbstractOperatorType> op_;
@@ -141,7 +145,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
     /// as a block-structured matrix (one block for all cell variables) for a fixed
     /// number of cell variables np .
     template <class TypeTag>
-    class ISTLSolver
+    class ISTLSolver : public AbstractISTLSolver<TypeTag>
     {
     protected:
         using GridView = GetPropType<TypeTag, Properties::GridView>;
@@ -159,6 +163,8 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         using AbstractPreconditionerType = Dune::PreconditionerWithUpdate<Vector, Vector>;
         using WellModelOperator = WellModelAsLinearOperator<WellModel, Vector, Vector>;
         using ElementMapper = GetPropType<TypeTag, Properties::ElementMapper>;
+        using ElementChunksType = ElementChunks<GridView, Dune::Partitions::All>;
+
         constexpr static std::size_t pressureIndex = GetPropType<TypeTag, Properties::Indices>::pressureSwitchIdx;
 
         enum { enablePolymerMolarWeight = getPropValue<TypeTag, Properties::EnablePolymerMW>() };
@@ -190,7 +196,6 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
                    bool forceSerial = false)
             : simulator_(simulator),
               iterations_( 0 ),
-              converged_(false),
               matrix_(nullptr),
               parameters_{parameters},
               forceSerial_(forceSerial)
@@ -204,7 +209,6 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             : simulator_(simulator),
               iterations_( 0 ),
               solveCount_(0),
-              converged_(false),
               matrix_(nullptr)
         {
             parameters_.resize(1);
@@ -303,22 +307,17 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
 #endif
 
             // Print parameters to PRT/DBG logs.
-            if (on_io_rank && parameters_[activeSolverNum_].linear_solver_print_json_definition_) {
-                std::ostringstream os;
-                os << "\nProperty tree for linear solvers:\n";
-                for (std::size_t i = 0; i<prm_.size(); i++) {
-                    prm_[i].write_json(os, true);
-                }
-                OpmLog::note(os.str());
-            }
+            detail::printLinearSolverParameters(parameters_, activeSolverNum_, prm_,  simulator_.gridView().comm());
+
+            element_chunks_ = std::make_unique<ElementChunksType>(simulator_.vanguard().gridView(), Dune::Partitions::all, ThreadManager::maxThreads());
         }
 
         // nothing to clean here
-        void eraseMatrix()
+        void eraseMatrix() override
         {
         }
 
-        void setActiveSolver(const int num)
+        void setActiveSolver(const int num) override
         {
             if (num > static_cast<int>(prm_.size()) - 1) {
                 OPM_THROW(std::logic_error, "Solver number " + std::to_string(num) + " not available.");
@@ -330,7 +329,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             }
         }
 
-        int numAvailableSolvers()
+        int numAvailableSolvers() const override
         {
             return flexibleSolver_.size();
         }
@@ -358,17 +357,20 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             rhs_ = &b;
 
             // TODO: check all solvers, not just one.
-            if (isParallel() && prm_[activeSolverNum_].template get<std::string>("preconditioner.type") != "ParOverILU0") {
+            // We use lower case as the internal canonical representation of solver names
+            std::string type = prm_[activeSolverNum_].template get<std::string>("preconditioner.type", "paroverilu0");
+            std::transform(type.begin(), type.end(), type.begin(), ::tolower);
+            if (isParallel() && type != "paroverilu0") {
                 detail::makeOverlapRowsInvalid(getMatrix(), overlapRows_);
             }
         }
 
-        void prepare(const SparseMatrixAdapter& M, Vector& b)
+        void prepare(const SparseMatrixAdapter& M, Vector& b) override
         {
             prepare(M.istlMatrix(), b);
         }
 
-        void prepare(const Matrix& M, Vector& b)
+        void prepare(const Matrix& M, Vector& b) override
         {
             OPM_TIMEBLOCK(istlSolverPrepare);
             try {
@@ -379,22 +381,22 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         }
 
 
-        void setResidual(Vector& /* b */)
+        void setResidual(Vector& /* b */) override
         {
             // rhs_ = &b; // Must be handled in prepare() instead.
         }
 
-        void getResidual(Vector& b) const
+        void getResidual(Vector& b) const override
         {
             b = *rhs_;
         }
 
-        void setMatrix(const SparseMatrixAdapter& /* M */)
+        void setMatrix(const SparseMatrixAdapter& /* M */) override
         {
             // matrix_ = &M.istlMatrix(); // Must be handled in prepare() instead.
         }
 
-        int getSolveCount() const {
+        int getSolveCount() const override {
             return solveCount_;
         }
 
@@ -402,7 +404,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             solveCount_ = 0;
         }
 
-        bool solve(Vector& x)
+        bool solve(Vector& x) override
         {
             OPM_TIMEBLOCK(istlSolverSolve);
             ++solveCount_;
@@ -424,10 +426,10 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
                 flexibleSolver_[activeSolverNum_].solver_->apply(x, *rhs_, result);
             }
 
-            // Check convergence, iterations etc.
-            checkConvergence(result);
+            iterations_ = result.iterations;
 
-            return converged_;
+            // Check convergence, iterations etc.
+            return checkConvergence(result);
         }
 
 
@@ -438,12 +440,12 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         /// \return               the solution x
 
         /// \copydoc NewtonIterationBlackoilInterface::iterations
-        int iterations () const { return iterations_; }
+        int iterations () const override { return iterations_; }
 
         /// \copydoc NewtonIterationBlackoilInterface::parallelInformation
         const std::any& parallelInformation() const { return parallelInformation_; }
 
-        const CommunicationType* comm() const { return comm_.get(); }
+        const CommunicationType* comm() const override { return comm_.get(); }
 
         void setDomainIndex(const int index)
         {
@@ -460,27 +462,10 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         using Comm = Dune::OwnerOverlapCopyCommunication<int, int>;
 #endif
 
-        void checkConvergence( const Dune::InverseOperatorResult& result ) const
+        bool checkConvergence(const Dune::InverseOperatorResult& result) const
         {
-            // store number of iterations
-            iterations_ = result.iterations;
-            converged_ = result.converged;
-            if(!converged_){
-                if(result.reduction < parameters_[activeSolverNum_].relaxed_linear_solver_reduction_){
-                    std::stringstream ss;
-                    ss<< "Full linear solver tolerance not achieved. The reduction is:" << result.reduction
-                      << " after " << result.iterations << " iterations ";
-                    OpmLog::warning(ss.str());
-                    converged_ = true;
-                }
-            }
-            // Check for failure of linear solver.
-            if (!parameters_[activeSolverNum_].ignoreConvergenceFailure_ && !converged_) {
-                const std::string msg("Convergence failure for linear solver.");
-                OPM_THROW_NOLOG(NumericalProblem, msg);
-            }
+            return AbstractISTLSolver<TypeTag>::checkConvergence(result, parameters_[activeSolverNum_]);
         }
-    protected:
 
         bool isParallel() const {
 #if HAVE_MPI
@@ -513,7 +498,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
                                                          pressureIndex,
                                                          weightCalculator,
                                                          forceSerial_,
-                                                         *comm_);
+                                                         comm_.get());
             }
             else
             {
@@ -590,41 +575,51 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
             using namespace std::string_literals;
 
             auto preconditionerType = prm.get("preconditioner.type"s, "cpr"s);
+            // We use lower case as the internal canonical representation of solver names
+            std::transform(preconditionerType.begin(), preconditionerType.end(), preconditionerType.begin(), ::tolower);
             if (preconditionerType == "cpr" || preconditionerType == "cprt"
                 || preconditionerType == "cprw" || preconditionerType == "cprwt") {
                 const bool transpose = preconditionerType == "cprt" || preconditionerType == "cprwt";
+                const bool enableThreadParallel = this->parameters_[0].cpr_weights_thread_parallel_;
                 const auto weightsType = prm.get("preconditioner.weight_type"s, "quasiimpes"s);
                 if (weightsType == "quasiimpes") {
                     // weights will be created as default in the solver
                     // assignment p = pressureIndex prevent compiler warning about
                     // capturing variable with non-automatic storage duration
-                    weightsCalculator = [matrix, transpose, pressIndex]() {
+                    weightsCalculator = [matrix, transpose, pressIndex, enableThreadParallel]() {
                         return Amg::getQuasiImpesWeights<Matrix, Vector>(matrix,
                                                                          pressIndex,
-                                                                         transpose);
+                                                                         transpose,
+                                                                         enableThreadParallel);
                     };
                 } else if ( weightsType == "trueimpes" ) {
                     weightsCalculator =
-                        [this, pressIndex]
+                        [this, pressIndex, enableThreadParallel]
                         {
                             Vector weights(rhs_->size());
                             ElementContext elemCtx(simulator_);
-                            Amg::getTrueImpesWeights(pressIndex, weights,
-                                                             simulator_.vanguard().gridView(),
-                                                             elemCtx, simulator_.model(),
-                                                             ThreadManager::threadId());
+                            Amg::getTrueImpesWeights(pressIndex,
+                                                     weights,
+                                                     elemCtx,
+                                                     simulator_.model(),
+                                                     *element_chunks_,
+                                                     enableThreadParallel
+                            );
                             return weights;
                         };
                 } else if  (weightsType == "trueimpesanalytic" ) {
                     weightsCalculator =
-                        [this, pressIndex]
+                        [this, pressIndex, enableThreadParallel]
                         {
                             Vector weights(rhs_->size());
                             ElementContext elemCtx(simulator_);
-                            Amg::getTrueImpesWeightsAnalytic(pressIndex, weights,
-                                                             simulator_.vanguard().gridView(),
-                                                             elemCtx, simulator_.model(),
-                                                             ThreadManager::threadId());
+                            Amg::getTrueImpesWeightsAnalytic(pressIndex,
+                                                             weights,
+                                                             elemCtx,
+                                                             simulator_.model(),
+                                                             *element_chunks_,
+                                                             enableThreadParallel
+                            );
                             return weights;
                         };
                 } else {
@@ -651,7 +646,6 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         const Simulator& simulator_;
         mutable int iterations_;
         mutable int solveCount_;
-        mutable bool converged_;
         std::any parallelInformation_;
 
         // non-const to be able to scale the linear system
@@ -672,6 +666,7 @@ std::unique_ptr<Matrix> blockJacobiAdjacency(const Grid& grid,
         std::vector<PropertyTree> prm_;
 
         std::shared_ptr< CommunicationType > comm_;
+        std::unique_ptr<ElementChunksType> element_chunks_;
     }; // end ISTLSolver
 
 } // namespace Opm
