@@ -4,6 +4,11 @@
 
 namespace Opm {
 
-INSTANCE_PF(6)
+INSTANTIATE_PF(double,6)
+
+#if FLOW_INSTANTIATE_FLOAT
+INSTANTIATE_PF(float,6)
+#endif
+
 
 }
