@@ -19,53 +19,63 @@
 #ifndef OPM_GPUDILU_HPP
 #define OPM_GPUDILU_HPP
 
+#include <map>
 #include <memory>
 #include <opm/grid/utility/SparseTable.hpp>
 #include <opm/simulators/linalg/PreconditionerWithUpdate.hpp>
-#include <opm/simulators/linalg/gpuistl/GpuSparseMatrix.hpp>
+#include <opm/simulators/linalg/gpuistl/GpuBuffer.hpp>
+#include <opm/simulators/linalg/gpuistl/GpuSparseMatrixWrapper.hpp>
+#include <opm/simulators/linalg/gpuistl/GpuVector.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/kernel_enums.hpp>
+#include <opm/simulators/linalg/gpuistl/detail/preconditionerKernels/ILU_variants_helper_kernels.hpp>
 #include <opm/simulators/linalg/gpuistl/gpu_resources.hpp>
-#include <vector>
-#include <map>
 #include <utility>
+#include <vector>
 
 
 namespace Opm::gpuistl
 {
 //! \brief DILU preconditioner on the GPU.
 //!
-//! \tparam M The matrix type to operate on
+//! \tparam CPUMatrixT Type of the matrix on the CPU
 //! \tparam X Type of the update
 //! \tparam Y Type of the defect
 //! \tparam l Ignored. Just there to have the same number of template arguments
 //!    as other preconditioners.
 //!
+//! \todo Remove the reliance on CPUMatrix. We should be able to use the GPU matrix type
+//!    directly.
+//!
 //! \note We assume X and Y are both GpuVector<real_type>, but we leave them as template
 //! arguments in case of future additions.
-template <class M, class X, class Y, int l = 1>
+template <class CPUMatrixT, class X, class Y, int l = 1>
 class GpuDILU : public Dune::PreconditionerWithUpdate<X, Y>
 {
 public:
-    //! \brief The matrix type the preconditioner is for.
-    using matrix_type = typename std::remove_const<M>::type;
     //! \brief The domain type of the preconditioner.
     using domain_type = X;
     //! \brief The range type of the preconditioner.
     using range_type = Y;
     //! \brief The field type of the preconditioner.
     using field_type = typename X::field_type;
-    //! \brief The GPU matrix type
-    using CuMat = GpuSparseMatrix<field_type>;
-    using FloatMat = GpuSparseMatrix<float>;
+
+    using GPUMatrix = GpuSparseMatrixWrapper<field_type>;
+    using FloatMat = GpuSparseMatrixWrapper<float>;
     using FloatVec = GpuVector<float>;
 
+    //! \brief The matrix type the preconditioner is for.
+    using matrix_type = GPUMatrix;
+
+
     //! \brief Constructor.
     //!
     //!  Constructor gets all parameters to operate the prec.
-    //! \param A The matrix to operate on.
-    //! \param w The relaxation factor.
-    //!
-    explicit GpuDILU(const M& A, bool splitMatrix, bool tuneKernels, int mixedPrecisionScheme);
+    explicit GpuDILU(const GPUMatrix& gpuMatrix,
+                     const CPUMatrixT& cpuMatrix,
+                     bool splitMatrix,
+                     bool tuneKernels,
+                     int mixedPrecisionScheme,
+                     bool reorder);
 
     //! \brief Prepare the preconditioner.
     //! \note Does nothing at the time being.
@@ -106,7 +116,8 @@ public:
         return false;
     }
 
-    virtual bool hasPerfectUpdate() const override {
+    virtual bool hasPerfectUpdate() const override
+    {
         return true;
     }
 
@@ -116,10 +127,8 @@ private:
     void apply(X& v, const Y& d, int lowerSolveThreadBlockSize, int upperSolveThreadBlockSize);
     //! \brief Updates the matrix data.
     void update(int moveThreadBlockSize, int factorizationThreadBlockSize);
-    //! \brief Reference to the underlying matrix
-    const M& m_cpuMatrix;
     //! \brief size_t describing the dimensions of the square block elements
-    static constexpr const size_t blocksize_ = matrix_type::block_type::cols;
+    static constexpr const size_t blocksize_ = CPUMatrixT::block_type::cols;
     //! \brief SparseTable storing each row by level
     Opm::SparseTable<size_t> m_levelSets;
     //! \brief converts from index in reordered structure to index natural ordered structure
@@ -127,12 +136,12 @@ private:
     //! \brief converts from index in natural ordered structure to index reordered strucutre
     std::vector<int> m_naturalToReordered;
     //! \brief The A matrix stored on the gpu, and its reordred version
-    CuMat m_gpuMatrix;
+    const GPUMatrix& m_gpuMatrix;
     //! \brief Stores the matrix in its entirety reordered. Optional in case splitting is used
-    std::unique_ptr<CuMat> m_gpuMatrixReordered;
+    std::unique_ptr<GPUMatrix> m_gpuMatrixReordered;
     //! \brief If matrix splitting is enabled, then we store the lower and upper part separately
-    std::unique_ptr<CuMat> m_gpuMatrixReorderedLower;
-    std::unique_ptr<CuMat> m_gpuMatrixReorderedUpper;
+    std::unique_ptr<GPUMatrix> m_gpuMatrixReorderedLower;
+    std::unique_ptr<GPUMatrix> m_gpuMatrixReorderedUpper;
     //! \brief If matrix splitting is enabled, we also store the diagonal separately
     std::unique_ptr<GpuVector<field_type>> m_gpuMatrixReorderedDiag;
     //! \brief If mixed precision is enabled, store a float matrix
@@ -144,6 +153,8 @@ private:
     GpuVector<int> m_gpuNaturalToReorder;
     //! row conversion from reordered to natural matrix indices stored on the GPU
     GpuVector<int> m_gpuReorderToNatural;
+    //! \brief Potentially stores the level sets on the GPU
+    GpuBuffer<size_t> m_gpuLevelSets; // TODO: make this an optional to not spend time on the copy if not needed
     //! \brief Stores the inverted diagonal that we use in DILU
     GpuVector<field_type> m_gpuDInv;
     //! \brief Bool storing whether or not we should store matrices in a split format
@@ -158,16 +169,21 @@ private:
     int m_lowerSolveThreadBlockSize = -1;
     int m_moveThreadBlockSize = -1;
     int m_DILUFactorizationThreadBlockSize = -1;
+    //! \brief The reorder parameter can be set to false to shorten the update time, but slows the apply time
+    bool m_reorder;
+
+    //! \brief The indices of the blocks that are the diagonal entries
+    GpuBuffer<size_t> m_diagIdxs;
 
     // Graphs for Apply
     std::map<std::pair<field_type*, const field_type*>, GPUGraph> m_apply_graphs;
     std::map<std::pair<field_type*, const field_type*>, GPUGraphExec> m_executableGraphs;
 
     // Stream for the DILU operations on the GPU
-    GPUStream m_stream{};
+    GPUStream m_stream {};
     // Events for synchronization with main stream
-    GPUEvent m_before{};
-    GPUEvent m_after{};
+    GPUEvent m_before {};
+    GPUEvent m_after {};
 };
 } // end namespace Opm::gpuistl
 
