@@ -20,9 +20,11 @@
 #include <cuda.h>
 #include <cuda_runtime.h>
 #include <fmt/core.h>
+#include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/simulators/linalg/gpuistl/GpuVector.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/cublas_safe_call.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/cublas_wrapper.hpp>
+#include <opm/simulators/linalg/gpuistl/detail/gpu_constants.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/gpu_safe_call.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/vector_operations.hpp>
 
@@ -65,13 +67,15 @@ template <class T>
 GpuVector<T>&
 GpuVector<T>::operator=(const GpuVector<T>& other)
 {
-    assertHasElements();
-    assertSameSize(other);
-
-    OPM_GPU_SAFE_CALL(cudaMemcpy(m_dataOnDevice,
-                                  other.m_dataOnDevice,
-                                  detail::to_size_t(m_numberOfElements) * sizeof(T),
-                                  cudaMemcpyDeviceToDevice));
+    // Only copy data if both vectors have elements and same size
+    if (m_numberOfElements > 0 && other.m_numberOfElements > 0) {
+        assertSameSize(other);
+        OPM_GPU_SAFE_CALL(cudaMemcpy(m_dataOnDevice,
+                                      other.m_dataOnDevice,
+                                      detail::to_size_t(m_numberOfElements) * sizeof(T),
+                                      cudaMemcpyDeviceToDevice));
+    }
+    // If both are zero-sized, assignment is trivial (do nothing)
     return *this;
 }
 
@@ -79,12 +83,15 @@ template <class T>
 GpuVector<T>::GpuVector(const GpuVector<T>& other)
     : GpuVector(other.m_numberOfElements)
 {
-    assertHasElements();
-    assertSameSize(other);
-    OPM_GPU_SAFE_CALL(cudaMemcpy(m_dataOnDevice,
-                                  other.m_dataOnDevice,
-                                  detail::to_size_t(m_numberOfElements) * sizeof(T),
-                                  cudaMemcpyDeviceToDevice));
+    // Only copy data if both vectors have elements and same size
+    if (m_numberOfElements > 0) {
+        assertSameSize(other);
+        OPM_GPU_SAFE_CALL(cudaMemcpy(m_dataOnDevice,
+                                      other.m_dataOnDevice,
+                                      detail::to_size_t(m_numberOfElements) * sizeof(T),
+                                      cudaMemcpyDeviceToDevice));
+    }
+    // If other is zero-sized, assignment is trivial (do nothing)
 }
 
 template <class T>
@@ -112,6 +119,47 @@ GpuVector<T>::dim() const
     return detail::to_size_t(m_numberOfElements);
 }
 
+template <typename T>
+void
+GpuVector<T>::resize(size_t new_size)
+{
+    const int new_elements = detail::to_int(new_size);
+
+    if (new_elements == m_numberOfElements) {
+        return;
+    }
+
+    if (new_elements == 0) {
+        // Free existing memory and set to empty state
+        if (m_dataOnDevice != nullptr) {
+            OPM_GPU_WARN_IF_ERROR(cudaFree(m_dataOnDevice));
+            m_dataOnDevice = nullptr;
+        }
+        m_numberOfElements = 0;
+        return;
+    }
+
+    // Allocate new memory
+    T* new_data = nullptr;
+    OPM_GPU_SAFE_CALL(cudaMalloc(&new_data, sizeof(T) * new_size));
+
+    if (m_dataOnDevice != nullptr && m_numberOfElements > 0) {
+        // Copy existing data (up to the minimum of old and new size)
+        const size_t copy_elements = std::min(detail::to_size_t(m_numberOfElements), new_size);
+        if (copy_elements > 0) {
+            OPM_GPU_SAFE_CALL(cudaMemcpy(new_data, m_dataOnDevice,
+                                          sizeof(T) * copy_elements,
+                                          cudaMemcpyDeviceToDevice));
+        }
+
+        // Free old memory
+        OPM_GPU_WARN_IF_ERROR(cudaFree(m_dataOnDevice));
+    }
+
+    m_dataOnDevice = new_data;
+    m_numberOfElements = new_elements;
+}
+
 template <typename T>
 std::vector<T>
 GpuVector<T>::asStdVector() const
@@ -268,7 +316,7 @@ GpuVector<T>::copyFromHost(const T* dataPointer, size_t numberOfElements)
 
 template <class T>
 void
-GpuVector<T>::copyFromHost(const T* dataPointer, size_t numberOfElements, cudaStream_t stream)
+GpuVector<T>::copyFromHostAsync(const T* dataPointer, size_t numberOfElements, cudaStream_t stream)
 {
     if (numberOfElements > dim()) {
         OPM_THROW(std::runtime_error,
@@ -276,15 +324,26 @@ GpuVector<T>::copyFromHost(const T* dataPointer, size_t numberOfElements, cudaSt
                               dim(),
                               numberOfElements));
     }
+    // Asynchronous copy. CUDA runtime will use pinned memory if dataPointer is in a registered region.
     OPM_GPU_SAFE_CALL(cudaMemcpyAsync(data(), dataPointer, numberOfElements * sizeof(T), cudaMemcpyHostToDevice, stream));
 }
 
 template <class T>
 void
 GpuVector<T>::copyToHost(T* dataPointer, size_t numberOfElements) const
+{
+    // Synchronous version: use default stream and then synchronize.
+    copyToHostAsync(dataPointer, numberOfElements, detail::DEFAULT_STREAM);
+    OPM_GPU_SAFE_CALL(cudaStreamSynchronize(detail::DEFAULT_STREAM));
+}
+
+template <class T>
+void
+GpuVector<T>::copyToHostAsync(T* dataPointer, size_t numberOfElements, cudaStream_t stream) const
 {
     assertSameSize(detail::to_int(numberOfElements));
-    OPM_GPU_SAFE_CALL(cudaMemcpy(dataPointer, data(), numberOfElements * sizeof(T), cudaMemcpyDeviceToHost));
+    // Asynchronous copy. CUDA runtime will use pinned memory if dataPointer is in a registered region.
+    OPM_GPU_SAFE_CALL(cudaMemcpyAsync(dataPointer, data(), numberOfElements * sizeof(T), cudaMemcpyDeviceToHost, stream));
 }
 
 template <class T>
@@ -293,6 +352,14 @@ GpuVector<T>::copyFromHost(const std::vector<T>& data)
 {
     copyFromHost(data.data(), data.size());
 }
+
+template <class T>
+void
+GpuVector<T>::copyFromHostAsync(const std::vector<T>& data, cudaStream_t stream)
+{
+    copyFromHostAsync(data.data(), data.size(), stream);
+}
+
 template <class T>
 void
 GpuVector<T>::copyToHost(std::vector<T>& data) const
@@ -300,6 +367,26 @@ GpuVector<T>::copyToHost(std::vector<T>& data) const
     copyToHost(data.data(), data.size());
 }
 
+template <class T>
+void
+GpuVector<T>::copyToHostAsync(std::vector<T>& data, cudaStream_t stream) const
+{
+    copyToHostAsync(data.data(), data.size(), stream);
+}
+
+template <class T>
+void
+GpuVector<T>::copyFromDeviceToDevice(const GpuVector<T>& data) const
+{
+    assertHasElements();
+    assertSameSize(data);
+
+    OPM_GPU_SAFE_CALL(cudaMemcpy(m_dataOnDevice,
+                                data.m_dataOnDevice,
+                                detail::to_size_t(m_numberOfElements) * sizeof(T),
+                                cudaMemcpyDeviceToDevice));
+}
+
 template <typename T>
 void
 GpuVector<T>::prepareSendBuf(GpuVector<T>& buffer, const GpuVector<int>& indexSet) const
