@@ -26,7 +26,7 @@
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/TimingMacros.hpp>
 #include <opm/simulators/linalg/GraphColoring.hpp>
-#include <opm/simulators/linalg/gpuistl/GpuSparseMatrix.hpp>
+#include <opm/simulators/linalg/gpuistl/GpuSparseMatrixWrapper.hpp>
 #include <opm/simulators/linalg/gpuistl/GpuVector.hpp>
 #include <opm/simulators/linalg/gpuistl/OpmGpuILU0.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/autotuner.hpp>
@@ -42,12 +42,11 @@ namespace Opm::gpuistl
 {
 
 template <class M, class X, class Y, int l>
-OpmGpuILU0<M, X, Y, l>::OpmGpuILU0(const M& A, bool splitMatrix, bool tuneKernels, int mixedPrecisionScheme)
-    : m_cpuMatrix(A)
-    , m_levelSets(Opm::getMatrixRowColoring(m_cpuMatrix, Opm::ColoringType::LOWER))
+OpmGpuILU0<M, X, Y, l>::OpmGpuILU0(const OpmGpuILU0<M, X, Y, l>::matrix_type& gpuMatrix, const M& cpuMatrix, bool splitMatrix, bool tuneKernels, int mixedPrecisionScheme)
+    : m_levelSets(Opm::getMatrixRowColoring(cpuMatrix, Opm::ColoringType::LOWER))
     , m_reorderedToNatural(detail::createReorderedToNatural(m_levelSets))
     , m_naturalToReordered(detail::createNaturalToReordered(m_levelSets))
-    , m_gpuMatrix(GpuSparseMatrix<field_type>::fromMatrix(m_cpuMatrix, true))
+    , m_gpuMatrix(gpuMatrix)
     , m_gpuMatrixReorderedLower(nullptr)
     , m_gpuMatrixReorderedUpper(nullptr)
     , m_gpuMatrixReorderedLowerFloat(nullptr)
@@ -61,29 +60,29 @@ OpmGpuILU0<M, X, Y, l>::OpmGpuILU0(const M& A, bool splitMatrix, bool tuneKernel
 {
     // TODO: Should in some way verify that this matrix is symmetric, only do it debug mode?
     // Some sanity check
-    OPM_ERROR_IF(A.N() != m_gpuMatrix.N(),
-                 fmt::format("CuSparse matrix not same size as DUNE matrix. {} vs {}.", m_gpuMatrix.N(), A.N()));
-    OPM_ERROR_IF(A[0][0].N() != m_gpuMatrix.blockSize(),
+    OPM_ERROR_IF(cpuMatrix.N() != m_gpuMatrix.N(),
+                 fmt::format("CuSparse matrix not same size as DUNE matrix. {} vs {}.", m_gpuMatrix.N(), cpuMatrix.N()));
+    OPM_ERROR_IF(cpuMatrix[0][0].N() != m_gpuMatrix.blockSize(),
                  fmt::format("CuSparse matrix not same blocksize as DUNE matrix. {} vs {}.",
                              m_gpuMatrix.blockSize(),
-                             A[0][0].N()));
-    OPM_ERROR_IF(A.N() * A[0][0].N() != m_gpuMatrix.dim(),
+                             cpuMatrix[0][0].N()));
+    OPM_ERROR_IF(cpuMatrix.N() * cpuMatrix[0][0].N() != m_gpuMatrix.dim(),
                  fmt::format("CuSparse matrix not same dimension as DUNE matrix. {} vs {}.",
                              m_gpuMatrix.dim(),
-                             A.N() * A[0][0].N()));
-    OPM_ERROR_IF(A.nonzeroes() != m_gpuMatrix.nonzeroes(),
+                             cpuMatrix.N() * cpuMatrix[0][0].N()));
+    OPM_ERROR_IF(cpuMatrix.nonzeroes() != m_gpuMatrix.nonzeroes(),
                  fmt::format("CuSparse matrix not same number of non zeroes as DUNE matrix. {} vs {}. ",
                              m_gpuMatrix.nonzeroes(),
-                             A.nonzeroes()));
+                             cpuMatrix.nonzeroes()));
 
     if (m_splitMatrix) {
-        m_gpuMatrixReorderedDiag.emplace(GpuVector<field_type>(blocksize_ * blocksize_ * m_cpuMatrix.N()));
+        m_gpuMatrixReorderedDiag.emplace(GpuVector<field_type>(blocksize_ * blocksize_ * cpuMatrix.N()));
         std::tie(m_gpuMatrixReorderedLower, m_gpuMatrixReorderedUpper)
-            = detail::extractLowerAndUpperMatrices<M, field_type, GpuSparseMatrix<field_type>>(m_cpuMatrix,
+            = detail::extractLowerAndUpperMatrices<M, field_type, GpuSparseMatrixWrapper<field_type>>(cpuMatrix,
                                                                                               m_reorderedToNatural);
     } else {
-        m_gpuReorderedLU = detail::createReorderedMatrix<M, field_type, GpuSparseMatrix<field_type>>(
-            m_cpuMatrix, m_reorderedToNatural);
+        m_gpuReorderedLU = detail::createReorderedMatrix<M, field_type, GpuSparseMatrixWrapper<field_type>>(
+            cpuMatrix, m_reorderedToNatural);
     }
 
     if (m_mixedPrecisionScheme != MatrixStorageMPScheme::DOUBLE_DIAG_DOUBLE_OFFDIAG){
@@ -196,7 +195,7 @@ OpmGpuILU0<M, X, Y, l>::apply(X& v, const Y& d, int lowerSolveThreadBlockSize, i
         levelStartIdx += numOfRowsInLevel;
     }
 
-    levelStartIdx = m_cpuMatrix.N();
+    levelStartIdx = m_gpuMatrix.N();
     for (int level = m_levelSets.size() - 1; level >= 0; --level) {
         const int numOfRowsInLevel = m_levelSets[level].size();
         levelStartIdx -= numOfRowsInLevel;
@@ -271,7 +270,6 @@ template <class M, class X, class Y, int l>
 void
 OpmGpuILU0<M, X, Y, l>::update()
 {
-    m_gpuMatrix.updateNonzeroValues(m_cpuMatrix); // send updated matrix to the gpu
     reorderAndSplitMatrix(m_moveThreadBlockSize);
     LUFactorizeMatrix(m_ILU0FactorizationThreadBlockSize);
 }
@@ -286,7 +284,6 @@ OpmGpuILU0<M, X, Y, l>::update(int moveThreadBlockSize, int factorizationThreadB
         OPM_GPU_SAFE_CALL(cudaEventRecord(m_before.get(), 0));
         OPM_GPU_SAFE_CALL(cudaStreamWaitEvent(m_stream.get(), m_before.get(), 0));
 
-        m_gpuMatrix.updateNonzeroValues(m_cpuMatrix, true); // send updated matrix to the gpu
         reorderAndSplitMatrix(moveThreadBlockSize);
         LUFactorizeMatrix(factorizationThreadBlockSize);
 
@@ -437,7 +434,7 @@ OpmGpuILU0<M, X, Y, l>::tuneThreadBlockSizes()
 }
 
 } // namespace Opm::gpuistl
-#define INSTANTIATE_CUDILU_DUNE(realtype, blockdim)                                                                    \
+#define INSTANTIATE_GPUILU_DUNE(realtype, blockdim)                                                                    \
     template class ::Opm::gpuistl::OpmGpuILU0<Dune::BCRSMatrix<Dune::FieldMatrix<realtype, blockdim, blockdim>>,         \
                                             ::Opm::gpuistl::GpuVector<realtype>,                                         \
                                             ::Opm::gpuistl::GpuVector<realtype>>;                                        \
@@ -445,16 +442,16 @@ OpmGpuILU0<M, X, Y, l>::tuneThreadBlockSizes()
                                             ::Opm::gpuistl::GpuVector<realtype>,                                         \
                                             ::Opm::gpuistl::GpuVector<realtype>>
 
-INSTANTIATE_CUDILU_DUNE(double, 1);
-INSTANTIATE_CUDILU_DUNE(double, 2);
-INSTANTIATE_CUDILU_DUNE(double, 3);
-INSTANTIATE_CUDILU_DUNE(double, 4);
-INSTANTIATE_CUDILU_DUNE(double, 5);
-INSTANTIATE_CUDILU_DUNE(double, 6);
+INSTANTIATE_GPUILU_DUNE(double, 1);
+INSTANTIATE_GPUILU_DUNE(double, 2);
+INSTANTIATE_GPUILU_DUNE(double, 3);
+INSTANTIATE_GPUILU_DUNE(double, 4);
+INSTANTIATE_GPUILU_DUNE(double, 5);
+INSTANTIATE_GPUILU_DUNE(double, 6);
 
-INSTANTIATE_CUDILU_DUNE(float, 1);
-INSTANTIATE_CUDILU_DUNE(float, 2);
-INSTANTIATE_CUDILU_DUNE(float, 3);
-INSTANTIATE_CUDILU_DUNE(float, 4);
-INSTANTIATE_CUDILU_DUNE(float, 5);
-INSTANTIATE_CUDILU_DUNE(float, 6);
+INSTANTIATE_GPUILU_DUNE(float, 1);
+INSTANTIATE_GPUILU_DUNE(float, 2);
+INSTANTIATE_GPUILU_DUNE(float, 3);
+INSTANTIATE_GPUILU_DUNE(float, 4);
+INSTANTIATE_GPUILU_DUNE(float, 5);
+INSTANTIATE_GPUILU_DUNE(float, 6);
