@@ -22,6 +22,7 @@
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/simulators/linalg/gpuistl/detail/gpu_safe_call.hpp>
+#include <opm/simulators/linalg/gpuistl/gpu_resources.hpp>
 #include <string>
 #include <utility>
 
@@ -31,6 +32,7 @@ namespace Opm::gpuistl::detail
 /// @brief Function that tests the best thread block size, assumes the provided function depends on threadblock-size
 /// @tparam The type of the function to tune
 /// @param f the function to tune, which takes the thread block size as the input
+/// @param descriptionOfFunction Description of function
 template <typename func>
 int
 tuneThreadBlockSize(func& f, std::string descriptionOfFunction)
@@ -43,12 +45,7 @@ tuneThreadBlockSize(func& f, std::string descriptionOfFunction)
 
     // TODO: figure out a more rigorous way of deciding how many runs will suffice?
     constexpr const int runs = 2;
-    cudaEvent_t events[runs + 1];
-
-    // create the events
-    for (int i = 0; i < runs + 1; ++i) {
-        OPM_GPU_SAFE_CALL(cudaEventCreate(&events[i]));
-    }
+    std::array<GPUEvent, runs+1> events;
 
     // Initialize helper variables
     float bestTime = std::numeric_limits<float>::max();
@@ -59,21 +56,21 @@ tuneThreadBlockSize(func& f, std::string descriptionOfFunction)
     for (int thrBlockSize = interval; thrBlockSize <= 1024; thrBlockSize += interval) {
 
         // record a first event, and then an event after each kernel
-        OPM_GPU_SAFE_CALL(cudaEventRecord(events[0]));
+        OPM_GPU_SAFE_CALL(cudaEventRecord(events[0].get()));
         for (int i = 0; i < runs; ++i) {
             f(thrBlockSize); // runs an arbitrary function with the provided arguments
-            OPM_GPU_SAFE_CALL(cudaEventRecord(events[i + 1]));
+            OPM_GPU_SAFE_CALL(cudaEventRecord(events[i + 1].get()));
         }
 
-        // make suret he runs are over
-        OPM_GPU_SAFE_CALL(cudaEventSynchronize(events[runs]));
+        // make sure the runs are over
+        OPM_GPU_SAFE_CALL(cudaEventSynchronize(events[runs].get()));
 
         // kernel launch was valid
         if (cudaSuccess == cudaGetLastError()) {
             // check if we beat the record for the fastest kernel
             for (int i = 0; i < runs; ++i) {
                 float candidateBlockSizeTime;
-                OPM_GPU_SAFE_CALL(cudaEventElapsedTime(&candidateBlockSizeTime, events[i], events[i + 1]));
+                OPM_GPU_SAFE_CALL(cudaEventElapsedTime(&candidateBlockSizeTime, events[i].get(), events[i + 1].get()));
                 if (candidateBlockSizeTime < bestTime) { // checks if this configuration beat the current best
                     bestTime = candidateBlockSizeTime;
                     bestBlockSize = thrBlockSize;
