@@ -27,11 +27,16 @@
 #include <opm/simulators/timestepping/AdaptiveSimulatorTimer.hpp>
 #endif
 
+#include <dune/istl/istlexception.hh>
+
 #include <opm/common/Exceptions.hpp>
 #include <opm/common/ErrorMacros.hpp>
+#include <opm/common/OpmLog/OpmLog.hpp>
 
 #include <opm/grid/utility/StopWatch.hpp>
 
+#include <opm/input/eclipse/Schedule/Tuning.hpp>
+
 #include <opm/input/eclipse/Units/Units.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
@@ -48,7 +53,6 @@
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <fmt/format.h>
 #include <fmt/ranges.h>
-
 namespace Opm {
 /*********************************************
  * Public methods of AdaptiveTimeStepping
@@ -57,11 +61,11 @@ namespace Opm {
 
 //! \brief contructor taking parameter object
 template<class TypeTag>
-AdaptiveTimeStepping<TypeTag>::AdaptiveTimeStepping(
-                                 const UnitSystem& unit_system,
-                                 const SimulatorReport& report,
-                                 const double max_next_tstep,
-                                 const bool terminal_output
+AdaptiveTimeStepping<TypeTag>::
+AdaptiveTimeStepping(const UnitSystem& unit_system,
+                     const SimulatorReport& report,
+                     const double max_next_tstep,
+                     const bool terminal_output
 )
     : time_step_control_{}
     , restart_factor_{Parameters::Get<Parameters::SolverRestartFactor<Scalar>>()} // 0.33
@@ -92,20 +96,25 @@ AdaptiveTimeStepping<TypeTag>::AdaptiveTimeStepping(
 }
 
 //! \brief contructor
+//! \param max_next_tstep Maximum next time step allowed
 //! \param tuning Pointer to ecl TUNING keyword
+//! \param unit_system Unit system to use
+//! \param report Simulator report to use
+//! \param terminal_output True to print to terminal
 template<class TypeTag>
-AdaptiveTimeStepping<TypeTag>::AdaptiveTimeStepping(double max_next_tstep,
-                                                 const Tuning& tuning,
-                                                 const UnitSystem& unit_system,
-                                                 const SimulatorReport& report,
-                                                 const bool terminal_output
+AdaptiveTimeStepping<TypeTag>::
+AdaptiveTimeStepping(double max_next_tstep,
+                     const Tuning& tuning,
+                     const UnitSystem& unit_system,
+                     const SimulatorReport& report,
+                     const bool terminal_output
 )
     : time_step_control_{}
     , restart_factor_{tuning.TSFCNV}
     , growth_factor_{tuning.TFDIFF}
     , max_growth_{tuning.TSFMAX}
     , max_time_step_{tuning.TSMAXZ} // 365.25
-    , min_time_step_{tuning.TSFMIN} // 0.1;
+    , min_time_step_{tuning.TSMINZ} // 0.1;
     , ignore_convergence_failure_{true}
     , solver_restart_max_{Parameters::Get<Parameters::SolverMaxRestarts>()} // 10
     , solver_verbose_{Parameters::Get<Parameters::SolverVerbosity>() > 0 && terminal_output} // 2
@@ -178,27 +187,12 @@ registerParameters()
     detail::registerAdaptiveParameters();
 }
 
-#ifdef RESERVOIR_COUPLING_ENABLED
-template<class TypeTag>
-void
-AdaptiveTimeStepping<TypeTag>::
-setReservoirCouplingMaster(ReservoirCouplingMaster *reservoir_coupling_master)
-{
-    this->reservoir_coupling_master_ = reservoir_coupling_master;
-}
-
-template<class TypeTag>
-void
-AdaptiveTimeStepping<TypeTag>::
-setReservoirCouplingSlave(ReservoirCouplingSlave *reservoir_coupling_slave)
-{
-    this->reservoir_coupling_slave_ = reservoir_coupling_slave;
-}
-#endif
-
 /** \brief  step method that acts like the solver::step method
             in a sub cycle of time steps
-    \param tuningUpdater Function used to update TUNING parameters before each
+    \param simulator_timer Simulator timer
+    \param solver Solver to use
+    \param is_event True if this is an event
+    \param tuning_updater Function used to update TUNING parameters before each
                          time step. ACTIONX might change tuning.
 */
 template<class TypeTag>
@@ -208,11 +202,7 @@ AdaptiveTimeStepping<TypeTag>::
 step(const SimulatorTimer& simulator_timer,
      Solver& solver,
      const bool is_event,
-     const std::function<bool(const double /*current_time*/,
-                              const double /*dt*/,
-                              const int    /*substep_number*/
-                             )> tuning_updater
-)
+     const TuningUpdateCallback& tuning_updater)
 {
     SubStepper<Solver> sub_stepper{
         *this, simulator_timer, solver, is_event, tuning_updater,
@@ -325,6 +315,14 @@ suggestedNextStep() const
     return this->suggested_next_timestep_;
 }
 
+template<class TypeTag>
+const TimeStepControlInterface&
+AdaptiveTimeStepping<TypeTag>::
+timeStepControl() const
+{
+    return *this->time_step_control_;
+}
+
 
 template<class TypeTag>
 void
@@ -454,17 +452,11 @@ init_(const UnitSystem& unitSystem)
 template<class TypeTag>
 template<class Solver>
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
-SubStepper(
-    AdaptiveTimeStepping<TypeTag>& adaptive_time_stepping,
-    const SimulatorTimer& simulator_timer,
-    Solver& solver,
-    const bool is_event,
-    const std::function<bool(const double /*current_time*/,
-                             const double /*dt*/,
-                             const int    /*substep_number*/
-                            )>& tuning_updater
-
-)
+SubStepper(AdaptiveTimeStepping<TypeTag>& adaptive_time_stepping,
+           const SimulatorTimer& simulator_timer,
+           Solver& solver,
+           const bool is_event,
+           const TuningUpdateCallback& tuning_updater)
     : adaptive_time_stepping_{adaptive_time_stepping}
     , simulator_timer_{simulator_timer}
     , solver_{solver}
@@ -514,7 +506,7 @@ bool
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 isReservoirCouplingMaster_() const
 {
-    return this->adaptive_time_stepping_.reservoir_coupling_master_ != nullptr;
+    return this->solver_.model().simulator().reservoirCouplingMaster() != nullptr;
 }
 
 template<class TypeTag>
@@ -523,7 +515,7 @@ bool
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 isReservoirCouplingSlave_() const
 {
-    return this->adaptive_time_stepping_.reservoir_coupling_slave_ != nullptr;
+    return this->solver_.model().simulator().reservoirCouplingSlave() != nullptr;
 }
 #endif
 
@@ -565,9 +557,9 @@ SimulatorReport
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 runStepOriginal_()
 {
-    auto elapsed = this->simulator_timer_.simulationTimeElapsed();
-    auto original_time_step = this->simulator_timer_.currentStepLength();
-    auto report_step = this->simulator_timer_.reportStepNum();
+    const auto elapsed = this->simulator_timer_.simulationTimeElapsed();
+    const auto original_time_step = this->simulator_timer_.currentStepLength();
+    const auto report_step = this->simulator_timer_.reportStepNum();
     maybeUpdateTuning_(elapsed, original_time_step, report_step);
     maybeModifySuggestedTimeStepAtBeginningOfReportStep_(original_time_step);
 
@@ -590,7 +582,7 @@ ReservoirCouplingMaster&
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 reservoirCouplingMaster_()
 {
-    return *adaptive_time_stepping_.reservoir_coupling_master_;
+    return *(this->solver_.model().simulator().reservoirCouplingMaster());
 }
 #endif
 
@@ -601,7 +593,7 @@ ReservoirCouplingSlave&
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 reservoirCouplingSlave_()
 {
-    return *this->adaptive_time_stepping_.reservoir_coupling_slave_;
+    return *(this->solver_.model().simulator().reservoirCouplingSlave());
 }
 #endif
 
@@ -642,14 +634,15 @@ SimulatorReport
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 runStepReservoirCouplingMaster_()
 {
-    bool substep_done = false;
     int iteration = 0;
     const double original_time_step = this->simulator_timer_.currentStepLength();
     double current_time{this->simulator_timer_.simulationTimeElapsed()};
     double step_end_time = current_time + original_time_step;
     auto current_step_length = original_time_step;
     SimulatorReport report;
-    while(!substep_done) {
+    // The master needs to know which slaves have activated before it can start the substep loop
+    reservoirCouplingMaster_().maybeReceiveActivationHandshakeFromSlaves(current_time);
+    while (true) {
         reservoirCouplingMaster_().receiveNextReportDateFromSlaves();
         if (iteration == 0) {
             maybeUpdateTuning_(current_time, current_step_length, /*substep=*/0);
@@ -668,11 +661,11 @@ runStepReservoirCouplingMaster_()
             /*reportStep=*/this->simulator_timer_.reportStepNum(),
             maxTimeStep_()
         };
-        bool final_step = ReservoirCoupling::Seconds::compare_gt_or_eq(
+        const bool final_step = ReservoirCoupling::Seconds::compare_gt_or_eq(
             current_time + current_step_length, step_end_time
         );
         SubStepIteration<Solver> substepIteration{*this, substep_timer, current_step_length, final_step};
-        auto sub_steps_report = substepIteration.run();
+        const auto sub_steps_report = substepIteration.run();
         report += sub_steps_report;
         current_time += current_step_length;
         if (final_step) {
@@ -691,15 +684,14 @@ SimulatorReport
 AdaptiveTimeStepping<TypeTag>::SubStepper<Solver>::
 runStepReservoirCouplingSlave_()
 {
-    bool substep_done = false;
     int iteration = 0;
     const double original_time_step = this->simulator_timer_.currentStepLength();
     double current_time{this->simulator_timer_.simulationTimeElapsed()};
     double step_end_time = current_time + original_time_step;
     SimulatorReport report;
-    while(!substep_done) {
+    while (true) {
         reservoirCouplingSlave_().sendNextReportDateToMasterProcess();
-        auto timestep = reservoirCouplingSlave_().receiveNextTimeStepFromMaster();
+        const auto timestep = reservoirCouplingSlave_().receiveNextTimeStepFromMaster();
         if (iteration == 0) {
             maybeUpdateTuning_(current_time, original_time_step, /*substep=*/0);
             maybeModifySuggestedTimeStepAtBeginningOfReportStep_(timestep);
@@ -712,15 +704,14 @@ runStepReservoirCouplingSlave_()
             this->simulator_timer_.reportStepNum(),
             maxTimeStep_()
         };
-        bool final_step = ReservoirCoupling::Seconds::compare_gt_or_eq(
+        const bool final_step = ReservoirCoupling::Seconds::compare_gt_or_eq(
             current_time + timestep, step_end_time
         );
         SubStepIteration<Solver> substepIteration{*this, substep_timer, timestep, final_step};
-        auto sub_steps_report = substepIteration.run();
+        const auto sub_steps_report = substepIteration.run();
         report += sub_steps_report;
         current_time += timestep;
         if (final_step) {
-            substep_done = true;
             break;
         }
         iteration++;
@@ -748,12 +739,10 @@ suggestedNextTimestep_() const
 template<class TypeTag>
 template<class Solver>
 AdaptiveTimeStepping<TypeTag>::SubStepIteration<Solver>::
-SubStepIteration(
-    SubStepper<Solver>& substepper,
-    AdaptiveSimulatorTimer& substep_timer,
-    const double original_time_step,
-    bool final_step
-)
+SubStepIteration(SubStepper<Solver>& substepper,
+                 AdaptiveSimulatorTimer& substep_timer,
+                 const double original_time_step,
+                 bool final_step)
     : substepper_{substepper}
     , substep_timer_{substep_timer}
     , original_time_step_{original_time_step}
@@ -777,7 +766,7 @@ run()
     // sub step time loop
     while (!this->substep_timer_.done()) {
         // if we just chopped the timestep due to convergence i.e. restarts>0
-        // we dont what to update the next timestep based on Tuning
+        // we dont want to update the next timestep based on Tuning
         if (restarts == 0) {
             maybeUpdateTuningAndTimeStep_();
         }
@@ -786,7 +775,7 @@ run()
             detail::logTimer(this->substep_timer_);
         }
 
-        auto substep_report = runSubStep_();
+        const auto substep_report = runSubStep_();
 
         //Pass substep to eclwriter for summary output
         problem.setSubStepReport(substep_report);
@@ -822,11 +811,19 @@ run()
             report.success.converged = this->substep_timer_.done();
             this->substep_timer_.setLastStepFailed(false);
         }
-        else { // in case of no convergence
+        else { // in case of no convergence or time step tolerance test failure
             this->substep_timer_.setLastStepFailed(true);
             checkTimeStepMaxRestartLimit_(restarts);
 
-            const double new_time_step = restartFactor_() * dt;
+            double new_time_step = restartFactor_() * dt;
+            if (substep_report.time_step_rejected) {
+                const double tol = Parameters::Get<Parameters::TimeStepControlTolerance>();
+                const double safetyFactor = Parameters::Get<Parameters::TimeStepControlSafetyFactor>();
+                const double temp_time_step = std::sqrt(safetyFactor * tol / solver_().model().relativeChange()) * dt;
+                if (temp_time_step < dt) { // added in case suggested time step is not a reduction
+                    new_time_step = temp_time_step;
+                }
+            }
             checkTimeStepMinLimit_(new_time_step);
             bool wells_shut = false;
             if (new_time_step > minTimeStepBeforeClosingWells_()) {
@@ -859,7 +856,7 @@ bool
 AdaptiveTimeStepping<TypeTag>::SubStepIteration<Solver>::
 checkContinueOnUnconvergedSolution_(double dt) const
 {
-    bool continue_on_uncoverged_solution = ignoreConvergenceFailure_() && dt <= minTimeStep_();
+    const bool continue_on_uncoverged_solution = ignoreConvergenceFailure_() && dt <= minTimeStep_();
     if (continue_on_uncoverged_solution && solverVerbose_()) {
         // NOTE: This method is only called if the solver failed to converge.
         const auto msg = fmt::format(
@@ -898,14 +895,25 @@ void
 AdaptiveTimeStepping<TypeTag>::SubStepIteration<Solver>::
 checkTimeStepMinLimit_(const double new_time_step) const
 {
+    using Meas = UnitSystem::measure;
     // If we have restarted (i.e. cut the timestep) too
     // much, we have failed and throw an exception.
     if (new_time_step < minTimeStep_()) {
-        const auto msg = fmt::format(
-            "Solver failed to converge after cutting timestep to {}\n"
-            "which is the minimum threshold given by option --solver-min-time-step\n",
-            minTimeStep_()
-        );
+        auto msg = fmt::format("Solver failed to converge after cutting timestep to ");
+        if (Parameters::Get<Parameters::EnableTuning>()) {
+            const UnitSystem& unit_system = solver_().model().simulator().vanguard().eclState().getDeckUnitSystem();
+            msg += fmt::format(
+                "{:.3E} {}\nwhich is the minimum threshold given by the TUNING keyword\n",
+                unit_system.from_si(Meas::time, minTimeStep_()),
+                unit_system.name(Meas::time)
+            );
+        }
+        else {
+            msg += fmt::format(
+                "{:.3E} DAYS\nwhich is the minimum threshold given by option --solver-min-time-step\n",
+                minTimeStep_() / 86400.0
+            );
+        }
         if (solverVerbose_()) {
             OpmLog::error(msg);
         }
@@ -942,8 +950,10 @@ chopTimeStepOrCloseFailingWells_(const double new_time_step)
     // If we already have chopped the timestep two times that is
     // new_time_step < minTimeStepBeforeClosingWells_()*restartFactor_()*restartFactor_()
     // We also shut wells that fails only on this step.
-    bool requireRepeatedFailures = new_time_step > ( minTimeStepBeforeClosingWells_()*restartFactor_()*restartFactor_());
-    std::set<std::string> failing_wells = detail::consistentlyFailingWells(solver_().model().stepReports(), requireRepeatedFailures);
+    const bool requireRepeatedFailures =
+        new_time_step > (minTimeStepBeforeClosingWells_() * restartFactor_() * restartFactor_());
+    const std::set<std::string> failing_wells =
+        detail::consistentlyFailingWells(solver_().model().stepReports(), requireRepeatedFailures);
 
     if (failing_wells.empty()) {
         // Found no wells to close, chop the timestep
@@ -952,8 +962,10 @@ chopTimeStepOrCloseFailingWells_(const double new_time_step)
         // Close all consistently failing wells that are not under group control
         std::vector<std::string> shut_wells;
         for (const auto& well : failing_wells) {
-            bool was_shut = solver_().model().wellModel().forceShutWellByName(
-                        well, this->substep_timer_.simulationTimeElapsed(), /*dont_shut_grup_wells =*/ true);
+            const bool was_shut =
+                solver_().model().wellModel().forceShutWellByName(well,
+                                                                  this->substep_timer_.simulationTimeElapsed(),
+                                                                  /*dont_shut_grup_wells =*/ true);
             if (was_shut) {
                 shut_wells.push_back(well);
             }
@@ -961,8 +973,10 @@ chopTimeStepOrCloseFailingWells_(const double new_time_step)
         // If no wells are closed we also try to shut wells under group control
         if (shut_wells.empty()) {
             for (const auto& well : failing_wells) {
-                bool was_shut = solver_().model().wellModel().forceShutWellByName(
-                        well, this->substep_timer_.simulationTimeElapsed(), /*dont_shut_grup_wells =*/ false);
+                const bool was_shut =
+                    solver_().model().wellModel().forceShutWellByName(well,
+                                                                      this->substep_timer_.simulationTimeElapsed(),
+                                                                      /*dont_shut_grup_wells =*/ false);
                 if (was_shut) {
                     shut_wells.push_back(well);
                 }
@@ -1080,10 +1094,10 @@ maybeUpdateTuningAndTimeStep_()
     // the current definition of the maybeUpdateTuning_() callback is actually calling
     // adaptiveTimeStepping_->updateTUNING(max_next_tstep, tuning) which is updating the tuning
     // see SimulatorFullyImplicitBlackoil::runStep() for more details.
-    auto old_value = suggestedNextTimestep_();
+    const auto old_value = suggestedNextTimestep_();
     if (this->substepper_.maybeUpdateTuning_(this->substep_timer_.simulationTimeElapsed(),
-                                                this->substep_timer_.currentStepLength(),
-                                                this->substep_timer_.currentStepNum()))
+                                             this->substep_timer_.currentStepLength(),
+                                             this->substep_timer_.currentStepNum()))
     {
         // Either NEXTSTEP and WCYCLE wants to change the current time step, but they cannot
         // change the current time step directly. Instead, they change the suggested next time step
@@ -1136,15 +1150,12 @@ runSubStep_()
         substep_report = solver_().failureReport();
         this->cause_of_failure_ = failure_reason;
         if (log_exception && solverVerbose_()) {
-            std::string message;
-            message = "Caught Exception: ";
-            message += e.what();
-            OpmLog::debug(message);
+            OpmLog::debug(std::string("Caught Exception: ") + e.what());
         }
     };
 
     try {
-        substep_report = solver_().step(this->substep_timer_);
+        substep_report = solver_().step(this->substep_timer_, &this->adaptive_time_stepping_.timeStepControl());
         if (solverVerbose_()) {
             // report number of linear iterations
             OpmLog::debug("Overall linear iterations used: "
@@ -1154,6 +1165,9 @@ runSubStep_()
     catch (const TooManyIterations& e) {
         handleFailure("Solver convergence failure - Iteration limit reached", e);
     }
+    catch (const TimeSteppingBreakdown& e) {
+        handleFailure(e.what(), e);
+    }
     catch (const ConvergenceMonitorFailure& e) {
         handleFailure("Convergence monitor failure", e, /*log_exception=*/false);
     }
@@ -1253,10 +1267,11 @@ template <class TypeTag>
 template <class Solver>
 double
 AdaptiveTimeStepping<TypeTag>::SubStepIteration<Solver>::
-timeStepControlComputeEstimate_(const double dt, const int iterations, AdaptiveSimulatorTimer& substepTimer) const
+timeStepControlComputeEstimate_(const double dt, const int iterations,
+                                const AdaptiveSimulatorTimer& substepTimer) const
 {
     // create object to compute the time error, simply forwards the call to the model
-    SolutionTimeErrorSolverWrapper<Solver> relative_change{solver_()};
+    const SolutionTimeErrorSolverWrapper<Solver> relative_change{solver_()};
     return this->adaptive_time_stepping_.time_step_control_->computeTimeStepSize(
         dt, iterations, relative_change, substepTimer);
 }
@@ -1325,9 +1340,9 @@ writeOutput_() const
 
 template<class TypeTag>
 template<class Solver>
-AdaptiveTimeStepping<TypeTag>::SolutionTimeErrorSolverWrapper<Solver>::SolutionTimeErrorSolverWrapper(
-    const Solver& solver
-)
+AdaptiveTimeStepping<TypeTag>::
+SolutionTimeErrorSolverWrapper<Solver>::
+SolutionTimeErrorSolverWrapper(const Solver& solver)
     : solver_{solver}
 {}
 
