@@ -35,8 +35,8 @@ namespace Opm
         return SimulatorReportSingle{1.0, 2.0, 3.0, 4.0, 5.0, 6.0,
                                      7.0, 8.0, 9.0, 10.0, 11.0, 12.0,
                                      13, 14, 15, 16, 17, 18,
-                                     true, false, 19, 20.0, 21.0,
-                                     22, 23, 24, 25, 26, 27, 28};
+                                     true, false, false, 19, 20.0, 21.0,
+                                     22, 23, 24, 25, 26, 27, 28, 29};
     }
 
     bool SimulatorReportSingle::operator==(const SimulatorReportSingle& rhs) const
@@ -60,6 +60,7 @@ namespace Opm
                this->min_linear_iterations == rhs.min_linear_iterations &&
                this->max_linear_iterations == rhs.max_linear_iterations &&
                this->converged == rhs.converged &&
+               this->time_step_rejected == rhs.time_step_rejected &&
                this->well_group_control_changed == rhs.well_group_control_changed &&
                this->exit_status == rhs.exit_status &&
                this->global_time == rhs.global_time &&
@@ -70,7 +71,8 @@ namespace Opm
                this->num_owned_cells == rhs.num_owned_cells &&
                this->converged_domains == rhs.converged_domains &&
                this->unconverged_domains == rhs.unconverged_domains &&
-               this->accepted_unconverged_domains == rhs.accepted_unconverged_domains;
+               this->accepted_unconverged_domains == rhs.accepted_unconverged_domains &&
+               this->skipped_domains == rhs.skipped_domains;
     }
 
     void SimulatorReportSingle::operator+=(const SimulatorReportSingle& sr)
@@ -99,9 +101,9 @@ namespace Opm
         converged_domains += sr.converged_domains;
         unconverged_domains += sr.unconverged_domains;
         accepted_unconverged_domains += sr.accepted_unconverged_domains;
-
-        // It makes no sense adding time points. Therefore, do not 
-        // overwrite the value of global_time which gets set in 
+        skipped_domains += sr.skipped_domains;
+        // It makes no sense adding time points. Therefore, do not
+        // overwrite the value of global_time which gets set in
         // NonlinearSolver.hpp by the line:
         //     report.global_time = timer.simulationTimeElapsed();
     }
@@ -331,6 +333,9 @@ namespace Opm
         }
         os << std::endl;
         os << fmt::format("-------------------------------------------------------\n");
+        n = skipped_domains + (failureReport ? failureReport->skipped_domains : 0);
+        os << fmt::format("Skipped domain solves:       {:7}", n);
+        os << std::endl;
         n = converged_domains + (failureReport ? failureReport->converged_domains : 0);
         os << fmt::format("Converged domain solves:     {:7}", n);
         os << std::endl;
