@@ -28,6 +28,7 @@
 #include <string>
 #include <fstream>
 #include <iostream>
+#include <sstream>
 #include <limits>
 
 #include <opm/common/ErrorMacros.hpp>
@@ -74,8 +75,11 @@ namespace Opm
         return {1, 1.0, 2.0, true};
     }
 
-    double SimpleIterationCountTimeStepControl::
-    computeTimeStepSize( const double dt, const int iterations, const RelativeChangeInterface& /* relativeChange */, const AdaptiveSimulatorTimer& /* substepTimer */) const
+    double
+    SimpleIterationCountTimeStepControl::computeTimeStepSize(const double dt,
+                                                             const int iterations,
+                                                             const RelativeChangeInterface& /* relativeChange */,
+                                                             const AdaptiveSimulatorTimer& /* substepTimer */) const
     {
         double dtEstimate = dt ;
 
@@ -91,6 +95,10 @@ namespace Opm
             dtEstimate *= growthrate_;
         }
 
+        if( verbose_ )
+            OpmLog::debug(fmt::format("Suggested next time step size (from controller): {} days",
+                                      unit::convert::to( dtEstimate, unit::day )));
+
         return dtEstimate;
     }
 
@@ -114,7 +122,8 @@ namespace Opm
     {
         std::ifstream infile (filename);
         if (!infile.is_open()) {
-            OPM_THROW(std::runtime_error,"Incorrect or no filename is provided to the hardcodedTimeStep. Use timestep.control.filename=your_file_name");
+            OPM_THROW(std::runtime_error,"Incorrect or no filename is provided to the hardcodedTimeStep. "
+                      "Use timestep.control.filename=your_file_name");
         }
         std::string::size_type sz;
         std::string line;
@@ -135,10 +144,14 @@ namespace Opm
         return result;
     }
 
-    double HardcodedTimeStepControl::
-    computeTimeStepSize( const double /*dt */, const int /*iterations */, const RelativeChangeInterface& /* relativeChange */ , const AdaptiveSimulatorTimer& substepTimer) const
+    double
+    HardcodedTimeStepControl::computeTimeStepSize(const double /*dt */,
+                                                  const int /*iterations */,
+                                                  const RelativeChangeInterface& /* relativeChange */,
+                                                  const AdaptiveSimulatorTimer& substepTimer) const
     {
-        auto nextTime = std::upper_bound(subStepTime_.begin(), subStepTime_.end(), substepTimer.simulationTimeElapsed());
+        auto nextTime
+            = std::upper_bound(subStepTime_.begin(), subStepTime_.end(), substepTimer.simulationTimeElapsed());
         return (*nextTime - substepTimer.simulationTimeElapsed());
     }
 
@@ -171,8 +184,11 @@ namespace Opm
         return result;;
     }
 
-    double PIDTimeStepControl::
-    computeTimeStepSize( const double dt, const int /* iterations */, const RelativeChangeInterface& relChange, const AdaptiveSimulatorTimer& /* substepTimer */) const
+    double
+    PIDTimeStepControl::computeTimeStepSize(const double dt,
+                                            const int /* iterations */,
+                                            const RelativeChangeInterface& relChange,
+                                            const AdaptiveSimulatorTimer& /* substepTimer */) const
     {
         // shift errors
         for( int i=0; i<2; ++i ) {
@@ -191,13 +207,15 @@ namespace Opm
             // adjust dt by given tolerance
             const double newDt = dt * tol_ / error;
             if ( verbose_ )
-                    OpmLog::info(fmt::format("Computed step size (tol): {} days", unit::convert::to( newDt, unit::day )));
+                    OpmLog::debug(fmt::format("Suggested next time step size (from PID controller): {} days",
+                                              unit::convert::to( newDt, unit::day )));
             return newDt;
         }
         else if (errors_[0] == 0 || errors_[1] == 0 || errors_[2] == 0.)
         {
             if ( verbose_ )
-                OpmLog::info("The solution between time steps does not change, there is no time step constraint from the PID time step control ");
+                OpmLog::debug("The solution between time steps does not change, "
+                              "there is no time step constraint from the PID time step control ");
             return std::numeric_limits<double>::max();
         }
         else
@@ -210,7 +228,8 @@ namespace Opm
                                  std::pow( tol_         / errors_[ 2 ], kI ) *
                                  std::pow( errors_[1]*errors_[1]/errors_[ 0 ]/errors_[ 2 ], kD ));
             if( verbose_ )
-                OpmLog::info(fmt::format("Computed step size (pow): {} days", unit::convert::to( newDt, unit::day )));
+                OpmLog::debug(fmt::format("Suggested next time step size (from PID controller): {} days",
+                                          unit::convert::to( newDt, unit::day )));
             return newDt;
         }
     }
@@ -242,6 +261,7 @@ namespace Opm
         , decayDampingFactor_( decayDampingFactor )
         , growthDampingFactor_( growthDampingFactor )
         , minTimeStepBasedOnIterations_(minTimeStepBasedOnIterations)
+        , verbose_(verbose)
     {}
 
     PIDAndIterationCountTimeStepControl
@@ -250,8 +270,11 @@ namespace Opm
         return PIDAndIterationCountTimeStepControl{1, 2.0, 3.0, 4.0, 5.0, true};
     }
 
-    double PIDAndIterationCountTimeStepControl::
-    computeTimeStepSize( const double dt, const int iterations, const RelativeChangeInterface& relChange,  const AdaptiveSimulatorTimer& substepTimer) const
+    double
+    PIDAndIterationCountTimeStepControl::computeTimeStepSize(const double dt,
+                                                             const int iterations,
+                                                             const RelativeChangeInterface& relChange,
+                                                             const AdaptiveSimulatorTimer& substepTimer) const
     {
         double dtEstimatePID = PIDTimeStepControl :: computeTimeStepSize( dt, iterations, relChange, substepTimer);
 
@@ -269,6 +292,10 @@ namespace Opm
             dtEstimateIter = dt * (1.0 + off_target_fraction * growthDampingFactor_);
         }
 
+        if( verbose_ )
+            OpmLog::debug(fmt::format("Suggested next time step size (from iteration target): {} days",
+                                      unit::convert::to( dtEstimateIter, unit::day )));
+
         return std::min(dtEstimatePID, dtEstimateIter);
     }
 
@@ -278,7 +305,8 @@ namespace Opm
                this->target_iterations_ == ctrl.target_iterations_ &&
                this->decayDampingFactor_ == ctrl.decayDampingFactor_ &&
                this->growthDampingFactor_ == ctrl.growthDampingFactor_ &&
-               this->minTimeStepBasedOnIterations_ == ctrl.minTimeStepBasedOnIterations_;
+               this->minTimeStepBasedOnIterations_ == ctrl.minTimeStepBasedOnIterations_ &&
+               this->verbose_ == ctrl.verbose_;
     }
 
 
@@ -291,82 +319,182 @@ namespace Opm
 
     General3rdOrderController::General3rdOrderController( const double tolerance,
                                                           const double safetyFactor,
+                                                          const bool rejectCompletedStep,
+                                                          const std::string& toleranceTestVersion,
+                                                          const double maxReductionTimeStep,
+                                                          const std::string& parameters,
                                                           const bool verbose)
         : tolerance_( tolerance )
         , safetyFactor_( safetyFactor )
-        , errors_( 3, tolerance_ )
-        , timeSteps_ ( 3, 1.0 )
+        , rejectCompletedStep_( rejectCompletedStep )
+        , maxReductionTimeStep_( maxReductionTimeStep )
         , verbose_( verbose )
-    {}
+    {
+        std::istringstream parameters_stream(parameters);
+        std::string value;
+        int counter = 0;
+        try {
+            while (std::getline(parameters_stream, value, ';'))
+            {
+                if (counter < 3) {
+                    beta_[counter] = std::stod(value);
+                }
+                else {
+                    alpha_[counter - 3] = std::stod(value);
+                }
+                counter++;
+            }
+        }
+        catch (const std::exception& e) {
+            OPM_THROW(std::runtime_error, "The parameter string for the time step controller is invalid: " + parameters);
+        }
+
+        if (toleranceTestVersion == "standard") {
+            toleranceTestVersion_ = ToleranceTestVersions::Standard;
+        }
+        else if (toleranceTestVersion == "control-error-filtering") {
+            toleranceTestVersion_ = ToleranceTestVersions::ControlErrorFiltering;
+        }
+        else {
+            OPM_THROW(std::runtime_error, "Unsupported tolerance test version given: " + toleranceTestVersion);
+        }
+    }
 
     General3rdOrderController
     General3rdOrderController::serializationTestObject()
     {
-        General3rdOrderController result(1.0, 0.8, true);
+        General3rdOrderController result(1.0, 2.0, false, "standard", 3.0, "0.125;0.25;0.125;0.75;0.25", false);
         result.errors_ = {2.0, 3.0};
 
         return result;
     }
 
-    double General3rdOrderController::
-    computeTimeStepSize(const double dt, const int /*iterations */, const RelativeChangeInterface& relChange, const AdaptiveSimulatorTimer& substepTimer) const
+    double
+    General3rdOrderController::computeTimeStepSize(const double dt,
+                                                   const int /* iterations */,
+                                                   const RelativeChangeInterface& /* relChange */,
+                                                   const AdaptiveSimulatorTimer& substepTimer) const
     {
-        // Shift errors and time steps
-        for( int i = 0; i < 2; ++i )
-        {
-            errors_[i] = errors_[i+1];
-            timeSteps_[i] = timeSteps_[i+1];
-        }
-
-        // Store new error and time step
-        const double error = relChange.relativeChange();
-        errors_[2] = error;
-        timeSteps_[2] = dt;
         for( int i = 0; i < 2; ++i )
         {
             assert(std::isfinite(errors_[i]));
         }
 
-        if (errors_[0] == 0 || errors_[1] == 0 || errors_[2] == 0.)
+        if (substepTimer.currentStepNum() < 3) // Use an I controller after report time steps
         {
-            if ( verbose_ )
-                OpmLog::info("The solution between time steps does not change, there is no time step constraint from the controller.");
-            return std::numeric_limits<double>::max();
+            controllerVersion_ = InternalControlVersions::IController;
         }
-        // Use an I controller after report time steps or chopped time steps
-        else if (substepTimer.currentStepNum() < 3 || substepTimer.lastStepFailed() || counterSinceFailure_ > 0)
+        else // Use the general third order controller for all other time steps
         {
-            if (substepTimer.lastStepFailed() || counterSinceFailure_ > 0)
-                counterSinceFailure_++;
-            if (counterSinceFailure_ > 1)
-                counterSinceFailure_ = 0;
-            const double newDt = dt * std::pow(safetyFactor_ * tolerance_ / errors_[2], 0.35);
-            if( verbose_ )
-                OpmLog::info(fmt::format("Computed step size (pow): {} days", unit::convert::to( newDt, unit::day )));
-            return newDt;
+            controllerVersion_ = InternalControlVersions::General3rdOrder;
+        }
+
+        bool noTimeStepConstraint = false;
+
+        if (controllerVersion_ == InternalControlVersions::IController)
+        {
+            if (errors_[2] == 0)
+            {
+                noTimeStepConstraint = true;
+            }
         }
-        // Use the general third order controller for all other time steps
         else
         {
-            const std::array<double, 3> beta = { 0.125, 0.25, 0.125 };
-            const std::array<double, 2> alpha = { 0.375, 0.125 };
-            const double newDt = dt * std::pow(safetyFactor_ * tolerance_ / errors_[2], beta[0]) *
-                                      std::pow(safetyFactor_ * tolerance_ / errors_[1], beta[1]) *
-                                      std::pow(safetyFactor_ * tolerance_ / errors_[0], beta[2]) *
-                                      std::pow(timeSteps_[2] / timeSteps_[1], -alpha[0]) *
-                                      std::pow(timeSteps_[1] / timeSteps_[0], -alpha[1]);
-            if( verbose_ )
-                OpmLog::info(fmt::format("Computed step size (pow): {} days", unit::convert::to( newDt, unit::day )));
-            return newDt;
+            if (errors_[0] == 0 || errors_[1] == 0 || errors_[2] == 0)
+            {
+                noTimeStepConstraint = true;
+            }
         }
+
+        if (noTimeStepConstraint)
+        {
+            if ( verbose_ )
+            {
+                OpmLog::debug("The solution between time steps does not change, "
+                              "there is no time step constraint from the controller.");
+            }
+            return std::numeric_limits<double>::max();
+        }
+
+        const double newDt = dt * timeStepFactor(errors_, timeSteps_);
+
+        if (verbose_)
+        {
+            OpmLog::debug(fmt::format("Suggested next time step size (from controller): {} days",
+                                      unit::convert::to( newDt, unit::day )));
+            OpmLog::debug(fmt::format("Most recent relative change: {}", errors_[2]));
+        }
+        return newDt;
+    }
+
+    double General3rdOrderController::
+    timeStepFactor(const std::array<double, 3>& errors, const std::array<double, 3>& timeSteps) const
+    {
+        if (controllerVersion_ == InternalControlVersions::General3rdOrder)
+        {
+            return std::pow(safetyFactor_ * tolerance_ / errors[2], beta_[0]) *
+                   std::pow(safetyFactor_ * tolerance_ / errors[1], beta_[1]) *
+                   std::pow(safetyFactor_ * tolerance_ / errors[0], beta_[2]) *
+                   std::pow(timeSteps[2] / timeSteps[1], -alpha_[0]) *
+                   std::pow(timeSteps[1] / timeSteps[0], -alpha_[1]);
+        }
+        // controllerVersion_ == InternalControlVersions::IController
+        return std::pow(safetyFactor_ * tolerance_ / errors[2], 0.35);
+    }
+
+    bool General3rdOrderController::
+    timeStepAccepted(const double error, const double timeStepJustCompleted) const
+    {
+        bool acceptTimeStep = true;
+
+        // Reject time step if chosen tolerance test version fails
+        if (toleranceTestVersion_ == ToleranceTestVersions::Standard)
+        {
+            if (rejectCompletedStep_ && error > tolerance_)
+            {
+                acceptTimeStep = false;
+            }
+        }
+        else if (toleranceTestVersion_ == ToleranceTestVersions::ControlErrorFiltering)
+        {
+            const std::array<double, 3> tempErrors{errors_[1], errors_[2], error};
+            const std::array<double, 3> tempTimeSteps{timeSteps_[1], timeSteps_[2], timeStepJustCompleted};
+            const double stepFactor = timeStepFactor(tempErrors, tempTimeSteps);
+            if (rejectCompletedStep_ && stepFactor < maxReductionTimeStep_)
+            {
+                acceptTimeStep = false;
+            }
+        }
+
+        if (acceptTimeStep)
+        {
+            // Shift errors and time steps
+            for( int i = 0; i < 2; ++i )
+            {
+                errors_[i] = errors_[i+1];
+                timeSteps_[i] = timeSteps_[i+1];
+            }
+
+            // Store new error and time step
+            errors_[2] = error;
+            timeSteps_[2] = timeStepJustCompleted;
+        }
+
+        return acceptTimeStep;
     }
 
     bool General3rdOrderController::operator==(const General3rdOrderController& ctrl) const
     {
         return this->tolerance_ == ctrl.tolerance_ &&
                this->safetyFactor_ == ctrl.safetyFactor_ &&
+               this->rejectCompletedStep_ == ctrl.rejectCompletedStep_ &&
                this->errors_ == ctrl.errors_ &&
                this->timeSteps_ == ctrl.timeSteps_ &&
+               this->beta_ == ctrl.beta_ &&
+               this->alpha_ == ctrl.alpha_ &&
+               this->controllerVersion_ == ctrl.controllerVersion_ &&
+               this->toleranceTestVersion_ == ctrl.toleranceTestVersion_ &&
+               this->maxReductionTimeStep_ == ctrl.maxReductionTimeStep_ &&
                this->verbose_ == ctrl.verbose_;
     }
 
