@@ -23,10 +23,10 @@
 
 #include <vector>
 
-namespace Opm
-{
+namespace Opm {
 
 class EclipseIO;
+namespace data { class Solution; }
 class SummaryState;
 class RestartKey;
 class RestartValue;
@@ -43,6 +43,11 @@ RestartValue loadParallelRestart(const EclipseIO* eclIO,
                                  const std::vector<RestartKey>& extraKeys,
                                  Parallel::Communication comm);
 
+data::Solution loadParallelRestartSolution(const EclipseIO* eclIO,
+                                           const std::vector<RestartKey>& solutionKeys,
+                                           Parallel::Communication comm,
+                                           const int step);
+
 } // end namespace Opm
 
 #endif // PARALLEL_RESTART_HPP
