@@ -20,10 +20,17 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
 #ifndef OPM_BLACKOILWELLMODEL_HEADER_INCLUDED
 #define OPM_BLACKOILWELLMODEL_HEADER_INCLUDED
 
+#if HAVE_MPI
+#define RESERVOIR_COUPLING_ENABLED
+#endif
+#ifdef RESERVOIR_COUPLING_ENABLED
+#include <opm/simulators/flow/ReservoirCouplingMaster.hpp>
+#include <opm/simulators/flow/ReservoirCouplingSlave.hpp>
+#endif
+
 #include <dune/common/fmatrix.hh>
 #include <dune/istl/bcrsmatrix.hh>
 #include <dune/istl/matrixmatrix.hh>
@@ -54,6 +61,7 @@
 #include <opm/simulators/wells/GasLiftSingleWell.hpp>
 #include <opm/simulators/wells/GasLiftSingleWellGeneric.hpp>
 #include <opm/simulators/wells/GasLiftWellState.hpp>
+#include <opm/simulators/wells/GuideRateHandler.hpp>
 #include <opm/simulators/wells/MultisegmentWell.hpp>
 #include <opm/simulators/wells/ParallelWBPCalculation.hpp>
 #include <opm/simulators/wells/ParallelWellInfo.hpp>
@@ -89,8 +97,8 @@ template<class Scalar> class WellContributions;
         /// Class for handling the blackoil well model.
         template<typename TypeTag>
         class BlackoilWellModel : public WellConnectionAuxiliaryModule<TypeTag, BlackoilWellModel<TypeTag>>
-                                , public BlackoilWellModelGeneric<GetPropType<TypeTag,
-                                                                              Properties::Scalar>>
+                                , public BlackoilWellModelGeneric<GetPropType<TypeTag, Properties::Scalar>,
+                                                                  typename GetPropType<TypeTag, Properties::FluidSystem>::IndexTraitsType>
         {
         public:
             // ---------      Types      ---------
@@ -108,6 +116,8 @@ template<class Scalar> class WellContributions;
 
             using WellConnectionModule = WellConnectionAuxiliaryModule<TypeTag, BlackoilWellModel<TypeTag>>;
 
+            using IndexTraits = typename FluidSystem::IndexTraitsType;
+
             constexpr static std::size_t pressureVarIndex = GetPropType<TypeTag, Properties::Indices>::pressureSwitchIdx;
 
             static const int numEq = Indices::numEq;
@@ -115,7 +125,7 @@ template<class Scalar> class WellContributions;
             static constexpr bool has_solvent_ = getPropValue<TypeTag, Properties::EnableSolvent>();
             static constexpr bool has_polymer_ = getPropValue<TypeTag, Properties::EnablePolymer>();
             static constexpr bool has_energy_ = getPropValue<TypeTag, Properties::EnableEnergy>();
-            static constexpr bool has_micp_ = getPropValue<TypeTag, Properties::EnableMICP>();
+            static constexpr bool has_micp_ = Indices::enableMICP;
 
             // TODO: where we should put these types, WellInterface or Well Model?
             // or there is some other strategy, like TypeTag
@@ -123,7 +133,7 @@ template<class Scalar> class WellContributions;
             using BVector = Dune::BlockVector<VectorBlockType>;
 
             using PolymerModule = BlackOilPolymerModule<TypeTag>;
-            using MICPModule = BlackOilMICPModule<TypeTag>;
+            using BioeffectsModule = BlackOilBioeffectsModule<TypeTag>;
 
             // For the conversion between the surface volume rate and reservoir voidage rate
             using RateConverterType = RateConverter::
@@ -133,6 +143,8 @@ template<class Scalar> class WellContributions;
             using AverageRegionalPressureType = RegionAverageCalculator::
                 AverageRegionalPressure<FluidSystem, std::vector<int> >;
 
+            using WellGroupHelpersType = WellGroupHelpers<Scalar, IndexTraits>;
+
             explicit BlackoilWellModel(Simulator& simulator);
 
             void init();
@@ -179,58 +191,56 @@ template<class Scalar> class WellContributions;
 
             using WellInterfacePtr = std::shared_ptr<WellInterface<TypeTag> >;
 
-            using BlackoilWellModelGeneric<Scalar>::initFromRestartFile;
+            using BlackoilWellModelGeneric<Scalar, IndexTraits>::initFromRestartFile;
             void initFromRestartFile(const RestartValue& restartValues)
             {
                 initFromRestartFile(restartValues,
                                     this->simulator_.vanguard().transferWTestState(),
                                     grid().size(0),
-                                    param_.use_multisegment_well_);
+                                    param_.use_multisegment_well_,
+                                    this->simulator_.vanguard().enableDistributedWells());
             }
 
-            using BlackoilWellModelGeneric<Scalar>::prepareDeserialize;
+            using BlackoilWellModelGeneric<Scalar, IndexTraits>::prepareDeserialize;
             void prepareDeserialize(const int report_step)
             {
                 prepareDeserialize(report_step, grid().size(0),
-                                   param_.use_multisegment_well_);
+                                   param_.use_multisegment_well_,
+                                   this->simulator_.vanguard().enableDistributedWells());
             }
 
             data::Wells wellData() const
             {
                 auto wsrpt = this->wellState()
-                    .report(simulator_.vanguard().globalCell().data(),
-                            [this](const int well_index) -> bool
+                    .report(this->simulator_.vanguard().globalCell().data(),
+                            [this](const int well_index)
                 {
                     return this->wasDynamicallyShutThisTimeStep(well_index);
                 });
 
-                {
-                    const auto& tracerRates = this->simulator_.problem()
-                        .tracerModel().getWellTracerRates();
-                    const auto& freeTracerRates = simulator_.problem()
-                        .tracerModel().getWellFreeTracerRates();
-                    const auto& solTracerRates = simulator_.problem()
-                        .tracerModel().getWellSolTracerRates();
-                    const auto& mswTracerRates = simulator_.problem()
-                        .tracerModel().getMswTracerRates();
-
-                    this->assignWellTracerRates(wsrpt, tracerRates, this->reportStepIndex());
-                    this->assignWellTracerRates(wsrpt, freeTracerRates, this->reportStepIndex());
-                    this->assignWellTracerRates(wsrpt, solTracerRates, this->reportStepIndex());
-                    this->assignMswTracerRates(wsrpt, mswTracerRates, this->reportStepIndex());
-                }
-
                 BlackoilWellModelGuideRates(*this)
                     .assignWellGuideRates(wsrpt, this->reportStepIndex());
 
-                this->assignWellTargets(wsrpt);
-                this->assignShutConnections(wsrpt, this->reportStepIndex());
-                // only used to compute gas injection mass rates for CO2STORE/H2STORE runs
-                // The gas reference density is thus the same for all pvt regions
-                // We therefore for simplicity use 0 here 
-                if (eclState().runspec().co2Storage() || eclState().runspec().h2Storage()) {
+                this->assignWellTracerRates(wsrpt);
+
+                if (const auto& rspec = eclState().runspec();
+                    rspec.co2Storage() || rspec.h2Storage())
+                {
+                    // The gas reference density (surface condition) is the
+                    // same for all PVT regions in CO2STORE/H2STORE runs so,
+                    // for simplicity, we use region zero (0) here.
+
                     this->assignMassGasRate(wsrpt, FluidSystem::referenceDensity(FluidSystem::gasPhaseIdx, 0));
                 }
+
+                this->assignWellTargets(wsrpt);
+
+                this->assignDynamicWellStatus(wsrpt);
+
+                // Assigning (a subset of the) property values in shut
+                // connections should be the last step of wellData().
+                this->assignShutConnections(wsrpt, this->reportStepIndex());
+
                 return wsrpt;
             }
 
@@ -245,7 +255,7 @@ template<class Scalar> class WellContributions;
 #endif
 
             // Check if well equations is converged.
-            ConvergenceReport getWellConvergence(const std::vector<Scalar>& B_avg, const bool checkWellGroupControls = false) const;
+            ConvergenceReport getWellConvergence(const std::vector<Scalar>& B_avg, const bool checkWellGroupControlsAndNetwork = false) const;
 
             const SimulatorReportSingle& lastReport() const;
 
@@ -268,8 +278,12 @@ template<class Scalar> class WellContributions;
             // at the beginning of each time step (Not report step)
             void prepareTimeStep(DeferredLogger& deferred_logger);
 
-            std::tuple<bool, bool, Scalar>
-            updateWellControls(const bool mandatory_network_balance, DeferredLogger& deferred_logger, const bool relax_network_tolerance = false);
+            bool
+            updateWellControls(DeferredLogger& deferred_logger);
+
+            std::tuple<bool, Scalar>
+            updateNetworks(const bool mandatory_network_balance, DeferredLogger& deferred_logger, const bool relax_network_tolerance = false);
+
 
             void updateAndCommunicate(const int reportStepIdx,
                                       const int iterationIdx,
@@ -340,6 +354,11 @@ template<class Scalar> class WellContributions;
                 return simulator_.vanguard().compressedIndexForInterior(cartesian_cell_idx);
             }
 
+            int compressedIndexForInteriorLGR(const std::string& lgr_tag, const Connection& conn) const override
+            {
+                return simulator_.vanguard().compressedIndexForInteriorLGR(lgr_tag, conn);
+            }
+
             // using the solution x to recover the solution xw for wells and applying
             // xw to update Well State
             void recoverWellSolutionAndUpdateWellState(const BVector& x);
@@ -358,6 +377,28 @@ template<class Scalar> class WellContributions;
             void setNlddAdapter(BlackoilWellModelNldd<TypeTag>* mod)
             { nldd_ = mod; }
 
+#ifdef RESERVOIR_COUPLING_ENABLED
+            ReservoirCouplingMaster& reservoirCouplingMaster() {
+                return *(this->simulator_.reservoirCouplingMaster());
+            }
+            ReservoirCouplingSlave& reservoirCouplingSlave() {
+                return *(this->simulator_.reservoirCouplingSlave());
+            }
+            bool isReservoirCouplingMaster() const {
+                return this->simulator_.reservoirCouplingMaster() != nullptr;
+            }
+            bool isReservoirCouplingSlave() const {
+                return this->simulator_.reservoirCouplingSlave() != nullptr;
+            }
+            void setReservoirCouplingMaster(ReservoirCouplingMaster *master)
+            {
+                this->guide_rate_handler_.setReservoirCouplingMaster(master);
+            }
+            void setReservoirCouplingSlave(ReservoirCouplingSlave *slave)
+            {
+                this->guide_rate_handler_.setReservoirCouplingSlave(slave);
+            }
+        #endif
         protected:
             Simulator& simulator_;
 
@@ -396,6 +437,10 @@ template<class Scalar> class WellContributions;
             std::map<std::string, std::unique_ptr<AverageRegionalPressureType>> regionalAveragePressureCalculator_{};
 
             SimulatorReportSingle last_report_{};
+            GuideRateHandler<Scalar, IndexTraits> guide_rate_handler_{};
+
+            // A flag to tell the convergence report whether we need to take another newton step
+            bool network_needs_more_balancing_force_another_newton_iteration_{false};
 
             // Pre-step network solve at static reservoir conditions (group and well states might be updated)
             void doPreStepNetworkRebalance(DeferredLogger& deferred_logger);
@@ -457,7 +502,7 @@ template<class Scalar> class WellContributions;
             void updateAverageFormationFactor();
 
             void computePotentials(const std::size_t widx,
-                                   const WellState<Scalar>& well_state_copy,
+                                   const WellState<Scalar, IndexTraits>& well_state_copy,
                                    std::string& exc_msg,
                                    ExceptionType::ExcEnum& exc_type,
                                    DeferredLogger& deferred_logger) override;
@@ -469,8 +514,8 @@ template<class Scalar> class WellContributions;
             void calculateProductivityIndexValues(const WellInterface<TypeTag>* wellPtr,
                                                   DeferredLogger& deferred_logger);
 
-            // The number of components in the model.
-            int numComponents() const;
+            // The number of conservation quantities.
+            int numConservationQuantities() const;
 
             int reportStepIndex() const;
 
@@ -486,7 +531,7 @@ template<class Scalar> class WellContributions;
             void extractLegacyDepth_();
 
             /// upate the wellTestState related to economic limits
-            void updateWellTestState(const double& simulationTime, WellTestState& wellTestState) const;
+            void updateWellTestState(const double simulationTime, WellTestState& wellTestState);
 
             void wellTesting(const int timeStepIdx, const double simulationTime, DeferredLogger& deferred_logger);
 
@@ -502,8 +547,6 @@ template<class Scalar> class WellContributions;
             void computeWellTemperature();
 
         private:
-            BlackoilWellModel(Simulator& simulator, const PhaseUsage& pu);
-
             BlackoilWellModelGasLift<TypeTag> gaslift_;
             BlackoilWellModelNldd<TypeTag>* nldd_ = nullptr; //!< NLDD well model adapter (not owned)
 
@@ -512,11 +555,12 @@ template<class Scalar> class WellContributions;
             // Their state is not relevant between function calls, so they can
             // (and must) be mutable, as the functions using them are const.
             mutable BVector x_local_;
-        };
 
+            void assignWellTracerRates(data::Wells& wsrpt) const;
+        };
 
 } // namespace Opm
 
 #include "BlackoilWellModel_impl.hpp"
 
-#endif
+#endif // OPM_BLACKOILWELLMODEL_HEADER_INCLUDED
