@@ -36,12 +36,14 @@
 #include <opm/simulators/wells/ConnectionIndexMap.hpp>
 #include <opm/simulators/wells/ParallelPAvgDynamicSourceData.hpp>
 #include <opm/simulators/wells/ParallelWBPCalculation.hpp>
+#include <opm/simulators/wells/ParallelWellInfo.hpp>
 #include <opm/simulators/wells/PerforationData.hpp>
 #include <opm/simulators/wells/WellFilterCake.hpp>
 #include <opm/simulators/wells/WellProdIndexCalculator.hpp>
 #include <opm/simulators/wells/WellTracerRate.hpp>
 #include <opm/simulators/wells/WGState.hpp>
 
+#include <algorithm>
 #include <cstddef>
 #include <functional>
 #include <map>
@@ -52,23 +54,24 @@
 #include <unordered_set>
 #include <vector>
 
+
 namespace Opm {
     class DeferredLogger;
     class EclipseState;
-    template<class Scalar> class BlackoilWellModelGasLiftGeneric;
-    template<class Scalar> class GasLiftGroupInfo;
-    template<class Scalar> class GasLiftSingleWellGeneric;
+    template<typename Scalar, typename IndexTraits> class BlackoilWellModelGasLiftGeneric;
+    template<typename Scalar, typename IndexTraits> class GasLiftGroupInfo;
+    template<typename Scalar, typename IndexTraits> class GasLiftSingleWellGeneric;
     template<class Scalar> class GasLiftWellState;
     class Group;
     class GuideRateConfig;
-    template<class Scalar> class ParallelWellInfo;
     class RestartValue;
     class Schedule;
     struct SimulatorUpdate;
     class SummaryConfig;
-    template<class Scalar> class VFPProperties;
-    template<class Scalar> class WellInterfaceGeneric;
-    template<class Scalar> class WellState;
+    template<typename Scalar, typename IndexTraits> class VFPProperties;
+    template<typename Scalar, typename IndexTraits> class WellGroupHelpers;
+    template<typename Scalar, typename IndexTraits> class WellInterfaceGeneric;
+    template<typename Scalar, typename IndexTraits> class WellState;
 } // namespace Opm
 
 namespace Opm { namespace data {
@@ -87,18 +90,24 @@ struct EnableTerminalOutput { static constexpr bool value = true; };
 namespace Opm {
 
 /// Class for handling the blackoil well model.
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 class BlackoilWellModelGeneric
 {
+    using WellGroupHelpersType =  WellGroupHelpers<Scalar, IndexTraits>;
 public:
     BlackoilWellModelGeneric(Schedule& schedule,
-                             BlackoilWellModelGasLiftGeneric<Scalar>& gaslift,
+                             BlackoilWellModelGasLiftGeneric<Scalar, IndexTraits>& gaslift,
                              const SummaryState& summaryState,
                              const EclipseState& eclState,
-                             const PhaseUsage& phase_usage,
+                             const PhaseUsageInfo<IndexTraits>& phase_usage,
                              const Parallel::Communication& comm);
 
     virtual ~BlackoilWellModelGeneric() = default;
+    virtual int compressedIndexForInteriorLGR([[maybe_unused]] const std::string& lgr_tag,
+                                              [[maybe_unused]] const Connection& conn) const
+    {
+        throw std::runtime_error("compressedIndexForInteriorLGR not implemented");
+    }
 
     int numLocalWells() const;
     int numLocalWellsEnd() const;
@@ -129,18 +138,18 @@ public:
     const Well& getWellEcl(const std::string& well_name) const;
     std::vector<Well> getLocalWells(const int timeStepIdx) const;
     const Schedule& schedule() const { return schedule_; }
-    const PhaseUsage& phaseUsage() const { return phase_usage_; }
+    const PhaseUsageInfo<IndexTraits>& phaseUsage() const { return phase_usage_info_; }
     const GroupState<Scalar>& groupState() const { return this->active_wgstate_.group_state; }
-    std::vector<const WellInterfaceGeneric<Scalar>*> genericWells() const
+    std::vector<const WellInterfaceGeneric<Scalar, IndexTraits>*> genericWells() const
     { return {well_container_generic_.begin(), well_container_generic_.end()}; }
 
-    std::vector<WellInterfaceGeneric<Scalar>*> genericWells()
+    std::vector<WellInterfaceGeneric<Scalar, IndexTraits>*> genericWells()
     { return well_container_generic_; }
 
     /*
       Immutable version of the currently active wellstate.
     */
-    const WellState<Scalar>& wellState() const
+    const WellState<Scalar, IndexTraits>& wellState() const
     {
         return this->active_wgstate_.well_state;
     }
@@ -148,7 +157,7 @@ public:
     /*
       Mutable version of the currently active wellstate.
     */
-    WellState<Scalar>& wellState()
+    WellState<Scalar, IndexTraits>& wellState()
     {
         return this->active_wgstate_.well_state;
     }
@@ -157,7 +166,7 @@ public:
       Will return the currently active nupcolWellState; must update
       the internal nupcol wellstate with updateNupcolWGState() first.
     */
-    const WellState<Scalar>& nupcolWellState() const
+    const WellState<Scalar, IndexTraits>& nupcolWellState() const
     {
         return this->nupcol_wgstate_.well_state;
     }
@@ -177,11 +186,13 @@ public:
     void initFromRestartFile(const RestartValue& restartValues,
                              std::unique_ptr<WellTestState> wtestState,
                              const std::size_t numCells,
-                             bool handle_ms_well);
+                             bool handle_ms_well,
+                             bool enable_distributed_wells);
 
     void prepareDeserialize(int report_step,
                             const std::size_t numCells,
-                            bool handle_ms_well);
+                            bool handle_ms_well,
+                            bool enable_distributed_wells);
 
     /*
       Will assign the internal member last_valid_well_state_ to the
@@ -192,6 +203,7 @@ public:
     void commitWGState()
     {
         this->last_valid_wgstate_ = this->active_wgstate_;
+        this->last_valid_node_pressures_ = this->node_pressures_;
     }
 
     data::GroupAndNetworkValues groupAndNetworkData(const int reportStepIdx) const;
@@ -220,6 +232,7 @@ public:
     const SummaryState& summaryState() const { return summaryState_; }
 
     const GuideRate& guideRate() const { return guideRate_; }
+    GuideRate& guideRate() { return guideRate_; }
 
     const std::map<std::string, double>& wellOpenTimes() const { return well_open_times_; }
     const std::map<std::string, double>& wellCloseTimes() const { return well_close_times_; }
@@ -250,6 +263,7 @@ public:
         serializer(closed_this_step_);
         serializer(guideRate_);
         serializer(node_pressures_);
+        serializer(last_valid_node_pressures_);
         serializer(prev_inj_multipliers_);
         serializer(active_wgstate_);
         serializer(last_valid_wgstate_);
@@ -266,6 +280,18 @@ public:
     parallelWellInfo(const std::size_t idx) const
     { return local_parallel_well_info_[idx].get(); }
 
+    bool isOwner(const std::string& wname) const
+    {
+        return this->parallelWellSatisfies
+            (wname, [](const auto& pwInfo) { return pwInfo.isOwner(); });
+    }
+
+    bool hasLocalCells(const std::string& wname) const
+    {
+        return this->parallelWellSatisfies
+            (wname, [](const auto& pwInfo) { return pwInfo.hasLocalCells(); });
+    }
+
     const ConnectionIndexMap& connectionIndexMap(const std::size_t idx)
     { return conn_idx_map_[idx]; }
 
@@ -297,12 +323,12 @@ protected:
       prevWellState() must have been stored with the commitWGState()
       function first.
     */
-    const WellState<Scalar>& prevWellState() const
+    const WellState<Scalar, IndexTraits>& prevWellState() const
     {
         return this->last_valid_wgstate_.well_state;
     }
 
-    const WGState<Scalar>& prevWGState() const
+    const WGState<Scalar, IndexTraits>& prevWGState() const
     {
         return this->last_valid_wgstate_;
     }
@@ -312,7 +338,7 @@ protected:
       last_valid_well_state_ member, that state can then be recovered
       with a subsequent call to resetWellState().
     */
-    void commitWGState(WGState<Scalar> wgstate)
+    void commitWGState(WGState<Scalar, IndexTraits> wgstate)
     {
         this->last_valid_wgstate_ = std::move(wgstate);
     }
@@ -325,6 +351,7 @@ protected:
     void resetWGState()
     {
         this->active_wgstate_ = this->last_valid_wgstate_;
+        this->node_pressures_ = this->last_valid_node_pressures_;
     }
 
     /*
@@ -340,7 +367,7 @@ protected:
     void reportGroupSwitching(DeferredLogger& local_deferredLogger) const;
 
     /// \brief Create the parallel well information
-    /// \param localWells The local wells from ECL schedule
+    /// \param wells The local wells from ECL schedule
     std::vector<std::reference_wrapper<ParallelWellInfo<Scalar>>>
     createLocalParallelWellInfo(const std::vector<Well>& wells);
 
@@ -355,7 +382,7 @@ protected:
 
     void updateWsolvent(const Group& group,
                         const int reportStepIdx,
-                        const WellState<Scalar>& wellState);
+                        const WellState<Scalar, IndexTraits>& wellState);
     void setWsolvent(const Group& group,
                      const int reportStepIdx,
                      Scalar wsolvent);
@@ -367,11 +394,30 @@ protected:
                                   const int pvtreg,
                                   std::vector<Scalar>& resv_coeff) = 0;
 
+    /// Assign dynamic well status for each well owned by current rank
+    ///
+    /// \param[in,out] wsrpt Well solution object.  On exit, holds current
+    /// values for \code data::Well::dynamicStatus \endcode.
+    void assignDynamicWellStatus(data::Wells& wsrpt) const;
+
+    /// Assign basic result quantities for shut connections of wells owned
+    /// by current rank.
+    ///
+    /// Mostly provided for summary file output purposes.  Applies to fully
+    /// shut/stopped wells and shut connections of open/flowing wells.
+    ///
+    /// \param[in,out] wsrpt Well solution object.  On exit, also contains a
+    /// few quantities, like the D factor, the Kh product and the CTF, for
+    /// shut connections.
+    ///
+    /// \param[in] reportStepIndex Zero-based index of current report step.
     void assignShutConnections(data::Wells& wsrpt,
                                const int reportStepIndex) const;
+
     void assignWellTargets(data::Wells& wsrpt) const;
     void assignProductionWellTargets(const Well& well, data::WellControlLimits& limits) const;
     void assignInjectionWellTargets(const Well& well, data::WellControlLimits& limits) const;
+
     void assignGroupControl(const Group& group,
                             data::GroupData& gdata) const;
     void assignGroupValues(const int reportStepIdx,
@@ -382,7 +428,7 @@ protected:
     void calculateEfficiencyFactors(const int reportStepIdx);
 
     void checkGconsaleLimits(const Group& group,
-                             WellState<Scalar>& well_state,
+                             WellState<Scalar, IndexTraits>& well_state,
                              const int reportStepIdx,
                              DeferredLogger& deferred_logger);
 
@@ -394,11 +440,13 @@ protected:
     bool checkGroupHigherConstraints(const Group& group,
                                      DeferredLogger& deferred_logger,
                                      const int reportStepIdx,
-                                     const int max_number_of_group_switch);
+                                     const int max_number_of_group_switch,
+                                     const bool update_group_switching_log);
 
     void updateAndCommunicateGroupData(const int reportStepIdx,
                                        const int iterationIdx,
                                        const Scalar tol_nupcol,
+                                       const bool update_wellgrouptarget, // we only want to update the wellgrouptarget after the groups have found their controls
                                        DeferredLogger& deferred_logger);
 
     void inferLocalShutWells();
@@ -406,7 +454,7 @@ protected:
     void setRepRadiusPerfLength();
 
     virtual void computePotentials(const std::size_t widx,
-                                   const WellState<Scalar>& well_state_copy,
+                                   const WellState<Scalar, IndexTraits>& well_state_copy,
                                    std::string& exc_msg,
                                    ExceptionType::ExcEnum& exc_type,
                                    DeferredLogger& deferred_logger) = 0;
@@ -458,16 +506,18 @@ protected:
                               const unsigned reportStep) const;
 
     void assignMassGasRate(data::Wells& wsrpt,
-                           const Scalar& gasDensity) const;
+                           const Scalar gasDensity) const;
 
     Schedule& schedule_;
+
     const SummaryState& summaryState_;
     const EclipseState& eclState_;
     const Parallel::Communication& comm_;
-    BlackoilWellModelGasLiftGeneric<Scalar>& gen_gaslift_;
-    BlackoilWellModelWBP<Scalar> wbp_;
+    BlackoilWellModelGasLiftGeneric<Scalar, IndexTraits>& gen_gaslift_;
+    BlackoilWellModelWBP<Scalar, IndexTraits> wbp_;
 
-    PhaseUsage phase_usage_;
+
+    const PhaseUsageInfo<IndexTraits>& phase_usage_info_;
     bool terminal_output_{false};
     bool wells_active_{false};
     bool network_active_{false};
@@ -486,10 +536,10 @@ protected:
     std::map<std::string, double> well_close_times_;
 
     std::vector<ConnectionIndexMap> conn_idx_map_{};
-    std::function<bool(const Well&)> not_on_process_{};
+    std::function<bool(const std::string&)> not_on_process_{};
 
     // a vector of all the wells.
-    std::vector<WellInterfaceGeneric<Scalar>*> well_container_generic_{};
+    std::vector<WellInterfaceGeneric<Scalar, IndexTraits>*> well_container_generic_{};
 
     std::vector<int> local_shut_wells_{};
 
@@ -503,14 +553,18 @@ protected:
     mutable std::unordered_set<std::string> closed_this_step_;
 
     GuideRate guideRate_;
-    std::unique_ptr<VFPProperties<Scalar>> vfp_properties_{};
-    std::map<std::string, Scalar> node_pressures_; // Storing network pressures for output.
+    std::unique_ptr<VFPProperties<Scalar, IndexTraits>> vfp_properties_{};
+
+    // Network pressures for output and initialization
+    std::map<std::string, Scalar> node_pressures_;
+    // Valid network pressures for output and initialization for safe restart after failed iterations
+    std::map<std::string, Scalar> last_valid_node_pressures_;
 
     // previous injection multiplier, it is used in the injection multiplier calculation for WINJMULT keyword
     std::unordered_map<std::string, std::vector<Scalar>> prev_inj_multipliers_;
 
     // Handling for filter cake injection multipliers
-    std::unordered_map<std::string, WellFilterCake<Scalar>> filter_cake_;
+    std::unordered_map<std::string, WellFilterCake<Scalar, IndexTraits>> filter_cake_;
 
     /*
       The various wellState members should be accessed and modified
@@ -518,9 +572,9 @@ protected:
       commitWellState(), resetWellState(), nupcolWellState() and
       updateNupcolWGState().
     */
-    WGState<Scalar> active_wgstate_;
-    WGState<Scalar> last_valid_wgstate_;
-    WGState<Scalar> nupcol_wgstate_;
+    WGState<Scalar, IndexTraits> active_wgstate_;
+    WGState<Scalar, IndexTraits> last_valid_wgstate_;
+    WGState<Scalar, IndexTraits> nupcol_wgstate_;
     WellGroupEvents report_step_start_events_; //!< Well group events at start of report step
 
     bool wellStructureChangedDynamically_{false};
@@ -532,7 +586,7 @@ protected:
     std::map<std::string, std::pair<std::string, std::string>> closed_offending_wells_;
 
 private:
-    WellInterfaceGeneric<Scalar>* getGenWell(const std::string& well_name);
+    WellInterfaceGeneric<Scalar, IndexTraits>* getGenWell(const std::string& well_name);
 
     template <typename Iter, typename Body>
     void wellUpdateLoop(Iter first, Iter last, const int timeStepIdx, Body&& body);
@@ -543,8 +597,50 @@ private:
 
     void updateEclWellsCTFFromAction(const int timeStepIdx,
                                      const SimulatorUpdate& sim_update);
-};
 
+    /// Check if a well satisfies a particular MPI condition
+    ///
+    /// Common conditions in this context are whether or not a named well is
+    /// owned by the current rank or has connections on the current rank.
+    ///
+    /// \tparam Predicate Condition predicate function type.
+    ///
+    /// \param[in] wname Well name.
+    ///
+    /// \param[in] p Predicate function implementing the specific condition.
+    ///
+    /// \return Whether or not \p wname is a parallel well and, if so, if it
+    /// satisfies the condition \p p.
+    template <typename Predicate>
+    bool parallelWellSatisfies(const std::string& wname, Predicate&& p) const
+    {
+        auto pwInfoPos = std::find_if(this->parallel_well_info_.begin(),
+                                      this->parallel_well_info_.end(),
+                                      [&wname](const auto& pwInfo)
+                                      { return pwInfo.name() == wname; });
+
+        return (pwInfoPos != this->parallel_well_info_.end())
+            && p(*pwInfoPos);
+    }
+
+    /// Run caller-defined code for each well owned by current rank
+    ///
+    /// 'const' version.
+    ///
+    /// \tparam LoopBody Call-back type for user-defined code.  Expected to
+    /// support a function call operator of the form
+    /// \code
+    ///   void operator()(const std::size_t i, const Well& well) const
+    /// \endcode
+    /// which will be invoked for each well owned by the local process.  The
+    /// parameters are the index into \c wells_ecl_ and the object at that
+    /// index, respectively.
+    ///
+    /// \param[in] loopBody Call-back function.  Typically defined as a
+    /// lambda expression.
+    template <typename LoopBody>
+    void loopOwnedWells(LoopBody&& loopBody) const;
+};
 
 } // namespace Opm
 
