@@ -29,6 +29,8 @@
 #include <opm/output/data/Groups.hpp>
 #include <opm/output/data/GuideRateValue.hpp>
 
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
+
 #include <opm/simulators/wells/BlackoilWellModelGeneric.hpp>
 #include <opm/simulators/wells/WellGroupHelpers.hpp>
 #include <opm/simulators/wells/WellInterfaceGeneric.hpp>
@@ -283,8 +285,8 @@ const Opm::Well& GroupTreeWalker::getWell(std::string_view well) const
 
 namespace Opm {
 
-template<class Scalar>
-void BlackoilWellModelGuideRates<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 getGuideRateValues(const GuideRate::RateVector& qs,
                    const bool                   is_inj,
                    const std::string&           wgname,
@@ -309,9 +311,9 @@ getGuideRateValues(const GuideRate::RateVector& qs,
     }
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 data::GuideRateValue
-BlackoilWellModelGuideRates<Scalar>::
+BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 getGuideRateValues(const Well& well) const
 {
     auto grval = data::GuideRateValue{};
@@ -329,17 +331,17 @@ getGuideRateValues(const Well& well) const
         return grval;
     }
 
-    const auto qs = WellGroupHelpers<Scalar>::
-        getWellRateVector(wellModel_.wellState(), wellModel_.phaseUsage(), wname);
+    const auto qs = WellGroupHelpers<Scalar, IndexTraits>::
+        getWellRateVector(wellModel_.wellState(), wname);
 
     this->getGuideRateValues(qs, well.isInjector(), wname, grval);
 
     return grval;
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 data::GuideRateValue
-BlackoilWellModelGuideRates<Scalar>::
+BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 getGuideRateValues(const Group& group) const
 {
     auto grval = data::GuideRateValue{};
@@ -357,7 +359,7 @@ getGuideRateValues(const Group& group) const
         return grval;
     }
 
-    const auto qs = WellGroupHelpers<Scalar>::
+    const auto qs = WellGroupHelpers<Scalar, IndexTraits>::
         getProductionGroupRateVector(wellModel_.groupState(), wellModel_.phaseUsage(), gname);
 
     const auto is_inj = false; // This procedure only applies to G*PGR.
@@ -366,9 +368,9 @@ getGuideRateValues(const Group& group) const
     return grval;
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 data::GuideRateValue
-BlackoilWellModelGuideRates<Scalar>::
+BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 getGuideRateInjectionGroupValues(const Group& group) const
 {
     auto grval = data::GuideRateValue{};
@@ -387,10 +389,10 @@ getGuideRateInjectionGroupValues(const Group& group) const
     return grval;
 }
 
-template<class Scalar>
-void BlackoilWellModelGuideRates<Scalar>::
-assignWellGuideRates(data::Wells& wsrpt,
-                     const int    reportStepIdx) const
+template<typename Scalar, typename IndexTraits>
+std::unordered_map<std::string, data::GuideRateValue>
+BlackoilWellModelGuideRates<Scalar, IndexTraits>::
+calculateWellGuideRates(const int reportStepIdx) const
 {
     auto all = std::unordered_map<std::string, data::GuideRateValue>{};
     auto retrieve = std::unordered_map<std::string, RetrieveWellGuideRate>{};
@@ -437,8 +439,8 @@ assignWellGuideRates(data::Wells& wsrpt,
         const auto get_gr = parent
             || RetrieveWellGuideRate{wellModel_.guideRate(), wname};
 
-        const auto qs = WellGroupHelpers<Scalar>::
-            getWellRateVector(wellModel_.wellState(), wellModel_.phaseUsage(), wname);
+        const auto qs = WellGroupHelpers<Scalar, IndexTraits>::
+            getWellRateVector(wellModel_.wellState(), wname);
 
         auto getGR = [this, &wname, &qs](const GuideRateModel::Target t)
         {
@@ -470,7 +472,16 @@ assignWellGuideRates(data::Wells& wsrpt,
     // Upon completion, 'all' contains guide rate values for all wells
     // reachable from 'FIELD' at this time/report step.
     walker.traversePreOrder();
+    return all;
+}
 
+template<typename Scalar, typename IndexTraits>
+void
+BlackoilWellModelGuideRates<Scalar, IndexTraits>::
+assignWellGuideRates(data::Wells& wsrpt,
+                     const int    reportStepIdx) const
+{
+    auto all = calculateWellGuideRates(reportStepIdx);
     for (const auto* well : wellModel_.genericWells()) {
         auto xwPos = wsrpt.find(well->name());
         if (xwPos == wsrpt.end()) { // No well results.  Unexpected.
@@ -486,9 +497,9 @@ assignWellGuideRates(data::Wells& wsrpt,
     }
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 std::unordered_map<std::string, data::GroupGuideRates>
-BlackoilWellModelGuideRates<Scalar>::
+BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 calculateAllGroupGuideRates(const int reportStepIdx) const
 {
     auto gr = std::unordered_map<std::string, data::GroupGuideRates>{};
@@ -547,8 +558,8 @@ calculateAllGroupGuideRates(const int reportStepIdx) const
     return gr;
 }
 
-template<class Scalar>
-void BlackoilWellModelGuideRates<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 assignGroupGuideRates(const Group& group,
                       const std::unordered_map<std::string, data::GroupGuideRates>& groupGuideRates,
                       data::GroupData& gdata) const
@@ -575,14 +586,14 @@ assignGroupGuideRates(const Group& group,
     }
 }
 
-template<class Scalar>
-bool BlackoilWellModelGuideRates<Scalar>::
+template<typename Scalar, typename IndexTraits>
+bool BlackoilWellModelGuideRates<Scalar, IndexTraits>::
 guideRateUpdateIsNeeded(const int reportStepIdx) const
 {
     const auto& genWells = wellModel_.genericWells();
     auto need_update =
     std::any_of(genWells.begin(), genWells.end(),
-                [](const WellInterfaceGeneric<Scalar>* well)
+                [](const WellInterfaceGeneric<Scalar, IndexTraits>* well)
     {
         return well->changedToOpenThisStep();
     });
@@ -594,7 +605,7 @@ guideRateUpdateIsNeeded(const int reportStepIdx) const
             + ScheduleEvents::NEW_WELL;
 
         need_update = std::any_of(genWells.begin(), genWells.end(),
-            [&events](const WellInterfaceGeneric<Scalar>* well)
+            [&events](const WellInterfaceGeneric<Scalar, IndexTraits>* well)
         {
             return events.hasEvent(well->name(), effective_events_mask);
         });
@@ -602,10 +613,10 @@ guideRateUpdateIsNeeded(const int reportStepIdx) const
     return wellModel_.comm().max(static_cast<int>(need_update));
 }
 
-template class BlackoilWellModelGuideRates<double>;
+template class BlackoilWellModelGuideRates<double, BlackOilDefaultFluidSystemIndices>;
 
 #if FLOW_INSTANTIATE_FLOAT
-template class BlackoilWellModelGuideRates<float>;
+template class BlackoilWellModelGuideRates<float, BlackOilDefaultFluidSystemIndices>;
 #endif
 
 } // namespace Opm
