@@ -25,8 +25,6 @@
 #include <opm/simulators/wells/GasLiftGroupInfo.hpp>
 #include <opm/simulators/wells/GasLiftCommon.hpp>
 
-#include <opm/simulators/utils/BlackoilPhases.hpp>
-
 #include <optional>
 #include <set>
 #include <stdexcept>
@@ -41,24 +39,24 @@ class GasLiftWell;
 template<class Scalar> class GasLiftWellState;
 class Schedule;
 class SummaryState;
-template<class Scalar> class WellInterfaceGeneric;
-template<class Scalar> class WellState;
+template<typename Scalar, typename IndexTraits> class WellInterfaceGeneric;
+template<typename Scalar, typename IndexTraits> class WellState;
 template<class Scalar> class GroupState;
 
-template<class Scalar>
-class GasLiftSingleWellGeneric : public GasLiftCommon<Scalar>
+template<typename Scalar, typename IndexTraits>
+class GasLiftSingleWellGeneric : public GasLiftCommon<Scalar, IndexTraits>
 {
 protected:
-    static constexpr int Water = BlackoilPhases::Aqua;
-    static constexpr int Oil = BlackoilPhases::Liquid;
-    static constexpr int Gas = BlackoilPhases::Vapour;
+    static constexpr int Water = IndexTraits::waterPhaseIdx;
+    static constexpr int Oil = IndexTraits::oilPhaseIdx;
+    static constexpr int Gas = IndexTraits::gasPhaseIdx;
     static constexpr int NUM_PHASES = 3;
     static constexpr Scalar ALQ_EPSILON = 1e-8;
 
 public:
     using GLiftSyncGroups = std::set<int>;
-    using Rate = typename GasLiftGroupInfo<Scalar>::Rate;
-    using MessageType = typename GasLiftCommon<Scalar>::MessageType;
+    using Rate = typename GasLiftGroupInfo<Scalar, IndexTraits>::Rate;
+    using MessageType = typename GasLiftCommon<Scalar, IndexTraits>::MessageType;
 
     struct GradInfo
     {
@@ -94,8 +92,6 @@ public:
         bool alq_is_limited;
     };
 
-    virtual ~GasLiftSingleWellGeneric() = default;
-
     const std::string& name() const { return well_name_; }
 
     std::optional<GradInfo> calcIncOrDecGradient(Scalar oil_rate,
@@ -108,16 +104,17 @@ public:
 
     std::unique_ptr<GasLiftWellState<Scalar>> runOptimize(const int iteration_idx);
 
-    virtual const WellInterfaceGeneric<Scalar>& getWell() const = 0;
+    std::pair<Scalar, bool> wellTestALQ();
+
+    virtual const WellInterfaceGeneric<Scalar, IndexTraits>& getWell() const = 0;
 
 protected:
     GasLiftSingleWellGeneric(DeferredLogger& deferred_logger,
-                             WellState<Scalar>& well_state,
+                             WellState<Scalar, IndexTraits>& well_state,
                              const GroupState<Scalar>& group_state,
                              const Well& ecl_well,
                              const SummaryState& summary_state,
-                             GasLiftGroupInfo<Scalar>& group_info,
-                             const PhaseUsage& phase_usage,
+                             GasLiftGroupInfo<Scalar, IndexTraits>& group_info,
                              const Schedule& schedule,
                              const int report_step_idx,
                              GLiftSyncGroups& sync_groups,
@@ -156,7 +153,7 @@ protected:
         // This copy constructor cannot be defined inline here since LimitedRates
         //   has not been defined yet (it is defined below). Instead it is defined in
         //   in the .cpp file
-        BasicRates(const LimitedRates& rates);
+        explicit BasicRates(const LimitedRates& rates);
 
         Scalar operator[](Rate rate_type) const
         {
@@ -412,8 +409,8 @@ protected:
                            Scalar delta_alq) const;
 
     LimitedRates
-    updateRatesToGroupLimits_(const BasicRates& rates,
-                              const LimitedRates& new_rates,
+    updateRatesToGroupLimits_(const BasicRates& old_rates,
+                              const LimitedRates& rates,
                               const std::string& gr_name = "") const;
 
     void updateWellStateAlqFixedValue_(const GasLiftWell& well);
@@ -427,8 +424,7 @@ protected:
 
     const Well& ecl_well_;
     const SummaryState& summary_state_;
-    GasLiftGroupInfo<Scalar>& group_info_;
-    const PhaseUsage& phase_usage_;
+    GasLiftGroupInfo<Scalar, IndexTraits>& group_info_;
     GLiftSyncGroups& sync_groups_;
     const WellProductionControls controls_;
 
