@@ -54,6 +54,8 @@ GroupState<Scalar> GroupState<Scalar>::serializationTestObject()
     result.injection_controls = {{{Phase::FOAM, "test12"}, Group::InjectionCMode::REIN}};
     result.gpmaint_state.add("foo", GPMaint::State::serializationTestObject());
     result.m_gconsump_rates = {{"testA", {0.2, 0.1}}};
+    result.m_number_of_wells_under_group_control = {{"test13", 3}};
+    result.m_number_of_wells_under_inj_group_control = {{ {Phase::WATER, "test14"}, 3}};
 
     return result;
 }
@@ -72,6 +74,8 @@ bool GroupState<Scalar>::operator==(const GroupState& other) const
            this->inj_surface_rates == other.inj_surface_rates &&
            this->m_grat_sales_target == other.m_grat_sales_target &&
            this->injection_controls == other.injection_controls &&
+           this->m_number_of_wells_under_group_control == m_number_of_wells_under_group_control &&
+           this->m_number_of_wells_under_inj_group_control == m_number_of_wells_under_inj_group_control &&
            this->gpmaint_state == other.gpmaint_state &&
            this->m_gconsump_rates == other.m_gconsump_rates;
 }
@@ -131,14 +135,14 @@ GroupState<Scalar>::network_leaf_node_production_rates(const std::string& gname)
 
 template<class Scalar>
 void GroupState<Scalar>::
-GroupState::update_well_group_thp(const std::string& gname, const double& thp) 
+GroupState::update_well_group_thp(const std::string& gname, const double& thp)
 {
     this->group_thp[gname] = thp;
 }
 
 template<class Scalar>
 Scalar GroupState<Scalar>::
-GroupState::well_group_thp(const std::string& gname) const 
+GroupState::well_group_thp(const std::string& gname) const
 {
     auto group_iter = this->group_thp.find(gname);
     if (group_iter == this->group_thp.end())
@@ -416,6 +420,48 @@ injection_control(const std::string& gname, Phase phase) const
     return group_iter->second;
 }
 
+
+
+//-------------------------------------------------------------------------
+
+template<class Scalar>
+void GroupState<Scalar>::
+update_number_of_wells_under_group_control(const std::string& gname, int number)
+{
+    this->m_number_of_wells_under_group_control[gname] = number;
+}
+template<class Scalar>
+int GroupState<Scalar>::
+number_of_wells_under_group_control(const std::string& gname) const
+{
+    auto group_iter = this->m_number_of_wells_under_group_control.find(gname);
+    if (group_iter == this->m_number_of_wells_under_group_control.end())
+        throw std::logic_error("No such group");
+
+    return group_iter->second;
+}
+
+//-------------------------------------------------------------------------
+
+
+template<class Scalar>
+void GroupState<Scalar>::
+update_number_of_wells_under_inj_group_control(const std::string& gname, Phase phase, int number)
+{
+    this->m_number_of_wells_under_inj_group_control[{phase, gname}] = number;
+}
+
+template<class Scalar>
+int GroupState<Scalar>::
+number_of_wells_under_inj_group_control(const std::string& gname, Phase phase) const
+{
+    auto group_iter = this->m_number_of_wells_under_inj_group_control.find({phase, gname});
+    if (group_iter == this->m_number_of_wells_under_inj_group_control.end())
+        throw std::logic_error("No such group");
+
+    return group_iter->second;
+}
+
 //-------------------------------------------------------------------------
 
 template<class Scalar>
@@ -509,6 +555,26 @@ gconsump_rates(const std::string& gname) const {
     return zero_pair;
 }
 
+template<class Scalar>
+void GroupState<Scalar>::
+update_group_production_potential(const std::string& gname, Scalar oil_rate,
+                                  Scalar gas_rate, Scalar water_rate)
+{
+    auto [it, inserted] = production_group_potentials.try_emplace(
+        gname, oil_rate, gas_rate, water_rate
+    );
+    if (!inserted) {
+        it->second = GroupPotential{oil_rate, gas_rate, water_rate};
+    }
+}
+
+template<class Scalar>
+const typename GroupState<Scalar>::GroupPotential&
+GroupState<Scalar>::
+get_production_group_potential(const std::string& gname) const
+{
+    return this->production_group_potentials.at(gname);
+}
 
 template class GroupState<double>;
 
