@@ -39,15 +39,15 @@ template<class M> class UMFPack;
 namespace Opm
 {
 
-template<class Scalar, int numWellEq, int numEq> class MultisegmentWellEquationAccess;
-template<class Scalar> class MultisegmentWellGeneric;
+template<class Scalar, typename IndexTraits, int numWellEq, int numEq> class MultisegmentWellEquationAccess;
+template<class Scalar, typename IndexTraits> class MultisegmentWellGeneric;
 #if COMPILE_GPU_BRIDGE
 template<class Scalar> class WellContributions;
 #endif
-template<class Scalar> class WellInterfaceGeneric;
-template<class Scalar> class WellState;
+template<typename Scalar, typename IndexTraits> class WellInterfaceGeneric;
+template<typename Scalar, typename IndexTraits> class WellState;
 
-template<class Scalar, int numWellEq, int numEq>
+template<class Scalar, typename IndexTraits, int numWellEq, int numEq>
 class MultisegmentWellEquations
 {
 public:
@@ -70,7 +70,7 @@ public:
     using OffDiagMatrixBlockWellType = Dune::FieldMatrix<Scalar,numWellEq,numEq>;
     using OffDiagMatWell = Dune::BCRSMatrix<OffDiagMatrixBlockWellType>;
 
-    MultisegmentWellEquations(const MultisegmentWellGeneric<Scalar>& well, const ParallelWellInfo<Scalar>& pw_info);
+    MultisegmentWellEquations(const MultisegmentWellGeneric<Scalar, IndexTraits>& well, const ParallelWellInfo<Scalar>& pw_info);
 
     //! \brief Setup sparsity pattern for the matrices.
     //! \param numPerfs Number of perforations
@@ -119,9 +119,9 @@ public:
                                   const BVector& weights,
                                   const int pressureVarIndex,
                                   const bool /*use_well_weights*/,
-                                  const WellInterfaceGeneric<Scalar>& well,
+                                  const WellInterfaceGeneric<Scalar, IndexTraits>& well,
                                   const int seg_pressure_var_ind,
-                                  const WellState<Scalar>& well_state) const;
+                                  const WellState<Scalar, IndexTraits>& well_state) const;
 
     //! \brief Sum with off-process contribution.
     void sumDistributed(Parallel::Communication comm);
@@ -133,7 +133,7 @@ public:
     }
 
   private:
-    friend class MultisegmentWellEquationAccess<Scalar,numWellEq,numEq>;
+    friend class MultisegmentWellEquationAccess<Scalar,IndexTraits,numWellEq,numEq>;
     // two off-diagonal matrices
     OffDiagMatWell duneB_;
     OffDiagMatWell duneC_;
@@ -148,7 +148,7 @@ public:
     // residuals of the well equations
     BVectorWell resWell_;
 
-    const MultisegmentWellGeneric<Scalar>& well_; //!< Reference to well
+    const MultisegmentWellGeneric<Scalar, IndexTraits>& well_; //!< Reference to well
 
     // Store the global index of well perforated cells
     std::vector<int> cells_;
