@@ -24,10 +24,10 @@
 
 #include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
 
-#include <opm/material/fluidsystems/BlackOilDefaultIndexTraits.hpp>
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
 #include <opm/material/fluidsystems/BlackOilFluidSystem.hpp>
 
-#include <opm/models/blackoil/blackoilindices.hh>
+#include <opm/models/blackoil/blackoilvariableandequationindices.hh>
 #include <opm/models/blackoil/blackoilonephaseindices.hh>
 #include <opm/models/blackoil/blackoiltwophaseindices.hh>
 
@@ -67,12 +67,9 @@ setEvaluationsFromValues()
 
 template<class FluidSystem, class Indices>
 void MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
-update(const WellState<Scalar>& well_state,
+update(const WellState<Scalar, IndexTraits>& well_state,
        const bool stop_or_zero_rate_target)
 {
-    static constexpr int Water = BlackoilPhases::Aqua;
-    static constexpr int Gas = BlackoilPhases::Vapour;
-
     // TODO: to test using rate conversion coefficients to see if it will be better than
     // this default one
     if (!well_.isOperableAndSolvable() && !well_.wellIsStopped())
@@ -87,7 +84,6 @@ update(const WellState<Scalar>& well_state,
     assert(segments.size() == value_.size());
     const auto& segment_rates = segments.rates;
     const auto& segment_pressure = segments.pressure;
-    const PhaseUsage& pu = well_.phaseUsage();
 
     for (std::size_t seg = 0; seg < value_.size(); ++seg) {
         // calculate the total rate for each segment
@@ -111,13 +107,22 @@ update(const WellState<Scalar>& well_state,
         if (stop_or_zero_rate_target && seg == 0) {
             value_[seg][WQTotal] = 0;
         }
-        if (std::abs(total_seg_rate) > 0.) {
+
+        // todo to map old fraction to new perforations for now start from scratch.
+        if (ws.primaryvar.size() == value_.size() * numWellEq) {
             if (has_wfrac_variable) {
-                const int water_pos = pu.phase_pos[Water];
+                value_[seg][WFrac] = ws.primaryvar[seg * numWellEq + WFrac];
+            }
+            if (has_gfrac_variable) {
+                value_[seg][GFrac] = ws.primaryvar[seg * numWellEq + GFrac];
+            }
+        } else if (std::abs(total_seg_rate) > 0.) {
+            if (has_wfrac_variable) {
+                const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
                 value_[seg][WFrac] = well_.scalingFactor(water_pos) * segment_rates[well_.numPhases() * seg + water_pos] / total_seg_rate;
             }
             if (has_gfrac_variable) {
-                const int gas_pos = pu.phase_pos[Gas];
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
                 value_[seg][GFrac] = well_.scalingFactor(gas_pos) * segment_rates[well_.numPhases() * seg + gas_pos] / total_seg_rate;
             }
         } else { // total_seg_rate == 0
@@ -214,22 +219,24 @@ updateNewton(const BVectorWell& dwells,
 
 template<class FluidSystem, class Indices>
 void MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
-copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
+copyToWellState(const  MultisegmentWellGeneric<Scalar, IndexTraits>& mswell,
                 const Scalar rho,
-                WellState<Scalar>& well_state,
+                WellState<Scalar, IndexTraits>& well_state,
                 const SummaryState& summary_state,
                 DeferredLogger& deferred_logger) const
 {
-    static constexpr int Gas = BlackoilPhases::Vapour;
-    static constexpr int Oil = BlackoilPhases::Liquid;
-    static constexpr int Water = BlackoilPhases::Aqua;
-
     const auto pvtReg = std::max(well_.wellEcl().pvt_table_number() - 1, 0);
 
-    const PhaseUsage& pu = well_.phaseUsage();
-    const int oil_pos = pu.phase_pos[Oil];
-
     auto& ws = well_state.well(well_.indexOfWell());
+
+    // Store primary variables
+    ws.primaryvar.resize(value_.size() * numWellEq);
+    for (std::size_t seg = 0; seg < value_.size(); ++seg) {
+        for (int ii = 0; ii < numWellEq; ++ii) {
+            ws.primaryvar[seg * numWellEq + ii] = value_[seg][ii];
+        }
+    }
+
     auto& segments = ws.segments;
     auto& segment_rates = segments.rates;
     auto& disgas = segments.dissolved_gas_rate;
@@ -239,31 +246,32 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
         std::vector<Scalar> fractions(well_.numPhases(), 0.0);
 
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
+            const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
             fractions[oil_pos] = 1.0;
             if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-                const int water_pos = pu.phase_pos[Water];
+                const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
                 fractions[water_pos] = value_[seg][WFrac];
                 fractions[oil_pos] -= fractions[water_pos];
             }
 
             if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-                const int gas_pos = pu.phase_pos[Gas];
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
                 fractions[gas_pos] = value_[seg][GFrac];
                 fractions[oil_pos] -= fractions[gas_pos];
             }
         }
         else if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-            const int water_pos = pu.phase_pos[Water];
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
             fractions[water_pos] = 1.0;
 
             if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-                const int gas_pos = pu.phase_pos[Gas];
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
                 fractions[gas_pos] = value_[seg][GFrac];
                 fractions[water_pos] -= fractions[gas_pos];
             }
         }
         else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            const int gas_pos = pu.phase_pos[Gas];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
             fractions[gas_pos] = 1.0;
         }
 
@@ -324,8 +332,8 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
         else {
             const auto* qs = &segment_rates[seg * well_.numPhases()];
             const auto denom = 1.0 - (Rs * Rv);
-            const auto io = pu.phase_pos[Oil];
-            const auto ig = pu.phase_pos[Gas];
+            const auto io = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+            const auto ig = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
             disgas[seg] = Rs * (qs[io] - Rv*qs[ig]) / denom;
             vapoil[seg] = Rv * (qs[ig] - Rs*qs[io]) / denom;
         }
@@ -374,19 +382,22 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
 
         // 5) Local condition phase viscosities.
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-            segments.phase_viscosity[seg * well_.numPhases() + pu.phase_pos[Oil]] =
+            const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+            segments.phase_viscosity[seg * well_.numPhases() + oil_pos] =
                     FluidSystem::oilPvt().viscosity(pvtReg, temperature, segment_pressure[seg], Rs);
         }
 
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-            segments.phase_viscosity[seg * well_.numPhases() + pu.phase_pos[Water]] =
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+            segments.phase_viscosity[seg * well_.numPhases() + water_pos] =
                 FluidSystem::waterPvt().viscosity(pvtReg, temperature,
                                                   segment_pressure[seg],
                                                   Scalar{0.0} /*Rsw*/, saltConc);
         }
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            segments.phase_viscosity[seg * well_.numPhases() + pu.phase_pos[Gas]] =
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            segments.phase_viscosity[seg * well_.numPhases() + gas_pos] =
                 FluidSystem::gasPvt().viscosity(pvtReg, temperature, segment_pressure[seg], Rv, Rvw);
         }
     }
@@ -426,9 +437,6 @@ copyToWellState(const MultisegmentWellGeneric<Scalar>& mswell,
     // We might want to unify the way regarding AQL value.
     WellBhpThpCalculator(well_)
         .updateThp(rho, [this, &summary_state]() { return well_.wellEcl().alq_value(summary_state); },
-                   {FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx),
-                    FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx),
-                    FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)},
                    well_state, summary_state, deferred_logger);
 }
 
@@ -436,80 +444,91 @@ template<class FluidSystem, class Indices>
 void MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
 processFractions(const int seg)
 {
-    static constexpr int Water = BlackoilPhases::Aqua;
-    static constexpr int Oil = BlackoilPhases::Liquid;
-    static constexpr int Gas = BlackoilPhases::Vapour;
-
-    const PhaseUsage& pu = well_.phaseUsage();
-
     std::vector<Scalar> fractions(well_.numPhases(), 0.0);
 
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        fractions[pu.phase_pos[Oil]] = 1.0;
+        const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+        fractions[oil_pos] = 1.0;
 
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-            fractions[pu.phase_pos[Water]] = value_[seg][WFrac];
-            fractions[pu.phase_pos[Oil]] -= fractions[pu.phase_pos[Water]];
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+            fractions[water_pos] = value_[seg][WFrac];
+            fractions[oil_pos] -= fractions[water_pos];
         }
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            fractions[pu.phase_pos[Gas]] = value_[seg][GFrac];
-            fractions[pu.phase_pos[Oil]] -= fractions[pu.phase_pos[Gas]];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            fractions[gas_pos] = value_[seg][GFrac];
+            fractions[oil_pos] -= fractions[gas_pos];
         }
     }
     else if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        fractions[pu.phase_pos[Water]] = 1.0;
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+        fractions[water_pos] = 1.0;
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            fractions[pu.phase_pos[Gas]] = value_[seg][GFrac];
-            fractions[pu.phase_pos[Water]] -= fractions[pu.phase_pos[Gas]];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            fractions[gas_pos] = value_[seg][GFrac];
+            fractions[water_pos] -= fractions[gas_pos];
         }
     } else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        fractions[pu.phase_pos[Gas]] = 1.0;
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        fractions[gas_pos] = 1.0;
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        if (fractions[pu.phase_pos[Water]] < 0.0) {
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+        if (fractions[water_pos] < 0.0) {
             if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
-                fractions[pu.phase_pos[Gas]] /= (1.0 - fractions[pu.phase_pos[Water]]);
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+                fractions[gas_pos] /= (1.0 - fractions[water_pos]);
             }
             if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-                fractions[pu.phase_pos[Oil]] /= (1.0 - fractions[pu.phase_pos[Water]]);
+                const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+                fractions[oil_pos] /= (1.0 - fractions[water_pos]);
             }
-            fractions[pu.phase_pos[Water]] = 0.0;
+            fractions[water_pos] = 0.0;
         }
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        if (fractions[pu.phase_pos[Gas]] < 0.0) {
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        if (fractions[gas_pos] < 0.0) {
             if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
-                fractions[pu.phase_pos[Water]] /= (1.0 - fractions[pu.phase_pos[Gas]]);
+                const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+                fractions[water_pos] /= (1.0 - fractions[gas_pos]);
             }
             if ( FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) ) {
-                fractions[pu.phase_pos[Oil]] /= (1.0 - fractions[pu.phase_pos[Gas]]);
+                const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+                fractions[oil_pos] /= (1.0 - fractions[gas_pos]);
             }
-            fractions[pu.phase_pos[Gas]] = 0.0;
+            fractions[gas_pos] = 0.0;
         }
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        if (fractions[pu.phase_pos[Oil]] < 0.0) {
+        const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+        if (fractions[oil_pos] < 0.0) {
             if ( FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx) ) {
-                fractions[pu.phase_pos[Water]] /= (1.0 - fractions[pu.phase_pos[Oil]]);
+                const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+                fractions[water_pos] /= (1.0 - fractions[oil_pos]);
             }
             if ( FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) ) {
-                fractions[pu.phase_pos[Gas]] /= (1.0 - fractions[pu.phase_pos[Oil]]);
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+                fractions[gas_pos] /= (1.0 - fractions[oil_pos]);
             }
-            fractions[pu.phase_pos[Oil]] = 0.0;
+            fractions[oil_pos] = 0.0;
         }
     }
 
     if constexpr (has_wfrac_variable) {
-        value_[seg][WFrac] = fractions[pu.phase_pos[Water]];
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+        value_[seg][WFrac] = fractions[water_pos];
     }
 
     if constexpr (has_gfrac_variable) {
-        value_[seg][GFrac] = fractions[pu.phase_pos[Gas]];
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        value_[seg][GFrac] = fractions[gas_pos];
     }
 }
 
@@ -519,11 +538,11 @@ MultisegmentWellPrimaryVariables<FluidSystem,Indices>::
 volumeFraction(const int seg,
                const int compIdx) const
 {
-    if (has_wfrac_variable && compIdx == Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx)) {
+    if (has_wfrac_variable && compIdx == FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx)) {
         return evaluation_[seg][WFrac];
     }
 
-    if (has_gfrac_variable && compIdx == Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx)) {
+    if (has_gfrac_variable && compIdx == FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx)) {
         return evaluation_[seg][GFrac];
     }
 
@@ -557,7 +576,7 @@ volumeFractionScaled(const int seg,
     // For reservoir rate control, the distr in well control is used for the
     // rate conversion coefficients. For the injection well, only the distr of the injection
     // phase is not zero.
-    const Scalar scale = well_.scalingFactor(well_.modelCompIdxToFlowCompIdx(comp_idx));
+    const Scalar scale = well_.scalingFactor(FluidSystem::activeCompToActivePhaseIdx(comp_idx));
     if (scale > 0.) {
         return this->volumeFraction(seg, comp_idx) / scale;
     }
@@ -572,7 +591,7 @@ surfaceVolumeFraction(const int seg,
                       const int comp_idx) const
 {
     EvalWell sum_volume_fraction_scaled = 0.;
-    for (int idx = 0; idx < well_.numComponents(); ++idx) {
+    for (int idx = 0; idx < well_.numConservationQuantities(); ++idx) {
         sum_volume_fraction_scaled += this->volumeFractionScaled(seg, idx);
     }
 
@@ -596,19 +615,19 @@ getSegmentRateUpwinding(const int seg,
         auto phase = well.getInjectionProperties().injectorType;
 
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)
-                && Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx) == comp_idx
+                && FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx) == comp_idx
                 && phase == InjectorType::WATER)
-            return evaluation_[seg][WQTotal] / well_.scalingFactor(well_.modelCompIdxToFlowCompIdx(comp_idx));
+            return evaluation_[seg][WQTotal] / well_.scalingFactor(FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx));;
 
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)
-                && Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx) == comp_idx
+                && FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx) == comp_idx
                 && phase == InjectorType::OIL)
-            return evaluation_[seg][WQTotal] / well_.scalingFactor(well_.modelCompIdxToFlowCompIdx(comp_idx));
+            return evaluation_[seg][WQTotal] / well_.scalingFactor(FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx));
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)
-                && Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx) == comp_idx
+                && FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx) == comp_idx
                 && phase == InjectorType::GAS)
-            return evaluation_[seg][WQTotal] / well_.scalingFactor(well_.modelCompIdxToFlowCompIdx(comp_idx));
+            return evaluation_[seg][WQTotal] / well_.scalingFactor(FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx));
 
         return 0.0;
     }
@@ -683,43 +702,12 @@ outputLowLimitPressureSegments(DeferredLogger& deferred_logger) const
     }
 }
 
-template<class Scalar>
-using FS = BlackOilFluidSystem<Scalar,BlackOilDefaultIndexTraits>;
+#include <opm/simulators/utils/InstantiationIndicesMacros.hpp>
 
-#define INSTANTIATE(T,...) \
-    template class MultisegmentWellPrimaryVariables<FS<T>,__VA_ARGS__>;
-
-#define INSTANTIATE_TYPE(T)                                                  \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,0u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,1u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,0u,false,false,0u,1u,5u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,1u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,true,0u,0u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,true,0u,2u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,2u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,true,0u,0u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<1u,0u,0u,0u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<1u,0u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,1u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,1u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,true,false,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,true,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,true,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,false,1u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,true,2u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<1u,0u,0u,0u,true,false,0u,0u>)
-
-INSTANTIATE_TYPE(double)
+INSTANTIATE_TYPE_INDICES(MultisegmentWellPrimaryVariables, double)
 
 #if FLOW_INSTANTIATE_FLOAT
-INSTANTIATE_TYPE(float)
+INSTANTIATE_TYPE_INDICES(MultisegmentWellPrimaryVariables, float)
 #endif
 
 }
