@@ -29,10 +29,10 @@
 
 #include <opm/material/densead/EvaluationFormat.hpp>
 
-#include <opm/material/fluidsystems/BlackOilDefaultIndexTraits.hpp>
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
 #include <opm/material/fluidsystems/BlackOilFluidSystem.hpp>
 
-#include <opm/models/blackoil/blackoilindices.hh>
+#include <opm/models/blackoil/blackoilvariableandequationindices.hh>
 #include <opm/models/blackoil/blackoilonephaseindices.hh>
 #include <opm/models/blackoil/blackoiltwophaseindices.hh>
 
@@ -41,8 +41,6 @@
 #include <opm/simulators/wells/SegmentState.hpp>
 #include <opm/simulators/wells/WellInterfaceGeneric.hpp>
 
-#include <opm/simulators/utils/BlackoilPhases.hpp>
-
 #include <fmt/format.h>
 
 #include <array>
@@ -61,18 +59,22 @@ template<class FluidSystem, class Indices>
 MultisegmentWellSegments<FluidSystem,Indices>::
 MultisegmentWellSegments(const int numSegments,
                          const ParallelWellInfo<Scalar>& parallel_well_info,
-                         WellInterfaceGeneric<Scalar>& well)
+                         WellInterfaceGeneric<Scalar, IndexTraits>& well)
     : perforations_(numSegments)
-    , local_perforation_depth_diffs_(well.numPerfs(), 0.0)
+    , local_perforation_depth_diffs_(well.numLocalPerfs(), 0.0)
+    // Generally, the info stored with the class MultisegmentWellSegments is global, i.e., the same across all
+    // processes, since segments do not have a 1:1 correspondence to the grid. All members of this class store
+    // global information, except for local_perforation_depth_diffs_, which is the only one that contains only
+    // local information. This is an exception and intentionally, since here, we only need the local entries.
     , inlets_(well.wellEcl().getSegments().size())
     , depth_diffs_(numSegments, 0.0)
     , densities_(numSegments, 0.0)
     , mass_rates_(numSegments, 0.0)
     , viscosities_(numSegments, 0.0)
     , upwinding_segments_(numSegments, 0)
-    , phase_densities_(numSegments, std::vector<EvalWell>(well.numComponents(), 0.0)) // number of phase here?
-    , phase_fractions_(numSegments, std::vector<EvalWell>(well.numComponents(), 0.0)) // number of phase here?
-    , phase_viscosities_(numSegments, std::vector<EvalWell>(well.numComponents(), 0.0)) // number of phase here?
+    , phase_densities_(numSegments, std::vector<EvalWell>(well.numConservationQuantities(), 0.0)) // number of phase here?
+    , phase_fractions_(numSegments, std::vector<EvalWell>(well.numConservationQuantities(), 0.0)) // number of phase here?
+    , phase_viscosities_(numSegments, std::vector<EvalWell>(well.numConservationQuantities(), 0.0)) // number of phase here?
     , well_(well)
 {
     // since we decide to use the WellSegments from the well parser. we can reuse a lot from it.
@@ -87,7 +89,7 @@ MultisegmentWellSegments(const int numSegments,
     // side
     int i_perf_wells = 0;
     // The perfDepth vector will contain the depths of all perforations across all processes of this well!
-    int num_perfs_whole_mswell = parallel_well_info.communication().sum(well.numPerfs());
+    int num_perfs_whole_mswell = parallel_well_info.communication().sum(well.numLocalPerfs());
     well.perfDepth().resize(num_perfs_whole_mswell, 0.0);
     const auto& segment_set = well_.wellEcl().getSegments();
     for (std::size_t perf = 0; perf < completion_set.size(); ++perf) {
@@ -104,7 +106,7 @@ MultisegmentWellSegments(const int numSegments,
             perforations_[segment_index].push_back(i_perf_wells);
             well.perfDepth()[i_perf_wells] = connection.depth();
             const Scalar segment_depth = segment_set[segment_index].depth();
-            int local_perf_index = parallel_well_info.activeToLocal(i_perf_wells);
+            int local_perf_index = parallel_well_info.activePerfToLocalPerf(i_perf_wells);
             if (local_perf_index > -1) // If local_perf_index == -1, then the perforation is not on this process
                 local_perforation_depth_diffs_[local_perf_index] = well_.perfDepth()[i_perf_wells] - segment_depth;
             i_perf_wells++;
@@ -141,32 +143,32 @@ computeFluidProperties(const EvalWell& temperature,
                        int pvt_region_index,
                        DeferredLogger& deferred_logger)
 {
-    std::vector<Scalar> surf_dens(well_.numComponents());
+    std::vector<Scalar> surf_dens(well_.numConservationQuantities());
     // Surface density.
     for (unsigned phaseIdx = 0; phaseIdx < FluidSystem::numPhases; ++phaseIdx) {
         if (!FluidSystem::phaseIsActive(phaseIdx)) {
             continue;
         }
 
-        const unsigned compIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::solventComponentIndex(phaseIdx));
+        const unsigned compIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::solventComponentIndex(phaseIdx));
         surf_dens[compIdx] = FluidSystem::referenceDensity( phaseIdx, pvt_region_index);
     }
 
     for (std::size_t seg = 0; seg < perforations_.size(); ++seg) {
         // the compostion of the components inside wellbore under surface condition
-        std::vector<EvalWell> mix_s(well_.numComponents(), 0.0);
-        for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+        std::vector<EvalWell> mix_s(well_.numConservationQuantities(), 0.0);
+        for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
             mix_s[comp_idx] = primary_variables.surfaceVolumeFraction(seg, comp_idx);
         }
 
-        std::vector<EvalWell> b(well_.numComponents(), 0.0);
-        std::vector<EvalWell> visc(well_.numComponents(), 0.0);
+        std::vector<EvalWell> b(well_.numConservationQuantities(), 0.0);
+        std::vector<EvalWell> visc(well_.numConservationQuantities(), 0.0);
         std::vector<EvalWell>& phase_densities = phase_densities_[seg];
 
         const EvalWell seg_pressure = primary_variables.getSegmentPressure(seg);
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
             EvalWell rsw(0.0);
-            const unsigned waterCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx);
+            const unsigned waterCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx);
             b[waterCompIdx] =
                 FluidSystem::waterPvt().inverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure, rsw, saltConcentration);
             visc[waterCompIdx] =
@@ -180,74 +182,62 @@ computeFluidProperties(const EvalWell& temperature,
         EvalWell rvw(0.0);
         // gas phase
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            const unsigned gasCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
-            if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-                const unsigned oilCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+            const unsigned gasCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
+            const bool oil_exist = (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && mix_s[FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx)] > 0.0);
+            if (oil_exist) {
+                const unsigned oilCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
                 const EvalWell rvmax = FluidSystem::gasPvt().saturatedOilVaporizationFactor(pvt_region_index, temperature, seg_pressure);
-                if (mix_s[oilCompIdx] > 0.0) {
-                    if (mix_s[gasCompIdx] > 0.0) {
-                        rv = mix_s[oilCompIdx] / mix_s[gasCompIdx];
-                    }
-
-                    if (rv > rvmax) {
-                        rv = rvmax;
-                    }
-                    b[gasCompIdx] =
-                        FluidSystem::gasPvt().inverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure, rv, rvw);
-                    visc[gasCompIdx] =
-                        FluidSystem::gasPvt().viscosity(pvt_region_index, temperature, seg_pressure, rv, rvw);
-                    phase_densities[gasCompIdx] = b[gasCompIdx] * surf_dens[gasCompIdx]
-                                                + rv * b[gasCompIdx] * surf_dens[oilCompIdx];
-                } else { // no oil exists
-                    b[gasCompIdx] =
-                        FluidSystem::gasPvt().saturatedInverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure);
-                    visc[gasCompIdx] =
-                        FluidSystem::gasPvt().saturatedViscosity(pvt_region_index, temperature, seg_pressure);
-                    phase_densities[gasCompIdx] = b[gasCompIdx] * surf_dens[gasCompIdx];
+                if (mix_s[gasCompIdx] > 0.0) {
+                    rv = mix_s[oilCompIdx] / mix_s[gasCompIdx];
                 }
-            } else { // no Liquid phase
-                // it is the same with zero mix_s[Oil]
+
+                if (rv > rvmax) {
+                    rv = rvmax;
+                }
+                b[gasCompIdx] =
+                        FluidSystem::gasPvt().inverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure,
+                                                                           rv, rvw);
+                visc[gasCompIdx] =
+                        FluidSystem::gasPvt().viscosity(pvt_region_index, temperature, seg_pressure, rv, rvw);
+                phase_densities[gasCompIdx] = b[gasCompIdx] * surf_dens[gasCompIdx]
+                                              + rv * b[gasCompIdx] * surf_dens[oilCompIdx];
+            } else { // no oil here
                 b[gasCompIdx] =
                     FluidSystem::gasPvt().saturatedInverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure);
                 visc[gasCompIdx] =
                     FluidSystem::gasPvt().saturatedViscosity(pvt_region_index, temperature, seg_pressure);
+                phase_densities[gasCompIdx] = b[gasCompIdx] * surf_dens[gasCompIdx];
             }
         }
 
         EvalWell rs(0.0);
         // oil phase
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-            const unsigned oilCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
-            if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-                const unsigned gasCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
+            const unsigned oilCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
+            const bool gas_exist = (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) && mix_s[FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx)] > 0.0);
+            if (gas_exist) {
+                const unsigned gasCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
                 const EvalWell rsmax = FluidSystem::oilPvt().saturatedGasDissolutionFactor(pvt_region_index, temperature, seg_pressure);
-                if (mix_s[gasCompIdx] > 0.0) {
-                    if (mix_s[oilCompIdx] > 0.0) {
-                        rs = mix_s[gasCompIdx] / mix_s[oilCompIdx];
-                    }
-
-                    if (rs > rsmax) {
-                        rs = rsmax;
-                    }
-                    b[oilCompIdx] =
-                        FluidSystem::oilPvt().inverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure, rs);
-                    visc[oilCompIdx] =
-                        FluidSystem::oilPvt().viscosity(pvt_region_index, temperature, seg_pressure, rs);
-                    phase_densities[oilCompIdx] = b[oilCompIdx] * surf_dens[oilCompIdx]
-                                                + rs * b[oilCompIdx] * surf_dens[gasCompIdx];
-                } else { // no oil exists
-                    b[oilCompIdx] =
-                        FluidSystem::oilPvt().saturatedInverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure);
-                    visc[oilCompIdx] =
-                        FluidSystem::oilPvt().saturatedViscosity(pvt_region_index, temperature, seg_pressure);
-                    phase_densities[oilCompIdx] = b[oilCompIdx] * surf_dens[oilCompIdx];
+                if (mix_s[oilCompIdx] > 0.0) {
+                    rs = mix_s[gasCompIdx] / mix_s[oilCompIdx];
                 }
-            } else { // no Liquid phase
-                // it is the same with zero mix_s[Oil]
+
+                if (rs > rsmax) {
+                    rs = rsmax;
+                }
+                b[oilCompIdx] =
+                        FluidSystem::oilPvt().inverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure,
+                                                                           rs);
+                visc[oilCompIdx] =
+                        FluidSystem::oilPvt().viscosity(pvt_region_index, temperature, seg_pressure, rs);
+                phase_densities[oilCompIdx] = b[oilCompIdx] * surf_dens[oilCompIdx]
+                                              + rs * b[oilCompIdx] * surf_dens[gasCompIdx];
+            } else { // no gas phase
                 b[oilCompIdx] =
                     FluidSystem::oilPvt().saturatedInverseFormationVolumeFactor(pvt_region_index, temperature, seg_pressure);
                 visc[oilCompIdx] =
                     FluidSystem::oilPvt().saturatedViscosity(pvt_region_index, temperature, seg_pressure);
+                phase_densities[oilCompIdx] = b[oilCompIdx] * surf_dens[oilCompIdx];
             }
         }
 
@@ -255,8 +245,8 @@ computeFluidProperties(const EvalWell& temperature,
 
         std::vector<EvalWell> mix(mix_s);
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            const unsigned gasCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
-            const unsigned oilCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+            const unsigned gasCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
+            const unsigned oilCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
 
             const EvalWell d = 1.0 - rs * rv;
             if (d <= 0.0) {
@@ -279,13 +269,13 @@ computeFluidProperties(const EvalWell& temperature,
         }
 
         EvalWell volrat(0.0);
-        for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+        for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
             volrat += mix[comp_idx] / b[comp_idx];
         }
 
         viscosities_[seg] = 0.;
         // calculate the average viscosity
-        for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+        for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
             const EvalWell fraction =  mix[comp_idx] / b[comp_idx] / volrat;
             // TODO: a little more work needs to be done to handle the negative fractions here
             phase_fractions_[seg][comp_idx] = fraction; // >= 0.0 ? fraction : 0.0;
@@ -293,14 +283,14 @@ computeFluidProperties(const EvalWell& temperature,
         }
 
         EvalWell density(0.0);
-        for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+        for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
             density += surf_dens[comp_idx] * mix_s[comp_idx];
         }
         densities_[seg] = density / volrat;
 
         // calculate the mass rates
         mass_rates_[seg] = 0.;
-        for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+        for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
             const int upwind_seg = upwinding_segments_[seg];
             const EvalWell rate = primary_variables.getSegmentRateUpwinding(seg,
                                                                             upwind_seg,
@@ -364,14 +354,14 @@ getSurfaceVolume(const EvalWell& temperature,
 {
     const EvalWell seg_pressure = primary_variables.getSegmentPressure(seg_idx);
 
-    std::vector<EvalWell> mix_s(well_.numComponents(), 0.0);
-    for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+    std::vector<EvalWell> mix_s(well_.numConservationQuantities(), 0.0);
+    for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
         mix_s[comp_idx] = primary_variables.surfaceVolumeFraction(seg_idx, comp_idx);
     }
 
-    std::vector<EvalWell> b(well_.numComponents(), 0.);
+    std::vector<EvalWell> b(well_.numConservationQuantities(), 0.);
     if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        const unsigned waterCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx);
+        const unsigned waterCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx);
         EvalWell rsw(0.0);
         b[waterCompIdx] =
             FluidSystem::waterPvt().inverseFormationVolumeFactor(pvt_region_index,
@@ -385,9 +375,9 @@ getSurfaceVolume(const EvalWell& temperature,
     EvalWell rvw(0.0);
     // gas phase
     if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        const unsigned gasCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
+        const unsigned gasCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
         if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-            const unsigned oilCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+            const unsigned oilCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
             EvalWell rvmax = FluidSystem::gasPvt().saturatedOilVaporizationFactor(pvt_region_index,
                                                                                   temperature,
                                                                                   seg_pressure);
@@ -426,9 +416,9 @@ getSurfaceVolume(const EvalWell& temperature,
     EvalWell rs(0.0);
     // oil phase
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        const unsigned oilCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+        const unsigned oilCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            const unsigned gasCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
+            const unsigned gasCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
             EvalWell rsmax = FluidSystem::oilPvt().saturatedGasDissolutionFactor(pvt_region_index,
                                                                                  temperature,
                                                                                  seg_pressure);
@@ -466,8 +456,8 @@ getSurfaceVolume(const EvalWell& temperature,
 
     std::vector<EvalWell> mix(mix_s);
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        const unsigned gasCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
-        const unsigned oilCompIdx = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+        const unsigned gasCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
+        const unsigned oilCompIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
 
         const EvalWell d = 1.0 - rs * rv;
         if (d <= 0.0 || d > 1.0) {
@@ -490,7 +480,7 @@ getSurfaceVolume(const EvalWell& temperature,
     }
 
     EvalWell vol_ratio(0.0);
-    for (int comp_idx = 0; comp_idx < well_.numComponents(); ++comp_idx) {
+    for (int comp_idx = 0; comp_idx < well_.numConservationQuantities(); ++comp_idx) {
         vol_ratio += mix[comp_idx] / b[comp_idx];
     }
 
@@ -503,7 +493,7 @@ getSurfaceVolume(const EvalWell& temperature,
 template<class FluidSystem, class Indices>
 typename MultisegmentWellSegments<FluidSystem,Indices>::EvalWell
 MultisegmentWellSegments<FluidSystem,Indices>::
-getFrictionPressureLoss(const int seg, 
+getFrictionPressureLoss(const int seg,
                         const bool extra_reverse_flow_derivatives /*false*/) const
 {
     EvalWell mass_rate = mass_rates_[seg];
@@ -568,7 +558,7 @@ pressureDropSpiralICD(const int seg,
     EvalWell water_fraction = 0.;
     EvalWell water_viscosity = 0.;
     if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        const int water_pos = Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx);
+        const int water_pos = FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx);
         water_fraction = phase_fractions[water_pos];
         water_viscosity = phase_viscosities[water_pos];
     }
@@ -576,7 +566,7 @@ pressureDropSpiralICD(const int seg,
     EvalWell oil_fraction = 0.;
     EvalWell oil_viscosity = 0.;
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        const int oil_pos = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+        const int oil_pos = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
         oil_fraction = phase_fractions[oil_pos];
         oil_viscosity = phase_viscosities[oil_pos];
     }
@@ -584,7 +574,7 @@ pressureDropSpiralICD(const int seg,
     EvalWell gas_fraction = 0.;
     EvalWell gas_viscosity = 0.;
     if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        const int gas_pos = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
+        const int gas_pos = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
         gas_fraction = phase_fractions[gas_pos];
         gas_viscosity = phase_viscosities[gas_pos];
     }
@@ -592,12 +582,12 @@ pressureDropSpiralICD(const int seg,
     EvalWell density = densities_[seg_upwind];
     EvalWell mass_rate = mass_rates_[seg];
     // In the reverse flow case, we don't have enough slots for all derivatives, e.g.,
-    // upwind pressure and flow. We amend this by a second function call option, where 
+    // upwind pressure and flow. We amend this by a second function call option, where
     // only these remaining derivatives are considered.
-    // For reference: the pressure equation assumes pressure/flow derivatives are given 
-    // at segment node while fraction derivatives are given at upwind node.   
+    // For reference: the pressure equation assumes pressure/flow derivatives are given
+    // at segment node while fraction derivatives are given at upwind node.
     if (seg != seg_upwind) {
-        constexpr int nvar = FluidSystem::numPhases + 1;
+        constexpr int nvar = PrimaryVariables::numWellEq;
         std::vector<bool> zero_mask(nvar, false);
         if (!extra_reverse_flow_derivatives){
             zero_mask[PrimaryVariables::WQTotal] = true;
@@ -660,7 +650,7 @@ template<class FluidSystem, class Indices>
 typename MultisegmentWellSegments<FluidSystem,Indices>::EvalWell
 MultisegmentWellSegments<FluidSystem,Indices>::
 pressureDropAutoICD(const int seg,
-                    const UnitSystem& unit_system, 
+                    const UnitSystem& unit_system,
                     const bool extra_reverse_flow_derivatives /*false*/) const
 {
     const auto& segment_set = well_.wellEcl().getSegments();
@@ -675,7 +665,7 @@ pressureDropAutoICD(const int seg,
     EvalWell water_viscosity = 0.;
     EvalWell water_density = 0.;
     if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        const int water_pos = Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx);
+        const int water_pos = FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx);
         water_fraction = phase_fractions[water_pos];
         water_viscosity = phase_viscosities[water_pos];
         water_density = phase_densities[water_pos];
@@ -685,7 +675,7 @@ pressureDropAutoICD(const int seg,
     EvalWell oil_viscosity = 0.;
     EvalWell oil_density = 0.;
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        const int oil_pos = Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx);
+        const int oil_pos = FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx);
         oil_fraction = phase_fractions[oil_pos];
         oil_viscosity = phase_viscosities[oil_pos];
         oil_density = phase_densities[oil_pos];
@@ -695,7 +685,7 @@ pressureDropAutoICD(const int seg,
     EvalWell gas_viscosity = 0.;
     EvalWell gas_density = 0.;
     if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        const int gas_pos = Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx);
+        const int gas_pos = FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx);
         gas_fraction = phase_fractions[gas_pos];
         gas_viscosity = phase_viscosities[gas_pos];
         gas_density = phase_densities[gas_pos];
@@ -704,12 +694,12 @@ pressureDropAutoICD(const int seg,
     EvalWell density = densities_[seg_upwind];
     EvalWell mass_rate = mass_rates_[seg];
     // In the reverse flow case, we don't have enough slots for all derivatives, e.g.,
-    // upwind pressure and flow. We amend this by a second function call option, where 
+    // upwind pressure and flow. We amend this by a second function call option, where
     // only these remaining derivatives are considered.
-    // For reference: the pressure equation assumes pressure/flow derivatives are given 
-    // at segment node while fraction derivatives are given at upwind node.   
+    // For reference: the pressure equation assumes pressure/flow derivatives are given
+    // at segment node while fraction derivatives are given at upwind node.
     if (seg != seg_upwind) {
-        constexpr int nvar = FluidSystem::numPhases + 1;
+        constexpr int nvar = PrimaryVariables::numWellEq;
         std::vector<bool> zero_mask(nvar, false);
         if (!extra_reverse_flow_derivatives){
             zero_mask[PrimaryVariables::WQTotal] = true;
@@ -763,8 +753,8 @@ pressureDropAutoICD(const int seg,
     const Scalar unit_volume_rate = unit_system.to_si(M::geometric_volume_rate, 1.);
 
     // TODO: we did not consider the maximum allowed rate here
-    const auto result = sign / rho_reference * mixture_density * mixture_density
-                      * safe_pow(visc_reference/mixture_viscosity, aicd.viscExponent())
+    const auto result = sign * safe_pow(mixture_density / rho_reference, aicd.densityExponent())
+                      * safe_pow(visc_reference/mixture_viscosity, aicd.viscExponent()) * mixture_density
                       * aicd.strength() * safe_pow( -sign * volume_rate_icd, aicd.flowRateExponent())
                       * std::pow(unit_volume_rate, (2. - aicd.flowRateExponent())) ;
     return result;
@@ -785,10 +775,10 @@ pressureDropValve(const int seg,
     EvalWell visc = viscosities_[seg_upwind];
     EvalWell density = densities_[seg_upwind];
     // In the reverse flow case, we don't have enough slots for all derivatives, e.g.,
-    // upwind pressure and flow. We amend this by a second function call optioin, where 
+    // upwind pressure and flow. We amend this by a second function call optioin, where
     // only these remaining derivatives are considered.
-    // For reference: the pressure equation assumes pressure/flow derivatives are given 
-    // at segment node while fraction derivatives are given at upwind node. 
+    // For reference: the pressure equation assumes pressure/flow derivatives are given
+    // at segment node while fraction derivatives are given at upwind node.
     if (seg != seg_upwind) {
         if (!extra_reverse_flow_derivatives) {
             constexpr int WQTotal = Indices::numEq + PrimaryVariables::WQTotal;
@@ -838,11 +828,11 @@ accelerationPressureLossContribution(const int seg,
                                      const Scalar area,
                                      const bool extra_reverse_flow_derivatives /*false*/) const
 {
-    // Compute the *signed* velocity head for given segment (sign is positive for flow towards surface, i.e., negative rate) 
+    // Compute the *signed* velocity head for given segment (sign is positive for flow towards surface, i.e., negative rate)
     // Optionally return derivatives for reversed flow case
     EvalWell mass_rate = mass_rates_[seg];
     const int seg_upwind = upwinding_segments_[seg];
-    EvalWell density = densities_[seg_upwind];    
+    EvalWell density = densities_[seg_upwind];
     if (seg != seg_upwind) {
         if (!extra_reverse_flow_derivatives) {
             constexpr int WQTotal = Indices::numEq + PrimaryVariables::WQTotal;
@@ -863,19 +853,21 @@ accelerationPressureLossContribution(const int seg,
     }
     const Scalar sign = mass_rate > 0 ? -1.0 : 1.0;
     return sign * mswellhelpers::velocityHead(area, mass_rate, density);
-}                                     
+}
 
 template <class FluidSystem, class Indices>
 void
 MultisegmentWellSegments<FluidSystem,Indices>::
-copyPhaseDensities(const PhaseUsage& pu, SegmentState<Scalar>& segSol) const
+copyPhaseDensities(SegmentState<Scalar>& segSol) const
 {
+    // TODO: probably we should change the the name phaseMap
+    // this part can be wrong and needs to be checked
     auto* rho = segSol.phase_density.data();
 
     const auto phaseMap = std::vector {
-        std::pair { BlackoilPhases::Liquid, FluidSystem::oilPhaseIdx },
-        std::pair { BlackoilPhases::Vapour, FluidSystem::gasPhaseIdx },
-        std::pair { BlackoilPhases::Aqua  , FluidSystem::waterPhaseIdx },
+        FluidSystem::oilPhaseIdx,
+        FluidSystem::gasPhaseIdx,
+        FluidSystem::waterPhaseIdx
     };
 
     // Densities stored in 'rho' as
@@ -884,16 +876,18 @@ copyPhaseDensities(const PhaseUsage& pu, SegmentState<Scalar>& segSol) const
     //  ...
     //  { p0, p1, ..., (np - 1), mixture, mixture_with_exponents }]
     // Stride is np + 2.
-    for (const auto& [boPhase, fsPhaseIdx] : phaseMap) {
-        if (pu.phase_used[boPhase]) {
-            this->copyPhaseDensities(fsPhaseIdx, pu.num_phases + 2,
-                                     rho + pu.phase_pos[boPhase]);
+    // here it has to be number of active phases
+    constexpr int num_phases = Indices::numPhases;
+    for (const auto& fsPhaseIdx : phaseMap) {
+        if (FluidSystem::phaseIsActive(fsPhaseIdx)) {
+            const auto phase_active_index = FluidSystem::canonicalToActivePhaseIdx(fsPhaseIdx);
+            this->copyPhaseDensities(fsPhaseIdx, num_phases + 2, rho + phase_active_index);
         }
     }
 
     // Mixture densities.
     for (auto seg = 0*this->densities_.size(); seg < this->densities_.size(); ++seg) {
-        const auto mixOffset = seg*(pu.num_phases + 2) + pu.num_phases;
+        const auto mixOffset = seg*(num_phases + 2) + num_phases;
 
         rho[mixOffset + 0] = this->mixtureDensity(seg);
         rho[mixOffset + 1] = this->mixtureDensityWithExponents(seg);
@@ -907,7 +901,7 @@ copyPhaseDensities(const unsigned    phaseIdx,
                    const std::size_t stride,
                    Scalar*           dens) const
 {
-    const auto compIdx = Indices::canonicalToActiveComponentIndex
+    const auto compIdx = FluidSystem::canonicalToActiveCompIdx
         (FluidSystem::solventComponentIndex(phaseIdx));
 
     for (const auto& phase_density : this->phase_densities_) {
@@ -936,8 +930,7 @@ mixtureDensity(const int seg) const
             continue;
         }
 
-        const auto active_comp_index = Indices::
-            canonicalToActiveComponentIndex(FluidSystem::solventComponentIndex(phIdx));
+        const auto active_comp_index = FluidSystem::canonicalToActiveCompIdx(FluidSystem::solventComponentIndex(phIdx));
 
         mixDens += q[active_comp_index].value() * rho[active_comp_index].value();
     }
@@ -979,8 +972,7 @@ mixtureDensityWithExponents(const AutoICD& aicd, const int seg) const
 
     for (const auto& [fsPhaseIdx, densityExponent] : densityExponents) {
         if (FluidSystem::phaseIsActive(fsPhaseIdx)) {
-            const auto compIdx = Indices::
-                canonicalToActiveComponentIndex(FluidSystem::solventComponentIndex(fsPhaseIdx));
+            const auto compIdx = FluidSystem::canonicalToActiveCompIdx(FluidSystem::solventComponentIndex(fsPhaseIdx));
 
             // exp = (aicd.*densityExponent)() in native syntax.
             const auto exp = std::invoke(densityExponent, aicd);
@@ -992,43 +984,12 @@ mixtureDensityWithExponents(const AutoICD& aicd, const int seg) const
     return mixDens;
 }
 
-template<class Scalar>
-using FS = BlackOilFluidSystem<Scalar,BlackOilDefaultIndexTraits>;
+#include <opm/simulators/utils/InstantiationIndicesMacros.hpp>
 
-#define INSTANTIATE(T,...) \
-    template class MultisegmentWellSegments<FS<T>,__VA_ARGS__>;
-
-#define INSTANTIATE_TYPE(T)                                                  \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,0u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,1u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,0u,false,false,0u,1u,5u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,true,0u,2u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,1u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,2u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,true,0u,0u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,true,0u,0u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<1u,0u,0u,0u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,true,false,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,true,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,true,2u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<1u,0u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,1u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,1u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,false,1u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,true,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<1u,0u,0u,0u,true,false,0u,0u>)
-
-INSTANTIATE_TYPE(double)
+INSTANTIATE_TYPE_INDICES(MultisegmentWellSegments, double)
 
 #if FLOW_INSTANTIATE_FLOAT
-INSTANTIATE_TYPE(float)
+INSTANTIATE_TYPE_INDICES(MultisegmentWellSegments, float)
 #endif
 
 } // namespace Opm
