@@ -24,6 +24,10 @@
 #include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 #include <opm/material/densead/Evaluation.hpp>
 
+#if HAVE_MPI
+#include <opm/simulators/utils/MPISerializer.hpp>
+#endif
+
 #include <fmt/format.h>
 #include <cassert>
 #include <iterator>
@@ -168,15 +172,15 @@ int GlobalPerfContainerFactory<Scalar>::globalToLocal(const int globalIndex) con
 template<class Scalar>
 std::vector<Scalar> GlobalPerfContainerFactory<Scalar>::
 createGlobal(const std::vector<Scalar>& local_perf_container,
-             std::size_t num_components) const
+             std::size_t num_quantities) const
 {
     // Could be become templated later.
     using Value = Scalar;
 
     if (comm_.size() > 1)
     {
-        std::vector<Value> global_recv(perf_ecl_index_.size() * num_components);
-        if (num_components == 1)
+        std::vector<Value> global_recv(perf_ecl_index_.size() * num_quantities);
+        if (num_quantities == 1)
         {
             comm_.allgatherv(local_perf_container.data(), local_perf_container.size(),
                              global_recv.data(), const_cast<int*>(sizes_.data()),
@@ -185,12 +189,12 @@ createGlobal(const std::vector<Scalar>& local_perf_container,
         else
         {
 #if HAVE_MPI
-            // Create MPI type for sending num_components entries
+            // Create MPI type for sending num_quantities entries
             MPI_Datatype data_type;
-            MPI_Type_contiguous(num_components, Dune::MPITraits<Value>::getType(), &data_type);
+            MPI_Type_contiguous(num_quantities, Dune::MPITraits<Value>::getType(), &data_type);
             MPI_Type_commit(&data_type);
             MPI_Allgatherv(local_perf_container.data(),
-                           local_perf_container.size()/num_components,
+                           local_perf_container.size() / num_quantities,
                            data_type, global_recv.data(), sizes_.data(),
                            displ_.data(), data_type, comm_);
             MPI_Type_free(&data_type);
@@ -198,13 +202,13 @@ createGlobal(const std::vector<Scalar>& local_perf_container,
         }
 
         // reorder by ascending ecl index.
-        std::vector<Value> global_remapped(perf_ecl_index_.size() * num_components);
+        std::vector<Value> global_remapped(perf_ecl_index_.size() * num_quantities);
         auto global = global_remapped.begin();
         for (auto map_entry = map_received_.begin(); map_entry !=  map_received_.end(); ++map_entry)
         {
-            auto global_index = *map_entry * num_components;
+            auto global_index = *map_entry * num_quantities;
 
-            for(std::size_t i = 0; i < num_components; ++i)
+            for(std::size_t i = 0; i < num_quantities; ++i)
                 *(global++) = global_recv[global_index++];
         }
         assert(global == global_remapped.end());
@@ -220,7 +224,7 @@ template<class Scalar>
 void GlobalPerfContainerFactory<Scalar>::
 copyGlobalToLocal(const std::vector<Scalar>& global,
                   std::vector<Scalar>& local,
-                  std::size_t num_components) const
+                  std::size_t num_quantities) const
 {
     if (global.empty())
     {
@@ -237,9 +241,9 @@ copyGlobalToLocal(const std::vector<Scalar>& global,
             global_perf = std::lower_bound(global_perf, perf_ecl_index_.end(), pair.global());
             assert(global_perf != perf_ecl_index_.end());
             assert(*global_perf == pair.global());
-            auto local_index = pair.local() * num_components;
-            auto global_index = (global_perf - perf_ecl_index_.begin()) * num_components;
-            for (std::size_t i = 0; i < num_components; ++i)
+            auto local_index = pair.local() * num_quantities;
+            auto global_index = (global_perf - perf_ecl_index_.begin()) * num_quantities;
+            for (std::size_t i = 0; i < num_quantities; ++i)
                 local[local_index++] = global[global_index++];
         }
     }
@@ -524,7 +528,7 @@ ParallelWellInfo<Scalar>::ParallelWellInfo(const std::pair<std::string, bool>& w
 }
 
 template<class Scalar>
-void ParallelWellInfo<Scalar>::setActiveToLocalMap(const std::unordered_map<int,int> active_to_local_map) const {
+void ParallelWellInfo<Scalar>::setActivePerfToLocalPerfMap(const std::unordered_map<int,int> active_to_local_map) const {
     //active_to_local_map_ is marked as mutable
     active_to_local_map_ = active_to_local_map;
     for (const auto& [key, value] : active_to_local_map) {
@@ -533,7 +537,7 @@ void ParallelWellInfo<Scalar>::setActiveToLocalMap(const std::unordered_map<int,
 }
 
 template<class Scalar>
-int ParallelWellInfo<Scalar>::localToActive(std::size_t localIndex) const {
+int ParallelWellInfo<Scalar>::localPerfToActivePerf(std::size_t localIndex) const {
     if (comm_->size() == 1)
         return localIndex;
     auto it = local_to_active_map_.find(localIndex);
@@ -543,7 +547,7 @@ int ParallelWellInfo<Scalar>::localToActive(std::size_t localIndex) const {
 }
 
 template<class Scalar>
-int ParallelWellInfo<Scalar>::activeToLocal(const int activeIndex) const {
+int ParallelWellInfo<Scalar>::activePerfToLocalPerf(const int activeIndex) const {
     if (comm_->size() == 1)
         return activeIndex;
     auto it = active_to_local_map_.find(activeIndex);
@@ -553,7 +557,7 @@ int ParallelWellInfo<Scalar>::activeToLocal(const int activeIndex) const {
 }
 
 template<class Scalar>
-int ParallelWellInfo<Scalar>::localToGlobal(std::size_t localIndex) const {
+int ParallelWellInfo<Scalar>::localPerfToGlobalPerf(std::size_t localIndex) const {
     if(globalPerfCont_)
         return globalPerfCont_->localToGlobal(localIndex);
     else // If globalPerfCont_ is not set up, then this is a sequential run and local and global indices are the same.
@@ -561,7 +565,7 @@ int ParallelWellInfo<Scalar>::localToGlobal(std::size_t localIndex) const {
 }
 
 template<class Scalar>
-int ParallelWellInfo<Scalar>::globalToLocal(const int globalIndex) const {
+int ParallelWellInfo<Scalar>::globalPerfToLocalPerf(const int globalIndex) const {
     if(globalPerfCont_)
         return globalPerfCont_->globalToLocal(globalIndex);
     else // If globalPerfCont_ is not set up, then this is a sequential run and local and global indices are the same.
@@ -624,7 +628,11 @@ template<class Scalar>
 template<class T>
 T ParallelWellInfo<Scalar>::broadcastFirstPerforationValue(const T& t) const
 {
+#if HAVE_MPI
     T res = t;
+    if (comm_->size() == 1) {
+        return res;
+    }
     if (rankWithFirstPerf_ >= 0) {
 #ifndef NDEBUG
         assert(rankWithFirstPerf_ < comm_->size());
@@ -632,12 +640,17 @@ T ParallelWellInfo<Scalar>::broadcastFirstPerforationValue(const T& t) const
         // with other communication if there are bugs
         comm_->barrier();
 #endif
-        comm_->broadcast(&res, 1, rankWithFirstPerf_);
+
+        Parallel::MpiSerializer ser(*comm_);
+        ser.broadcast(Parallel::RootRank{rankWithFirstPerf_}, res);
 #ifndef NDEBUG
         comm_->barrier();
 #endif
     }
     return res;
+#else // !HAVE_MPI
+    return t;
+#endif
 }
 
 template<class Scalar>
@@ -802,9 +815,11 @@ template<class Scalar> using dIter = typename std::vector<Scalar>::iterator;
 template<class Scalar> using cdIter = typename std::vector<Scalar>::const_iterator;
 
 #define INSTANTIATE_BROADCAST_FIRST_PERF_DENSEAD_EVALUATION(T, DIM)                 \
-    template Opm::DenseAd::Evaluation<T, DIM, 0u> Opm::ParallelWellInfo<T>::        \
-        broadcastFirstPerforationValue<Opm::DenseAd::Evaluation<T, DIM, 0u>>        \
-        (Opm::DenseAd::Evaluation<T, DIM, 0u> const&) const;
+    template std::tuple<T, Opm::DenseAd::Evaluation<T, DIM, 0u>,int>                \
+        Opm::ParallelWellInfo<T>::broadcastFirstPerforationValue                    \
+        <std::tuple<T, Opm::DenseAd::Evaluation<T, DIM, 0u>,int>>                   \
+        (std::tuple<T, Opm::DenseAd::Evaluation<T, DIM, 0u>,int> const&)            \
+        const;
 
 #define INSTANTIATE_TYPE(T)                                                         \
     template class CheckDistributedWellConnections<T>;                              \
