@@ -52,6 +52,7 @@ PerfData<Scalar>::PerfData(const std::size_t num_perf,
     , satnum_id(num_perf)
     , ecl_index(num_perf)
     , gas_mass_rates(num_perf)
+    , wat_mass_rates(num_perf)
 {
     if (injector) {
         prepareInjectorContainers();
@@ -97,6 +98,7 @@ PerfData<Scalar> PerfData<Scalar>::serializationTestObject()
     result.filtrate_data = ConnFiltrateData<Scalar>::serializationTestObject();
     result.connFracStatistics.assign(3, ConnFracStatistics<Scalar>::serializationTestObject());
     result.gas_mass_rates = {31.0};
+    result.wat_mass_rates = {32.0};
 
     return result;
 }
@@ -142,6 +144,7 @@ bool PerfData<Scalar>::try_assign(const PerfData& other)
     this->filtrate_data = other.filtrate_data;
     this->connFracStatistics = other.connFracStatistics;
     this->gas_mass_rates = other.gas_mass_rates;
+    this->wat_mass_rates = other.wat_mass_rates;
 
     return true;
 }
@@ -174,6 +177,7 @@ bool PerfData<Scalar>::operator==(const PerfData& rhs) const
         && (this->filtrate_data == rhs.filtrate_data)
         && (this->connFracStatistics == rhs.connFracStatistics)
         && (this->gas_mass_rates == rhs.gas_mass_rates)
+        && (this->wat_mass_rates == rhs.wat_mass_rates)
         ;
 }
 
