@@ -81,6 +81,7 @@ public:
         serializer(filtrate_data);
         serializer(connFracStatistics);
         serializer(gas_mass_rates);
+        serializer(wat_mass_rates);
     }
 
     bool operator==(const PerfData&) const;
@@ -110,6 +111,7 @@ public:
     std::vector<int> satnum_id{};
     std::vector<std::size_t> ecl_index{};
     std::vector<Scalar> gas_mass_rates{};
+    std::vector<Scalar> wat_mass_rates{};
 
     // The water_throughput, skin_pressure and water_velocity variables are
     // only used for injectors to check the injectivity.
