@@ -26,7 +26,6 @@
 
 #include <opm/simulators/wells/RegionAttributeHelpers.hpp>
 
-#include <opm/simulators/utils/BlackoilPhases.hpp>
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/simulators/utils/ParallelCommunication.hpp>
 
@@ -71,17 +70,14 @@ namespace Opm {
         class SurfaceToReservoirVoidage {
         public:
             using Scalar = typename FluidSystem::Scalar;
-
             /**
              * Constructor.
              *
              * \param[in] region Forward region mapping.  Often corresponds
              * to the "FIPNUM" mapping of an ECLIPSE input deck.
              */
-            SurfaceToReservoirVoidage(const PhaseUsage& phaseUsage,
-                                      const Region&     region)
-                : phaseUsage_(phaseUsage)
-                , rmap_      (region)
+            SurfaceToReservoirVoidage(const Region&     region)
+                : rmap_      (region)
                 , attr_      (rmap_, Attributes())
             {}
 
@@ -139,8 +135,7 @@ namespace Opm {
 
                     // only count oil and gas filled parts of the domain
                     Scalar hydrocarbon = 1.0;
-                    const auto& pu = phaseUsage_;
-                    if (RegionAttributeHelpers::PhaseUsed::water(pu)) {
+                    if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
                         hydrocarbon -= fs.saturation(FluidSystem::waterPhaseIdx).value();
                     }
 
@@ -152,15 +147,15 @@ namespace Opm {
                     if (hydrocarbonPV > 0.) {
                         auto& attr = attributes_hpv[reg];
                         attr.pv += hydrocarbonPV;
-                        if (RegionAttributeHelpers::PhaseUsed::oil(pu) && RegionAttributeHelpers::PhaseUsed::gas(pu)) {
+                        if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
                             attr.rs += fs.Rs().value() * hydrocarbonPV;
                             attr.rv += fs.Rv().value() * hydrocarbonPV;
                         }
-                        if (RegionAttributeHelpers::PhaseUsed::oil(pu)) {
+                        if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
                             attr.pressure += fs.pressure(FluidSystem::oilPhaseIdx).value() * hydrocarbonPV;
                             attr.temperature += fs.temperature(FluidSystem::oilPhaseIdx).value() * hydrocarbonPV;
                         } else {
-                            assert(RegionAttributeHelpers::PhaseUsed::gas(pu));
+                            assert(FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx));
                             attr.pressure += fs.pressure(FluidSystem::gasPhaseIdx).value() * hydrocarbonPV;
                             attr.temperature += fs.temperature(FluidSystem::gasPhaseIdx).value() * hydrocarbonPV;
                         }
@@ -176,18 +171,18 @@ namespace Opm {
                     if (pv_cell > 0.) {
                         auto& attr = attributes_pv[reg];
                         attr.pv += pv_cell;
-                        if (RegionAttributeHelpers::PhaseUsed::oil(pu) && RegionAttributeHelpers::PhaseUsed::gas(pu)) {
+                        if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
                             attr.rs += fs.Rs().value() * pv_cell;
                             attr.rv += fs.Rv().value() * pv_cell;
                         }
-                        if (RegionAttributeHelpers::PhaseUsed::oil(pu)) {
+                        if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
                             attr.pressure += fs.pressure(FluidSystem::oilPhaseIdx).value() * pv_cell;
                             attr.temperature += fs.temperature(FluidSystem::oilPhaseIdx).value() * pv_cell;
-                        } else if (RegionAttributeHelpers::PhaseUsed::gas(pu)) {
+                        } else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
                              attr.pressure += fs.pressure(FluidSystem::gasPhaseIdx).value() * pv_cell;
                              attr.temperature += fs.temperature(FluidSystem::gasPhaseIdx).value() * pv_cell;
                         } else {
-                            assert(RegionAttributeHelpers::PhaseUsed::water(pu));
+                            assert(FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx));
                             attr.pressure += fs.pressure(FluidSystem::waterPhaseIdx).value() * pv_cell;
                             attr.temperature += fs.temperature(FluidSystem::waterPhaseIdx).value() * pv_cell;
                         }
@@ -314,7 +309,7 @@ namespace Opm {
              *
              * \param[in] rsw Dissolved gas/water ratio.
              *
-             * \param[in] rwv Vaporised water/gas ratio.
+             * \param[in] rvw Vaporised water/gas ratio.
              *
              * \param[in] T Temperature.  Unused in non-thermal simulation
              *    runs.
@@ -355,7 +350,7 @@ namespace Opm {
              * \param[in] pvtRegionIdx PVT region of the well
              *
              *
-             * \param[out] double Surface-to-reservoir conversion
+             * \param[out] coeff Surface-to-reservoir conversion
              * coefficients for solvent.
              */
             template <class SolventModule>
@@ -371,11 +366,6 @@ namespace Opm {
             }
 
         private:
-            /**
-             * Fluid property object.
-             */
-            const PhaseUsage phaseUsage_;
-
             /**
              * "Fluid-in-place" region mapping (forward and reverse).
              */
