@@ -26,11 +26,12 @@
 #include <opm/input/eclipse/Schedule/Well/WellEnums.hpp>
 #include <opm/input/eclipse/Schedule/Events.hpp>
 
+#include <opm/material/fluidsystems/PhaseUsageInfo.hpp>
+
 #include <opm/simulators/wells/SegmentState.hpp>
 #include <opm/simulators/wells/PerfData.hpp>
 #include <opm/simulators/wells/ParallelWellInfo.hpp>
-
-#include <opm/simulators/utils/BlackoilPhases.hpp>
+#include <opm/simulators/wells/ALQState.hpp>
 
 namespace Opm {
 
@@ -38,15 +39,19 @@ template<class Scalar> struct PerforationData;
 class SummaryState;
 class Well;
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 class SingleWellState {
 public:
+    static const int waterPhaseIdx = PhaseUsageInfo<IndexTraits>::waterPhaseIdx;
+    static const int oilPhaseIdx = PhaseUsageInfo<IndexTraits>::oilPhaseIdx;
+    static const int gasPhaseIdx = PhaseUsageInfo<IndexTraits>::gasPhaseIdx;
+
     SingleWellState(const std::string& name,
                     const ParallelWellInfo<Scalar>& pinfo,
+                    const PhaseUsageInfo<IndexTraits>& pu,
                     bool is_producer,
                     Scalar presssure_first_connection,
                     const std::vector<PerforationData<Scalar>>& perf_input,
-                    const PhaseUsage& pu,
                     Scalar temp);
 
     static SingleWellState serializationTestObject(const ParallelWellInfo<Scalar>& pinfo);
@@ -65,7 +70,7 @@ public:
         serializer(well_potentials);
         serializer(productivity_index);
         serializer(implicit_ipr_a);
-        serializer(implicit_ipr_b);        
+        serializer(implicit_ipr_b);
         serializer(surface_rates);
         serializer(reservoir_rates);
         serializer(prev_surface_rates);
@@ -76,6 +81,9 @@ public:
         serializer(production_cmode);
         serializer(filtrate_conc);
         serializer(perf_data);
+        serializer(primaryvar);
+        serializer(alq_state);
+        serializer(group_target);
     }
 
     bool operator==(const SingleWellState&) const;
@@ -85,7 +93,7 @@ public:
 
     WellStatus status{WellStatus::OPEN};
     bool producer;
-    PhaseUsage pu;
+    PhaseUsageInfo<IndexTraits> pu;
     Scalar bhp{0};
     Scalar thp{0};
     Scalar temperature{0};
@@ -111,11 +119,13 @@ public:
     std::vector<Scalar> prev_surface_rates;
     PerfData<Scalar> perf_data;
     bool trivial_group_target;
+    std::optional<Scalar> group_target;
     SegmentState<Scalar> segments;
     Events events;
     WellInjectorCMode injection_cmode{WellInjectorCMode::CMODE_UNDEFINED};
     WellProducerCMode production_cmode{WellProducerCMode::CMODE_UNDEFINED};
-
+    std::vector<Scalar> primaryvar;
+    ALQState<Scalar> alq_state;
 
     /// Special purpose method to support dynamically rescaling a well's
     /// CTFs through WELPI.
@@ -131,8 +141,8 @@ public:
     /// This called after ACTIONX is executed to update well rates. The new status is
     /// in ecl_well and st.
     /// \return whether well was switched to a producer
-    bool update_type_and_targets(const Well& ecl_well, const SummaryState& st);
-    void updateStatus(WellStatus status);
+    void update_type_and_targets(const Well& ecl_well, const SummaryState& st);
+    bool updateStatus(WellStatus status);
     void init_timestep(const SingleWellState& other);
     void shut();
     void stop();
@@ -147,6 +157,7 @@ public:
     Scalar sum_microbial_rates() const;
     Scalar sum_oxygen_rates() const;
     Scalar sum_urea_rates() const;
+    Scalar sum_wat_mass_rates() const;
 
     Scalar sum_filtrate_rate() const;
     Scalar sum_filtrate_total() const;
