@@ -37,7 +37,7 @@
 #include <opm/models/blackoil/blackoilextbomodules.hh>
 #include <opm/models/blackoil/blackoilfoammodules.hh>
 #include <opm/models/blackoil/blackoilbrinemodules.hh>
-#include <opm/models/blackoil/blackoilmicpmodules.hh>
+#include <opm/models/blackoil/blackoilbioeffectsmodules.hh>
 
 #include <opm/material/densead/Evaluation.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
@@ -85,11 +85,11 @@ namespace Opm
         using Base::has_foam;
         using Base::has_brine;
         using Base::has_energy;
+        using Base::has_bioeffects;
         using Base::has_micp;
 
         using PolymerModule =  BlackOilPolymerModule<TypeTag>;
         using FoamModule = BlackOilFoamModule<TypeTag>;
-        using BrineModule = BlackOilBrineModule<TypeTag>;
         using typename Base::PressureMatrix;
 
         // number of the conservation equations
@@ -120,26 +120,28 @@ namespace Opm
         using EvalWell = typename StdWellEval::EvalWell;
         using BVectorWell = typename StdWellEval::BVectorWell;
 
+        using IndexTraits = typename FluidSystem::IndexTraitsType;
+        using WellStateType = WellState<Scalar, IndexTraits>;
+
         StandardWell(const Well& well,
                      const ParallelWellInfo<Scalar>& pw_info,
                      const int time_step,
                      const ModelParameters& param,
                      const RateConverterType& rate_converter,
                      const int pvtRegionIdx,
-                     const int num_components,
+                     const int num_conservation_quantities,
                      const int num_phases,
                      const int index_of_well,
                      const std::vector<PerforationData<Scalar>>& perf_data);
 
-        virtual void init(const PhaseUsage* phase_usage_arg,
-                          const std::vector<Scalar>& depth_arg,
+        virtual void init(const std::vector<Scalar>& depth_arg,
                           const Scalar gravity_arg,
                           const std::vector<Scalar>& B_avg,
                           const bool changed_to_open_this_step) override;
 
         /// check whether the well equations get converged for this well
         virtual ConvergenceReport getWellConvergence(const Simulator& simulator,
-                                                     const WellState<Scalar>& well_state,
+                                                     const WellStateType& well_state,
                                                      const std::vector<Scalar>& B_avg,
                                                      DeferredLogger& deferred_logger,
                                                      const bool relax_tolerance) const override;
@@ -153,30 +155,30 @@ namespace Opm
         /// xw to update Well State
         void recoverWellSolutionAndUpdateWellState(const Simulator& simulator,
                                                    const BVector& x,
-                                                   WellState<Scalar>& well_state,
+                                                   WellStateType& well_state,
                                                    DeferredLogger& deferred_logger) override;
 
         /// computing the well potentials for group control
         void computeWellPotentials(const Simulator& simulator,
-                                   const WellState<Scalar>& well_state,
+                                   const WellStateType& well_state,
                                    std::vector<Scalar>& well_potentials,
                                    DeferredLogger& deferred_logger) /* const */ override;
 
         void updatePrimaryVariables(const Simulator& simulator,
-                                    const WellState<Scalar>& well_state,
+                                    const WellStateType& well_state,
                                     DeferredLogger& deferred_logger) override;
 
         void solveEqAndUpdateWellState(const Simulator& simulator,
-                                       WellState<Scalar>& well_state,
+                                       WellStateType& well_state,
                                        DeferredLogger& deferred_logger) override;
 
         void calculateExplicitQuantities(const Simulator& simulator,
-                                         const WellState<Scalar>& well_state,
+                                         const WellStateType& well_state,
                                          DeferredLogger& deferred_logger) override; // should be const?
 
         void updateProductivityIndex(const Simulator& simulator,
                                      const WellProdIndexCalculator<Scalar>& wellPICalc,
-                                     WellState<Scalar>& well_state,
+                                     WellStateType& well_state,
                                      DeferredLogger& deferred_logger) const override;
 
         Scalar connectionDensity(const int globalConnIdx,
@@ -188,14 +190,14 @@ namespace Opm
                                       const BVector& x,
                                       const int pressureVarIndex,
                                       const bool use_well_weights,
-                                      const WellState<Scalar>& well_state) const override;
+                                      const WellStateType& well_state) const override;
 
         // iterate well equations with the specified control until converged
         bool iterateWellEqWithControl(const Simulator& simulator,
                                       const double dt,
                                       const Well::InjectionControls& inj_controls,
                                       const Well::ProductionControls& prod_controls,
-                                      WellState<Scalar>& well_state,
+                                      WellStateType& well_state,
                                       const GroupState<Scalar>& group_state,
                                       DeferredLogger& deferred_logger) override;
 
@@ -204,18 +206,12 @@ namespace Opm
                                         const double dt,
                                         const Well::InjectionControls& inj_controls,
                                         const Well::ProductionControls& prod_controls,
-                                        WellState<Scalar>& well_state,
+                                        WellStateType& well_state,
                                         const GroupState<Scalar>& group_state,
-                                        DeferredLogger& deferred_logger, 
+                                        DeferredLogger& deferred_logger,
                                         const bool fixed_control = false,
                                         const bool fixed_status = false) override;
 
-        /// \brief Wether the Jacobian will also have well contributions in it.
-        bool jacobianContainsWellContributions() const override
-        {
-            return this->param_.matrix_add_well_contributions_;
-        }
-
         /* returns BHP */
         Scalar computeWellRatesAndBhpWithThpAlqProd(const Simulator& ebos_simulator,
                                                     const SummaryState &summary_state,
@@ -237,7 +233,7 @@ namespace Opm
                                         bool iterate_if_no_solution) const override;
 
         void updateIPRImplicit(const Simulator& simulator,
-                               WellState<Scalar>& well_state,
+                               WellStateType& well_state,
                                DeferredLogger& deferred_logger) override;
 
         void computeWellRatesWithBhp(const Simulator& ebosSimulator,
@@ -246,7 +242,6 @@ namespace Opm
                                      DeferredLogger& deferred_logger) const override;
 
         // NOTE: These cannot be protected since they are used by GasLiftRuntime
-        using Base::phaseUsage;
         using Base::vfp_properties_;
 
         std::vector<Scalar>
@@ -263,7 +258,7 @@ namespace Opm
         // updating the well_state based on well solution dwells
         void updateWellState(const Simulator& simulator,
                              const BVectorWell& dwells,
-                             WellState<Scalar>& well_state,
+                             WellStateType& well_state,
                              DeferredLogger& deferred_logger);
 
         using WellConnectionProps = typename StdWellEval::StdWellConnections::Properties;
@@ -272,15 +267,15 @@ namespace Opm
         // pressure difference between well connections.
         WellConnectionProps
         computePropertiesForWellConnectionPressures(const Simulator& simulator,
-                                                    const WellState<Scalar>& well_state) const;
+                                                    const WellStateType& well_state) const;
 
         void computeWellConnectionDensitesPressures(const Simulator& simulator,
-                                                    const WellState<Scalar>& well_state,
+                                                    const WellStateType& well_state,
                                                     const WellConnectionProps& props,
                                                     DeferredLogger& deferred_logger);
 
         void computeWellConnectionPressures(const Simulator& simulator,
-                                            const WellState<Scalar>& well_state,
+                                            const WellStateType& well_state,
                                             DeferredLogger& deferred_logger);
 
         template<class Value>
@@ -320,13 +315,15 @@ namespace Opm
         std::vector<Scalar>
         computeWellPotentialWithTHP(const Simulator& ebosSimulator,
                                     DeferredLogger& deferred_logger,
-                                    const WellState<Scalar>& well_state) const;
+                                    const WellStateType& well_state) const;
 
         bool computeWellPotentialsImplicit(const Simulator& ebos_simulator,
-                                           const WellState<Scalar>& well_state,
+                                           const WellStateType& well_state,
                                            std::vector<Scalar>& well_potentials,
-                                           DeferredLogger& deferred_logger) const;               
+                                           DeferredLogger& deferred_logger) const;
 
+        // return the density at the perforation[0] of the rank owning this well,
+        // value is cached to minimize the number of broadcasts
         Scalar getRefDensity() const override;
 
         // get the mobility for specific perforation
@@ -345,7 +342,7 @@ namespace Opm
                                           const bool stop_or_zero_rate_target,
                                           DeferredLogger& deferred_logger);
 
-        void updateWellStateFromPrimaryVariables(WellState<Scalar>& well_state,
+        void updateWellStateFromPrimaryVariables(WellStateType& well_state,
                                                  const SummaryState& summary_state,
                                                  DeferredLogger& deferred_logger) const;
 
@@ -353,7 +350,7 @@ namespace Opm
                                             const double dt,
                                             const Well::InjectionControls& inj_controls,
                                             const Well::ProductionControls& prod_controls,
-                                            WellState<Scalar>& well_state,
+                                            WellStateType& well_state,
                                             const GroupState<Scalar>& group_state,
                                             DeferredLogger& deferred_logger) override;
 
@@ -361,13 +358,13 @@ namespace Opm
                                                 const double dt,
                                                 const Well::InjectionControls& inj_controls,
                                                 const Well::ProductionControls& prod_controls,
-                                                WellState<Scalar>& well_state,
+                                                WellStateType& well_state,
                                                 const GroupState<Scalar>& group_state,
                                                 DeferredLogger& deferred_logger);
 
         void calculateSinglePerf(const Simulator& simulator,
                                  const int perf,
-                                 WellState<Scalar>& well_state,
+                                 WellStateType& well_state,
                                  std::vector<RateVector>& connectionRates,
                                  std::vector<EvalWell>& cq_s,
                                  EvalWell& water_flux_s,
@@ -375,13 +372,13 @@ namespace Opm
                                  DeferredLogger& deferred_logger) const;
 
         // check whether the well is operable under BHP limit with current reservoir condition
-        void checkOperabilityUnderBHPLimit(const WellState<Scalar>& well_state,
+        void checkOperabilityUnderBHPLimit(const WellStateType& well_state,
                                            const Simulator& simulator,
                                            DeferredLogger& deferred_logger) override;
 
         // check whether the well is operable under THP limit with current reservoir condition
         void checkOperabilityUnderTHPLimit(const Simulator& simulator,
-                                           const WellState<Scalar>& well_state,
+                                           const WellStateType& well_state,
                                            DeferredLogger& deferred_logger) override;
 
         // updating the inflow based on the current reservoir condition
@@ -392,11 +389,6 @@ namespace Opm
         // be able to produce/inject .
         bool allDrawDownWrongDirection(const Simulator& simulator) const;
 
-        // whether the well can produce / inject based on the current well state (bhp)
-        bool canProduceInjectWithCurrentBhp(const Simulator& simulator,
-                                            const WellState<Scalar>& well_state,
-                                            DeferredLogger& deferred_logger);
-
         // turn on crossflow to avoid singular well equations
         // when the well is banned from cross-flow and the BHP is not properly initialized,
         // we turn on crossflow to avoid singular well equations. It can result in wrong-signed
@@ -430,13 +422,13 @@ namespace Opm
 
         // handle the extra equations for polymer injectivity study
         void handleInjectivityEquations(const Simulator& simulator,
-                                        const WellState<Scalar>& well_state,
+                                        const WellStateType& well_state,
                                         const int perf,
                                         const EvalWell& water_flux_s,
                                         DeferredLogger& deferred_logger);
 
         void updateWaterThroughput(const double dt,
-                                   WellState<Scalar>& well_state) const override;
+                                   WellStateType& well_state) const override;
 
         // checking convergence of extra equations, if there are any
         void checkConvergenceExtraEqs(const std::vector<Scalar>& res,
@@ -445,13 +437,13 @@ namespace Opm
         // updating the connectionRates_ related polymer molecular weight
         void updateConnectionRatePolyMW(const EvalWell& cq_s_poly,
                                         const IntensiveQuantities& int_quants,
-                                        const WellState<Scalar>& well_state,
+                                        const WellStateType& well_state,
                                         const int perf,
                                         std::vector<RateVector>& connectionRates,
                                         DeferredLogger& deferred_logger) const;
 
         std::optional<Scalar>
-        computeBhpAtThpLimitProd(const WellState<Scalar>& well_state,
+        computeBhpAtThpLimitProd(const WellStateType& well_state,
                                  const Simulator& simulator,
                                  const SummaryState& summary_state,
                                  DeferredLogger& deferred_logger) const;
@@ -462,10 +454,12 @@ namespace Opm
                                 DeferredLogger& deferred_logger) const;
 
     private:
-        Eval connectionRateEnergy(const Scalar maxOilSaturation,
-                                  const std::vector<EvalWell>& cq_s,
+        Eval connectionRateEnergy(const std::vector<EvalWell>& cq_s,
                                   const IntensiveQuantities& intQuants,
                                   DeferredLogger& deferred_logger) const;
+
+        // density of the first perforation, might not be from this rank
+        Scalar cachedRefDensity{0};
     };
 
 }
