@@ -31,7 +31,7 @@
 #include <opm/material/densead/Evaluation.hpp>
 #include <opm/material/fluidsystems/BlackOilFluidSystem.hpp>
 
-#include <opm/models/blackoil/blackoilindices.hh>
+#include <opm/models/blackoil/blackoilvariableandequationindices.hh>
 #include <opm/models/blackoil/blackoilonephaseindices.hh>
 #include <opm/models/blackoil/blackoiltwophaseindices.hh>
 
@@ -119,17 +119,12 @@ resize(const int numWellEq)
 
 template<class FluidSystem, class Indices>
 void StandardWellPrimaryVariables<FluidSystem,Indices>::
-update(const WellState<Scalar>& well_state,
+update(const WellState<Scalar, IndexTraits>& well_state,
        const bool stop_or_zero_rate_target,
        DeferredLogger& deferred_logger)
 {
-    static constexpr int Water = BlackoilPhases::Aqua;
-    static constexpr int Oil = BlackoilPhases::Liquid;
-    static constexpr int Gas = BlackoilPhases::Vapour;
-
     const int well_index = well_.indexOfWell();
     const int np = well_.numPhases();
-    const auto& pu = well_.phaseUsage();
     const auto& ws = well_state.well(well_index);
     // the weighted total well rate
     Scalar total_well_rate = 0.0;
@@ -141,15 +136,21 @@ update(const WellState<Scalar>& well_state,
     // under surface condition is used here
     if (well_.isInjector()) {
         switch (well_.wellEcl().injectorType()) {
-        case InjectorType::WATER:
-            value_[WQTotal] = ws.surface_rates[pu.phase_pos[Water]];
+        case InjectorType::WATER: {
+            const int phase_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+            value_[WQTotal] = ws.surface_rates[phase_pos];
             break;
-        case InjectorType::GAS:
-            value_[WQTotal] = ws.surface_rates[pu.phase_pos[Gas]];
+        }
+        case InjectorType::GAS: {
+            const int phase_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            value_[WQTotal] = ws.surface_rates[phase_pos];
             break;
-        case InjectorType::OIL:
-            value_[WQTotal] = ws.surface_rates[pu.phase_pos[Oil]];
+        }
+        case InjectorType::OIL: {
+            const int phase_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+            value_[WQTotal] = ws.surface_rates[phase_pos];
             break;
+        }
         case InjectorType::MULTI:
             // Not supported.
             deferred_logger.warning("MULTI_PHASE_INJECTOR_NOT_SUPPORTED",
@@ -163,19 +164,30 @@ update(const WellState<Scalar>& well_state,
             }
     }
 
-    if (std::abs(total_well_rate) > 0.) {
+    if (ws.primaryvar.size() > 0) {
         if constexpr (has_wfrac_variable) {
-            value_[WFrac] = well_.scalingFactor(pu.phase_pos[Water]) * ws.surface_rates[pu.phase_pos[Water]] / total_well_rate;
+             value_[WFrac] = ws.primaryvar[WFrac];
         }
         if constexpr (has_gfrac_variable) {
-            value_[GFrac] = well_.scalingFactor(pu.phase_pos[Gas]) *
-                            (ws.surface_rates[pu.phase_pos[Gas]] -
+             value_[GFrac] = ws.primaryvar[GFrac];
+        }
+        if constexpr (Indices::enableSolvent) {
+            value_[SFrac] = ws.primaryvar[SFrac];
+         }
+    } else if (std::abs(total_well_rate) > 0.) {
+        if constexpr (has_wfrac_variable) {
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+            value_[WFrac] = well_.scalingFactor(water_pos) * ws.surface_rates[water_pos] / total_well_rate;
+        }
+        if constexpr (has_gfrac_variable) {
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            value_[GFrac] = well_.scalingFactor(gas_pos) *
+                            (ws.surface_rates[gas_pos] -
                              (Indices::enableSolvent ? ws.sum_solvent_rates() : 0.0) ) / total_well_rate ;
         }
         if constexpr (Indices::enableSolvent) {
             value_[SFrac] = well_.scalingFactor(Indices::contiSolventEqIdx) * ws.sum_solvent_rates() / total_well_rate ;
         }
-
     } else { // total_well_rate == 0
         if (well_.isInjector()) {
             // only single phase injection handled
@@ -228,16 +240,16 @@ update(const WellState<Scalar>& well_state,
 
 template<class FluidSystem, class Indices>
 void StandardWellPrimaryVariables<FluidSystem,Indices>::
-updatePolyMW(const WellState<Scalar>& well_state)
+updatePolyMW(const WellState<Scalar, IndexTraits>& well_state)
 {
     if (well_.isInjector()) {
         const auto& ws = well_state.well(well_.indexOfWell());
         const auto& perf_data = ws.perf_data;
         const auto& water_velocity = perf_data.water_velocity;
         const auto& skin_pressure = perf_data.skin_pressure;
-        for (int perf = 0; perf < well_.numPerfs(); ++perf) {
+        for (int perf = 0; perf < well_.numLocalPerfs(); ++perf) {
             value_[Bhp + 1 + perf] = water_velocity[perf];
-            value_[Bhp + 1 + well_.numPerfs() + perf] = skin_pressure[perf];
+            value_[Bhp + 1 + well_.numLocalPerfs() + perf] = skin_pressure[perf];
         }
     }
     setEvaluationsFromValues();
@@ -311,9 +323,9 @@ void StandardWellPrimaryVariables<FluidSystem,Indices>::
 updateNewtonPolyMW(const BVectorWell& dwells)
 {
     if (well_.isInjector()) {
-        for (int perf = 0; perf < well_.numPerfs(); ++perf) {
+        for (int perf = 0; perf < well_.numLocalPerfs(); ++perf) {
             const int wat_vel_index = Bhp + 1 + perf;
-            const int pskin_index = Bhp + 1 + well_.numPerfs() + perf;
+            const int pskin_index = Bhp + 1 + well_.numLocalPerfs() + perf;
 
             const Scalar relaxation_factor = 0.9;
             const Scalar dx_wat_vel = dwells[0][wat_vel_index];
@@ -328,28 +340,23 @@ updateNewtonPolyMW(const BVectorWell& dwells)
 
 template<class FluidSystem, class Indices>
 void StandardWellPrimaryVariables<FluidSystem,Indices>::
-copyToWellState(WellState<Scalar>& well_state,
+copyToWellState(WellState<Scalar, IndexTraits>& well_state,
                 DeferredLogger& deferred_logger) const
 {
-    static constexpr int Water = BlackoilPhases::Aqua;
-    static constexpr int Oil = BlackoilPhases::Liquid;
-    static constexpr int Gas = BlackoilPhases::Vapour;
-
-    const PhaseUsage& pu = well_.phaseUsage();
     std::vector<Scalar> F(well_.numPhases(), 0.0);
     [[maybe_unused]] Scalar F_solvent = 0.0;
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        const int oil_pos = pu.phase_pos[Oil];
+        const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
         F[oil_pos] = 1.0;
 
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-            const int water_pos = pu.phase_pos[Water];
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
             F[water_pos] = value_[WFrac];
             F[oil_pos] -= F[water_pos];
         }
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            const int gas_pos = pu.phase_pos[Gas];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
             F[gas_pos] = value_[GFrac];
             F[oil_pos] -= F[gas_pos];
         }
@@ -360,11 +367,11 @@ copyToWellState(WellState<Scalar>& well_state,
         }
     }
     else if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        const int water_pos = pu.phase_pos[Water];
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
         F[water_pos] = 1.0;
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            const int gas_pos = pu.phase_pos[Gas];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
             F[gas_pos] = value_[GFrac];
             F[water_pos] -= F[gas_pos];
         }
@@ -375,7 +382,7 @@ copyToWellState(WellState<Scalar>& well_state,
         }
     }
     else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        const int gas_pos = pu.phase_pos[Gas];
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
         F[gas_pos] = 1.0;
     }
 
@@ -395,10 +402,12 @@ copyToWellState(WellState<Scalar>& well_state,
     // More testing is needed to make sure this is correct for well groups and THP.
     if constexpr (Indices::enableSolvent) {
         F_solvent /= well_.scalingFactor(Indices::contiSolventEqIdx);
-        F[pu.phase_pos[Gas]] += F_solvent;
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        F[gas_pos] += F_solvent;
     }
 
     auto& ws = well_state.well(well_.indexOfWell());
+    ws.primaryvar = value_;
     ws.bhp = value_[Bhp];
 
     // calculate the phase rates based on the primary variables
@@ -413,15 +422,21 @@ copyToWellState(WellState<Scalar>& well_state,
             ws.surface_rates[p] = 0.0;
         }
         switch (well_.wellEcl().injectorType()) {
-        case InjectorType::WATER:
-            ws.surface_rates[pu.phase_pos[Water]] = value_[WQTotal];
+        case InjectorType::WATER: {
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+            ws.surface_rates[water_pos] = value_[WQTotal];
             break;
-        case InjectorType::GAS:
-            ws.surface_rates[pu.phase_pos[Gas]] = value_[WQTotal];
+        }
+        case InjectorType::GAS: {
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            ws.surface_rates[gas_pos] = value_[WQTotal];
             break;
-        case InjectorType::OIL:
-            ws.surface_rates[pu.phase_pos[Oil]] = value_[WQTotal];
+        }
+        case InjectorType::OIL: {
+            const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+            ws.surface_rates[oil_pos] = value_[WQTotal];
             break;
+        }
         case InjectorType::MULTI:
             // Not supported.
             deferred_logger.warning("MULTI_PHASE_INJECTOR_NOT_SUPPORTED",
@@ -433,16 +448,16 @@ copyToWellState(WellState<Scalar>& well_state,
 
 template<class FluidSystem, class Indices>
 void StandardWellPrimaryVariables<FluidSystem,Indices>::
-copyToWellStatePolyMW(WellState<Scalar>& well_state) const
+copyToWellStatePolyMW(WellState<Scalar, IndexTraits>& well_state) const
 {
     if (well_.isInjector()) {
         auto& ws = well_state.well(well_.indexOfWell());
         auto& perf_data = ws.perf_data;
         auto& perf_water_velocity = perf_data.water_velocity;
         auto& perf_skin_pressure = perf_data.skin_pressure;
-        for (int perf = 0; perf < well_.numPerfs(); ++perf) {
+        for (int perf = 0; perf < well_.numLocalPerfs(); ++perf) {
             perf_water_velocity[perf] = value_[Bhp + 1 + perf];
-            perf_skin_pressure[perf] = value_[Bhp + 1 + well_.numPerfs() + perf];
+            perf_skin_pressure[perf] = value_[Bhp + 1 + well_.numLocalPerfs() + perf];
         }
     }
 }
@@ -456,7 +471,7 @@ volumeFraction(const int compIdx) const
         return EvalWell(numWellEq_ + Indices::numEq, 1.0);
     }
 
-    if (has_gfrac_variable && compIdx == Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx)) {
+    if (has_gfrac_variable && compIdx == FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx)) {
         return evaluation_[GFrac];
     }
 
@@ -465,7 +480,7 @@ volumeFraction(const int compIdx) const
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        if (has_wfrac_variable && compIdx == Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx)) {
+        if (has_wfrac_variable && compIdx == FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx)) {
             return evaluation_[WFrac];
         }
     }
@@ -489,7 +504,7 @@ typename StandardWellPrimaryVariables<FluidSystem,Indices>::EvalWell
 StandardWellPrimaryVariables<FluidSystem,Indices>::
 volumeFractionScaled(const int compIdx) const
 {
-    const int legacyCompIdx = well_.modelCompIdxToFlowCompIdx(compIdx);
+    const int legacyCompIdx = FluidSystem::activeCompToActivePhaseIdx(compIdx);
     const Scalar scal = well_.scalingFactor(legacyCompIdx);
     if (scal > 0)
         return this->volumeFraction(compIdx) / scal;
@@ -504,7 +519,7 @@ StandardWellPrimaryVariables<FluidSystem,Indices>::
 surfaceVolumeFraction(const int compIdx) const
 {
     EvalWell sum_volume_fraction_scaled(numWellEq_ + Indices::numEq, 0.);
-    for (int idx = 0; idx < well_.numComponents(); ++idx) {
+    for (int idx = 0; idx < well_.numConservationQuantities(); ++idx) {
         sum_volume_fraction_scaled += this->volumeFractionScaled(idx);
     }
 
@@ -519,32 +534,32 @@ StandardWellPrimaryVariables<FluidSystem,Indices>::
 getQs(const int comp_idx) const
 {
     // Note: currently, the WQTotal definition is still depends on Injector/Producer.
-    assert(comp_idx < well_.numComponents());
+    assert(comp_idx < well_.numConservationQuantities());
 
     if (well_.isInjector()) { // only single phase injection
         Scalar inj_frac = 0.0;
         switch (well_.wellEcl().injectorType()) {
         case InjectorType::WATER:
-            if (comp_idx == int(Indices::canonicalToActiveComponentIndex(FluidSystem::waterCompIdx))) {
+            if (comp_idx == int(FluidSystem::canonicalToActiveCompIdx(FluidSystem::waterCompIdx))) {
                 inj_frac = 1.0;
             }
             break;
         case InjectorType::GAS:
             if (Indices::enableSolvent && comp_idx == Indices::contiSolventEqIdx) { // solvent
                 inj_frac = well_.wsolvent();
-            } else if (comp_idx == int(Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx))) {
+            } else if (comp_idx == int(FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx))) {
                 inj_frac = 1.0 - well_.rsRvInj();
                 if constexpr (Indices::enableSolvent) {
                     inj_frac -= well_.wsolvent();
                 }
-            } else if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && comp_idx == int(Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx))) {
+            } else if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx) && comp_idx == int(FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx))) {
                 inj_frac = well_.rsRvInj();
             }
             break;
         case InjectorType::OIL:
-            if (comp_idx == int(Indices::canonicalToActiveComponentIndex(FluidSystem::oilCompIdx))) {
+            if (comp_idx == int(FluidSystem::canonicalToActiveCompIdx(FluidSystem::oilCompIdx))) {
                 inj_frac = 1.0 - well_.rsRvInj();
-            } else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) && comp_idx == int(Indices::canonicalToActiveComponentIndex(FluidSystem::gasCompIdx))) {
+            } else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx) && comp_idx == int(FluidSystem::canonicalToActiveCompIdx(FluidSystem::gasCompIdx))) {
                 inj_frac = well_.rsRvInj();
             }
             break;
@@ -564,99 +579,111 @@ template<class FluidSystem, class Indices>
 void StandardWellPrimaryVariables<FluidSystem,Indices>::
 processFractions()
 {
-    static constexpr int Water = BlackoilPhases::Aqua;
-    static constexpr int Oil = BlackoilPhases::Liquid;
-    static constexpr int Gas = BlackoilPhases::Vapour;
-
-    const auto pu = well_.phaseUsage();
     std::vector<Scalar> F(well_.numPhases(), 0.0);
 
     [[maybe_unused]] Scalar F_solvent = 0.0;
 
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        F[pu.phase_pos[Oil]] = 1.0;
+        const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+        F[oil_pos] = 1.0;
 
         if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-            F[pu.phase_pos[Water]] = value_[WFrac];
-            F[pu.phase_pos[Oil]] -= F[pu.phase_pos[Water]];
+            const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+            F[water_pos] = value_[WFrac];
+            F[oil_pos] -= F[water_pos];
         }
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            F[pu.phase_pos[Gas]] = value_[GFrac];
-            F[pu.phase_pos[Oil]] -= F[pu.phase_pos[Gas]];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            F[gas_pos] = value_[GFrac];
+            F[oil_pos] -= F[gas_pos];
         }
         if constexpr (Indices::enableSolvent) {
             F_solvent = value_[SFrac];
-            F[pu.phase_pos[Oil]] -= F_solvent;
+            F[oil_pos] -= F_solvent;
         }
     }
     else if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        F[pu.phase_pos[Water]] = 1.0;
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+        F[water_pos] = 1.0;
 
         if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-            F[pu.phase_pos[Gas]] = value_[GFrac];
-            F[pu.phase_pos[Water]] -= F[pu.phase_pos[Gas]];
+            const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+            F[gas_pos] = value_[GFrac];
+            F[water_pos] -= F[gas_pos];
         }
         if constexpr (Indices::enableSolvent) {
             F_solvent = value_[SFrac];
-            F[pu.phase_pos[Water]] -= F_solvent;
+            F[water_pos] -= F_solvent;
         }
     }
     else if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        F[pu.phase_pos[Gas]] = 1.0;
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        F[gas_pos] = 1.0;
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-        if (F[pu.phase_pos[Water]] < 0.0) {
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+        if (F[water_pos] < 0.0) {
             if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-                    F[pu.phase_pos[Gas]] /= (1.0 - F[pu.phase_pos[Water]]);
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+                    F[gas_pos] /= (1.0 - F[water_pos]);
             }
             if constexpr (Indices::enableSolvent) {
-                F_solvent /= (1.0 - F[pu.phase_pos[Water]]);
+                F_solvent /= (1.0 - F[water_pos]);
             }
             if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-                F[pu.phase_pos[Oil]] /= (1.0 - F[pu.phase_pos[Water]]);
+                const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+                F[oil_pos] /= (1.0 - F[water_pos]);
             }
-            F[pu.phase_pos[Water]] = 0.0;
+            F[water_pos] = 0.0;
         }
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-        if (F[pu.phase_pos[Gas]] < 0.0) {
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        if (F[gas_pos] < 0.0) {
             if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-                F[pu.phase_pos[Water]] /= (1.0 - F[pu.phase_pos[Gas]]);
+                const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+                F[water_pos] /= (1.0 - F[gas_pos]);
             }
             if constexpr (Indices::enableSolvent) {
-                F_solvent /= (1.0 - F[pu.phase_pos[Gas]]);
+                F_solvent /= (1.0 - F[gas_pos]);
             }
             if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-                F[pu.phase_pos[Oil]] /= (1.0 - F[pu.phase_pos[Gas]]);
+                const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+                F[oil_pos] /= (1.0 - F[gas_pos]);
             }
-            F[pu.phase_pos[Gas]] = 0.0;
+            F[gas_pos] = 0.0;
         }
     }
 
     if (FluidSystem::phaseIsActive(FluidSystem::oilPhaseIdx)) {
-        if (F[pu.phase_pos[Oil]] < 0.0) {
+        const int oil_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::oilPhaseIdx);
+        if (F[oil_pos] < 0.0) {
             if (FluidSystem::phaseIsActive(FluidSystem::waterPhaseIdx)) {
-                F[pu.phase_pos[Water]] /= (1.0 - F[pu.phase_pos[Oil]]);
+                const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+                F[water_pos] /= (1.0 - F[oil_pos]);
             }
             if (FluidSystem::phaseIsActive(FluidSystem::gasPhaseIdx)) {
-                F[pu.phase_pos[Gas]] /= (1.0 - F[pu.phase_pos[Oil]]);
+                const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+                F[gas_pos] /= (1.0 - F[oil_pos]);
             }
             if constexpr (Indices::enableSolvent) {
-                F_solvent /= (1.0 - F[pu.phase_pos[Oil]]);
+                F_solvent /= (1.0 - F[oil_pos]);
             }
-            F[pu.phase_pos[Oil]] = 0.0;
+            F[oil_pos] = 0.0;
         }
     }
 
     if constexpr (has_wfrac_variable) {
-        value_[WFrac] = F[pu.phase_pos[Water]];
+        const int water_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::waterPhaseIdx);
+        value_[WFrac] = F[water_pos];
     }
 
     if constexpr (has_gfrac_variable) {
-        value_[GFrac] = F[pu.phase_pos[Gas]];
+        const int gas_pos = FluidSystem::canonicalToActivePhaseIdx(FluidSystem::gasPhaseIdx);
+        value_[GFrac] = F[gas_pos];
     }
     if constexpr (Indices::enableSolvent) {
         value_[SFrac] = F_solvent;
@@ -731,43 +758,12 @@ checkFinite(DeferredLogger& deferred_logger) const
     }
 }
 
-template<class Scalar>
-using FS = BlackOilFluidSystem<Scalar,BlackOilDefaultIndexTraits>;
+#include <opm/simulators/utils/InstantiationIndicesMacros.hpp>
 
-#define INSTANTIATE(T,...) \
-    template class StandardWellPrimaryVariables<FS<T>,__VA_ARGS__>;
-
-#define INSTANTIATE_TYPE(T)                                                  \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,0u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,1u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilOnePhaseIndices<0u,0u,0u,0u,false,false,0u,1u,5u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,1u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,1u,0u,false,true,0u,2u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,false,0u,1u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,true,0u,0u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,0u,false,true,0u,2u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,2u,0u,false,false,0u,2u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<0u,0u,0u,1u,false,true,0u,0u,0u>)  \
-    INSTANTIATE(T,BlackOilTwoPhaseIndices<1u,0u,0u,0u,false,false,0u,0u,0u>) \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<1u,0u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,1u,0u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,1u,0u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,false,0u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,true,false,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,0u,false,true,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,true,0u,0u>)             \
-    INSTANTIATE(T,BlackOilIndices<0u,0u,0u,1u,false,false,1u,0u>)            \
-    INSTANTIATE(T,BlackOilIndices<1u,0u,0u,0u,true,false,0u,0u>)
-
-INSTANTIATE_TYPE(double)
+INSTANTIATE_TYPE_INDICES(StandardWellPrimaryVariables, double)
 
 #if FLOW_INSTANTIATE_FLOAT
-INSTANTIATE_TYPE(float)
+INSTANTIATE_TYPE_INDICES(StandardWellPrimaryVariables, float)
 #endif
 
 }
