@@ -22,6 +22,8 @@
 #include <config.h>
 #include <opm/simulators/wells/WellConvergence.hpp>
 
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
+
 #include <opm/simulators/timestepping/ConvergenceReport.hpp>
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
 #include <opm/simulators/wells/WellInterfaceGeneric.hpp>
@@ -30,18 +32,18 @@
 #include <cmath>
 #include <stdexcept>
 
-namespace Opm
-{
+namespace Opm {
 
-void WellConvergence::
-checkConvergenceControlEq(const WellState& well_state,
+template<typename Scalar, typename IndexTraits>
+void WellConvergence<Scalar, IndexTraits>::
+checkConvergenceControlEq(const WellState<Scalar, IndexTraits>& well_state,
                           const Tolerances& tolerances,
-                          const double well_control_residual,
-                          const bool well_is_stopped, 
+                          const Scalar well_control_residual,
+                          const bool well_is_stopped,
                           ConvergenceReport& report,
                           DeferredLogger& deferred_logger) const
 {
-    double control_tolerance = 0.;
+    Scalar control_tolerance = 0.;
     using CR = ConvergenceReport;
     CR::WellFailure::Type ctrltype = CR::WellFailure::Type::Invalid;
 
@@ -120,21 +122,21 @@ checkConvergenceControlEq(const WellState& well_state,
     }
 }
 
-void
-WellConvergence::
-checkConvergencePolyMW(const std::vector<double>& res,
+template<typename Scalar, typename IndexTraits>
+void WellConvergence<Scalar, IndexTraits>::
+checkConvergencePolyMW(const std::vector<Scalar>& res,
                        const int Bhp,
-                       const double maxResidualAllowed,
+                       const Scalar maxResidualAllowed,
                        ConvergenceReport& report) const
 {
   if (well_.isInjector()) {
       //  checking the convergence of the perforation rates
-      const double wat_vel_tol = 1.e-8;
+      const Scalar wat_vel_tol = 1.e-8;
       const int dummy_component = -1;
       using CR = ConvergenceReport;
       const auto wat_vel_failure_type = CR::WellFailure::Type::MassBalance;
-      for (int perf = 0; perf < well_.numPerfs(); ++perf) {
-          const double wat_vel_residual = res[Bhp + 1 + perf];
+      for (int perf = 0; perf < well_.numLocalPerfs(); ++perf) {
+          const Scalar wat_vel_residual = res[Bhp + 1 + perf];
           if (std::isnan(wat_vel_residual)) {
               report.setWellFailed({wat_vel_failure_type, CR::Severity::NotANumber, dummy_component, well_.name()});
           } else if (wat_vel_residual > maxResidualAllowed * 10.) {
@@ -145,10 +147,10 @@ checkConvergencePolyMW(const std::vector<double>& res,
       }
 
       // checking the convergence of the skin pressure
-      const double pskin_tol = 1000.; // 1000 pascal
+      const Scalar pskin_tol = 1000.; // 1000 pascal
       const auto pskin_failure_type = CR::WellFailure::Type::Pressure;
-      for (int perf = 0; perf < well_.numPerfs(); ++perf) {
-          const double pskin_residual = res[Bhp + 1 + perf + well_.numPerfs()];
+      for (int perf = 0; perf < well_.numLocalPerfs(); ++perf) {
+          const Scalar pskin_residual = res[Bhp + 1 + perf + well_.numLocalPerfs()];
           if (std::isnan(pskin_residual)) {
               report.setWellFailed({pskin_failure_type, CR::Severity::NotANumber, dummy_component, well_.name()});
           } else if (pskin_residual > maxResidualAllowed * 10.) {
@@ -160,4 +162,10 @@ checkConvergencePolyMW(const std::vector<double>& res,
   }
 }
 
+template class WellConvergence<double, BlackOilDefaultFluidSystemIndices>;
+
+#if FLOW_INSTANTIATE_FLOAT
+template class WellConvergence<float, BlackOilDefaultFluidSystemIndices>;
+#endif
+
 }
