@@ -25,25 +25,26 @@
 
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
+
 #include <opm/simulators/wells/WellGroupHelpers.hpp>
 #include <opm/simulators/wells/WellInterfaceGeneric.hpp>
 #include <opm/simulators/wells/WellState.hpp>
 
-#include <opm/simulators/utils/BlackoilPhases.hpp>
-
 namespace Opm
 {
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 std::pair<bool, Scalar>
-WellGroupConstraints<Scalar>::
+WellGroupConstraints<Scalar, IndexTraits>::
 checkGroupConstraintsInj(const Group& group,
-                         const WellState<Scalar>& well_state,
+                         const WellState<Scalar, IndexTraits>& well_state,
                          const GroupState<Scalar>& group_state,
                          const Scalar efficiencyFactor,
                          const Schedule& schedule,
                          const SummaryState& summaryState,
                          const RateConvFunc& rateConverter,
+                         const bool check_guide_rate,
                          DeferredLogger& deferred_logger) const
 {
     // Translate injector type from control to Phase.
@@ -71,12 +72,12 @@ checkGroupConstraintsInj(const Group& group,
     }
 
     // Make conversion factors for RESV <-> surface rates.
-    std::vector<Scalar> resv_coeff(well_.phaseUsage().num_phases, 1.0);
+    std::vector<Scalar> resv_coeff(well_.phaseUsage().numActivePhases(), 1.0);
     rateConverter(0, well_.pvtRegionIdx(), group.name(), resv_coeff); // FIPNUM region 0 here, should use FIPNUM from WELSPECS.
 
     const auto& ws = well_state.well(well_.indexOfWell());
     // Call check for the well's injection phase.
-    return WellGroupHelpers<Scalar>::checkGroupConstraintsInj(well_.name(),
+    return WellGroupHelpers<Scalar, IndexTraits>::checkGroupConstraintsInj(well_.name(),
                                                               well_.wellEcl().groupName(),
                                                               group,
                                                               well_state,
@@ -85,32 +86,33 @@ checkGroupConstraintsInj(const Group& group,
                                                               well_.guideRate(),
                                                               ws.surface_rates.data(),
                                                               injectionPhase,
-                                                              well_.phaseUsage(),
                                                               efficiencyFactor,
                                                               schedule,
                                                               summaryState,
                                                               resv_coeff,
+                                                              check_guide_rate,
                                                               deferred_logger);
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 std::pair<bool, Scalar>
-WellGroupConstraints<Scalar>::
+WellGroupConstraints<Scalar, IndexTraits>::
 checkGroupConstraintsProd(const Group& group,
-                          const WellState<Scalar>& well_state,
+                          const WellState<Scalar, IndexTraits>& well_state,
                           const GroupState<Scalar>& group_state,
                           const Scalar efficiencyFactor,
                           const Schedule& schedule,
                           const SummaryState& summaryState,
                           const RateConvFunc& rateConverter,
+                          const bool check_guide_rate,
                           DeferredLogger& deferred_logger) const
 {
     // Make conversion factors for RESV <-> surface rates.
-    std::vector<Scalar> resv_coeff(well_.phaseUsage().num_phases, 1.0);
+    std::vector<Scalar> resv_coeff(well_.phaseUsage().numActivePhases(), 1.0);
     rateConverter(0, well_.pvtRegionIdx(), group.name(), resv_coeff); // FIPNUM region 0 here, should use FIPNUM from WELSPECS.
 
     const auto& ws = well_state.well(well_.indexOfWell());
-    return WellGroupHelpers<Scalar>::checkGroupConstraintsProd(well_.name(),
+    return WellGroupHelpers<Scalar, IndexTraits>::checkGroupConstraintsProd(well_.name(),
                                                                well_.wellEcl().groupName(),
                                                                group,
                                                                well_state,
@@ -118,21 +120,22 @@ checkGroupConstraintsProd(const Group& group,
                                                                well_.currentStep(),
                                                                well_.guideRate(),
                                                                ws.surface_rates.data(),
-                                                               well_.phaseUsage(),
                                                                efficiencyFactor,
                                                                schedule,
                                                                summaryState,
                                                                resv_coeff,
+                                                               check_guide_rate,
                                                                deferred_logger);
 }
 
-template<class Scalar>
-bool WellGroupConstraints<Scalar>::
-checkGroupConstraints(WellState<Scalar>& well_state,
+template<typename Scalar, typename IndexTraits>
+bool WellGroupConstraints<Scalar, IndexTraits>::
+checkGroupConstraints(WellState<Scalar, IndexTraits>& well_state,
                       const GroupState<Scalar>& group_state,
                       const Schedule& schedule,
                       const SummaryState& summaryState,
                       const RateConvFunc& rateConverter,
+                      const bool check_guide_rate,
                       DeferredLogger& deferred_logger) const
 {
     const auto& well = well_.wellEcl();
@@ -157,6 +160,7 @@ checkGroupConstraints(WellState<Scalar>& well_state,
                                                group_state, efficiencyFactor,
                                                schedule, summaryState,
                                                rateConverter,
+                                               check_guide_rate,
                                                deferred_logger);
             // If a group constraint was broken, we set the current well control to
             // be GRUP.
@@ -188,7 +192,7 @@ checkGroupConstraints(WellState<Scalar>& well_state,
                 this->checkGroupConstraintsProd(group, well_state,
                                                 group_state, efficiencyFactor,
                                                 schedule, summaryState,
-                                                rateConverter, deferred_logger);
+                                                rateConverter, check_guide_rate, deferred_logger);
             // If a group constraint was broken, we set the current well control to
             // be GRUP.
             if (group_constraint.first) {
@@ -205,10 +209,10 @@ checkGroupConstraints(WellState<Scalar>& well_state,
     return false;
 }
 
-template class WellGroupConstraints<double>;
+template class WellGroupConstraints<double, BlackOilDefaultFluidSystemIndices>;
 
 #if FLOW_INSTANTIATE_FLOAT
-template class WellGroupConstraints<float>;
+template class WellGroupConstraints<float, BlackOilDefaultFluidSystemIndices>;
 #endif
 
 } // namespace Opm
