@@ -21,6 +21,8 @@
 #ifndef OPM_WELLGROUPHELPERS_HEADER_INCLUDED
 #define OPM_WELLGROUPHELPERS_HEADER_INCLUDED
 
+#include "opm/material/fluidsystems/PhaseUsageInfo.hpp"
+
 #include <opm/input/eclipse/Schedule/Group/GuideRate.hpp>
 #include <opm/input/eclipse/Schedule/Group/GSatProd.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
@@ -37,34 +39,44 @@ class DeferredLogger;
 class Group;
 template<class Scalar> class GroupState;
 namespace Network { class ExtNetwork; }
-struct PhaseUsage;
 class Schedule;
 template<class Scalar> class VFPProdProperties;
-template<class Scalar> class WellState;
+template<typename Scalar, typename IndexTraits> class WellState;
 class FieldPropsManager;
 
 namespace Network { class ExtNetwork; }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 class WellGroupHelpers
 {
 public:
 
+    using WellStateType = WellState<Scalar, IndexTraits>;
+
     static Scalar sumWellPhaseRates(bool res_rates,
                                     const Opm::Group& group,
                                     const Opm::Schedule& schedule,
-                                    const Opm::WellState<Scalar>& wellState,
+                                    const WellStateType& wellState,
+                                    const SummaryState& summaryState,
                                     const int reportStepIdx,
                                     const int phasePos,
                                     const bool injector,
                                     const bool network = false);
 
-    static Scalar satelliteProduction(const ScheduleState& sched,
-                                      const std::vector<std::string>& groups,
-                                      const GSatProd::GSatProdGroup::Rate rateComp);
+    static Scalar satelliteInjectionRate(const ScheduleState& sched,
+                                         const Group& group,
+                                         const PhaseUsageInfo<IndexTraits>& pu,
+                                         const int phase_pos,
+                                         bool res_rates);
 
-    static std::optional<GSatProd::GSatProdGroup::Rate>
-    selectRateComponent(const PhaseUsage& pu, const int phasePos);
+    static Scalar satelliteProductionRate(const SummaryState& summaryState,
+                                          const ScheduleState& sched,
+                                          const Group& group,
+                                          const GSatProd::GSatProdGroupProp::Rate rateComp,
+                                          bool res_rates);
+
+    static std::optional<GSatProd::GSatProdGroupProp::Rate>
+    selectRateComponent(const PhaseUsageInfo<IndexTraits>& pu, const int phasePos);
 
     static void setCmodeGroup(const Group& group,
                               const Schedule& schedule,
@@ -79,10 +91,11 @@ public:
 
     static Scalar sumWellSurfaceRates(const Group& group,
                                       const Schedule& schedule,
-                                      const WellState<Scalar>& wellState,
+                                      const WellStateType& wellState,
                                       const int reportStepIdx,
                                       const int phasePos,
-                                      const bool injector);
+                                      const bool injector,
+                                      const SummaryState& summaryState);
 
     /// Returns the name of the worst offending well and its fraction (i.e. violated_phase / preferred_phase)
     static std::pair<std::optional<std::string>, Scalar>
@@ -90,21 +103,21 @@ public:
                        const Schedule& schedule,
                        const int reportStepIdx,
                        const Group::ProductionCMode& offendedControl,
-                       const PhaseUsage& pu,
                        const Parallel::Communication& comm,
-                       const WellState<Scalar>& wellState,
+                       const WellStateType& wellState,
                        DeferredLogger& deferred_logger);
 
     static Scalar sumWellResRates(const Group& group,
                                   const Schedule& schedule,
-                                  const WellState<Scalar>& wellState,
+                                  const WellStateType& wellState,
                                   const int reportStepIdx,
                                   const int phasePos,
-                                  const bool injector);
+                                  const bool injector,
+                                  const SummaryState& summaryState);
 
     static Scalar sumSolventRates(const Group& group,
                                   const Schedule& schedule,
-                                  const WellState<Scalar>& wellState,
+                                  const WellStateType& wellState,
                                   const int reportStepIdx,
                                   const bool injector);
 
@@ -112,60 +125,18 @@ public:
                                            const Schedule& schedule,
                                            const int reportStepIdx,
                                            const bool isInjector,
-                                           const PhaseUsage& pu,
                                            const GuideRate& guide_rate,
-                                           const WellState<Scalar>& wellState,
+                                           const WellStateType& wellState,
                                            const SummaryState& summaryState,
                                            GroupState<Scalar>& group_state,
                                            std::vector<Scalar>& groupTargetReduction);
 
-    static void updateGuideRates(const Group& group,
-                                 const Schedule& schedule,
-                                 const SummaryState& summary_state,
-                                 const PhaseUsage& pu,
-                                 int report_step,
-                                 double sim_time,
-                                 WellState<Scalar>& well_state,
-                                 const GroupState<Scalar>& group_state,
-                                 const Parallel::Communication& comm,
-                                 GuideRate* guide_rate,
-                                 std::vector<Scalar>& pot,
-                                 DeferredLogger& deferred_logger);
-
-    static void updateGuideRateForProductionGroups(const Group& group,
-                                                   const Schedule& schedule,
-                                                   const PhaseUsage& pu,
-                                                   const int reportStepIdx,
-                                                   const double& simTime,
-                                                   WellState<Scalar>& wellState,
-                                                   const GroupState<Scalar>& group_state,
-                                                   const Parallel::Communication& comm,
-                                                   GuideRate* guideRate,
-                                                   std::vector<Scalar>& pot);
-
-    static void updateGuideRatesForWells(const Schedule& schedule,
-                                         const PhaseUsage& pu,
-                                         const int reportStepIdx,
-                                         const double& simTime,
-                                         const WellState<Scalar>& wellState,
-                                         const Parallel::Communication& comm,
-                                         GuideRate* guideRate);
-
-    static void updateGuideRatesForInjectionGroups(const Group& group,
-                                                   const Schedule& schedule,
-                                                   const SummaryState& summaryState,
-                                                   const PhaseUsage& pu,
-                                                   const int reportStepIdx,
-                                                   const WellState<Scalar>& wellState,
-                                                   const GroupState<Scalar>& group_state,
-                                                   GuideRate* guideRate,
-                                                   DeferredLogger& deferred_logger);
-
     static void updateVREPForGroups(const Group& group,
                                     const Schedule& schedule,
                                     const int reportStepIdx,
-                                    const WellState<Scalar>& wellState,
-                                    GroupState<Scalar>& group_state);
+                                    const WellStateType& wellState,
+                                    GroupState<Scalar>& group_state,
+                                    const SummaryState& summaryState);
 
     template <class RegionalValues>
     static void updateGpMaintTargetForGroups(const Group& group,
@@ -173,37 +144,41 @@ public:
                                              const RegionalValues& regional_values,
                                              const int reportStepIdx,
                                              const double dt,
-                                             const WellState<Scalar>& well_state,
+                                             const WellStateType& well_state,
                                              GroupState<Scalar>& group_state);
 
     static void updateReservoirRatesInjectionGroups(const Group& group,
                                                     const Schedule& schedule,
                                                     const int reportStepIdx,
-                                                    const WellState<Scalar>& wellState,
-                                                    GroupState<Scalar>& group_state);
+                                                    const WellStateType& wellState,
+                                                    GroupState<Scalar>& group_state,
+                                                    const SummaryState& summaryState);
 
     static void updateSurfaceRatesInjectionGroups(const Group& group,
                                                   const Schedule& schedule,
                                                   const int reportStepIdx,
-                                                  const WellState<Scalar>& wellState,
-                                                  GroupState<Scalar>& group_state);
+                                                  const WellStateType& wellState,
+                                                  GroupState<Scalar>& group_state,
+                                                  const SummaryState& summaryState);
 
     static void updateWellRates(const Group& group,
                                 const Schedule& schedule,
                                 const int reportStepIdx,
-                                const WellState<Scalar>& wellStateNupcol,
-                                WellState<Scalar>& wellState);
+                                const WellStateType& wellStateNupcol,
+                                WellStateType& wellState);
 
     static void updateGroupProductionRates(const Group& group,
                                            const Schedule& schedule,
                                            const int reportStepIdx,
-                                           const WellState<Scalar>& wellState,
-                                           GroupState<Scalar>& group_state);
+                                           const WellStateType& wellState,
+                                           GroupState<Scalar>& group_state,
+                                           const SummaryState& summaryState);
 
     static void updateNetworkLeafNodeProductionRates(const Schedule& schedule,
                                                      const int reportStepIdx,
-                                                     const WellState<Scalar>& wellState,
-                                                     GroupState<Scalar>& group_state);
+                                                     const WellStateType& wellState,
+                                                     GroupState<Scalar>& group_state,
+                                                     const SummaryState& summaryState);
 
 
     static void updateWellRatesFromGroupTargetScale(const Scalar scale,
@@ -212,60 +187,68 @@ public:
                                                     const int reportStepIdx,
                                                     bool isInjector,
                                                     const GroupState<Scalar>& group_state,
-                                                    WellState<Scalar>& wellState);
+                                                    WellStateType& wellState);
 
     static void updateREINForGroups(const Group& group,
                                     const Schedule& schedule,
                                     const int reportStepIdx,
-                                    const PhaseUsage& pu,
                                     const SummaryState& st,
-                                    const WellState<Scalar>& wellState,
+                                    const WellStateType& wellState,
                                     GroupState<Scalar>& group_state,
                                     bool sum_rank);
 
 
     static std::map<std::string, Scalar>
     computeNetworkPressures(const Network::ExtNetwork& network,
-                            const WellState<Scalar>& well_state,
+                            const WellStateType& well_state,
                             const GroupState<Scalar>& group_state,
                             const VFPProdProperties<Scalar>& vfp_prod_props,
                             const Schedule& schedule,
+                            const Parallel::Communication& comm,
                             const int report_time_step);
 
     static GuideRate::RateVector
-    getWellRateVector(const WellState<Scalar>& well_state,
-                      const PhaseUsage& pu,
+    getWellRateVector(const WellStateType& well_state,
                       const std::string& name);
 
     static GuideRate::RateVector
     getProductionGroupRateVector(const GroupState<Scalar>& group_state,
-                                 const PhaseUsage& pu,
+                                 const PhaseUsageInfo<IndexTraits>& pu,
                                  const std::string& group_name);
 
     static Scalar getGuideRate(const std::string& name,
                                const Schedule& schedule,
-                               const WellState<Scalar>& wellState,
+                               const WellStateType& wellState,
                                const GroupState<Scalar>& group_state,
                                const int reportStepIdx,
                                const GuideRate* guideRate,
-                               const GuideRateModel::Target target,
-                               const PhaseUsage& pu);
+                               const GuideRateModel::Target target);
 
     static Scalar getGuideRateInj(const std::string& name,
                                   const Schedule& schedule,
-                                  const WellState<Scalar>& wellState,
+                                  const WellStateType& wellState,
                                   const GroupState<Scalar>& group_state,
                                   const int reportStepIdx,
                                   const GuideRate* guideRate,
                                   const GuideRateModel::Target target,
-                                  const Phase& injectionPhase,
-                                  const PhaseUsage& pu);
+                                  const Phase& injectionPhase);
 
+    /// update the number of wells that are actively under group control for a given group with name given by group_name
+    /// its main usage is to detect cases where there is no wells under group control
+    static int updateGroupControlledWells(const Schedule& schedule,
+                                          const WellStateType& well_state,
+                                          GroupState<Scalar>& group_state,
+                                          const SummaryState& summary_state,
+                                          const GuideRate* guideRate,
+                                          const int report_step,
+                                          const std::string& group_name,
+                                          const bool is_production_group,
+                                          const Phase injection_phase);
+
+    /// returns the number of wells that are actively under group control for a given group with name given by group_name
     static int groupControlledWells(const Schedule& schedule,
-                                    const WellState<Scalar>& well_state,
+                                    const WellStateType& well_state,
                                     const GroupState<Scalar>& group_state,
-                                    const SummaryState& summary_state,
-                                    const GuideRate* guideRate,
                                     const int report_step,
                                     const std::string& group_name,
                                     const std::string& always_included_child,
@@ -276,25 +259,49 @@ public:
     checkGroupConstraintsInj(const std::string& name,
                              const std::string& parent,
                              const Group& group,
-                             const WellState<Scalar>& wellState,
+                             const WellStateType& wellState,
                              const GroupState<Scalar>& group_state,
                              const int reportStepIdx,
                              const GuideRate* guideRate,
                              const Scalar* rates,
                              Phase injectionPhase,
-                             const PhaseUsage& pu,
                              const Scalar efficiencyFactor,
                              const Schedule& schedule,
                              const SummaryState& summaryState,
                              const std::vector<Scalar>& resv_coeff,
+                             const bool check_guide_rate,
                              DeferredLogger& deferred_logger);
 
+    static Scalar
+    getWellGroupTargetInjector(const std::string& name,
+                               const std::string& parent,
+                               const Group& group,
+                               const WellStateType& wellState,
+                               const GroupState<Scalar>& group_state,
+                               const int reportStepIdx,
+                               const GuideRate* guideRate,
+                               const Scalar* rates,
+                               Phase injectionPhase,
+                               const Scalar efficiencyFactor,
+                               const Schedule& schedule,
+                               const SummaryState& summaryState,
+                               const std::vector<Scalar>& resv_coeff,
+                               DeferredLogger& deferred_logger);
+
     static std::vector<std::string>
     groupChainTopBot(const std::string& bottom,
                      const std::string& top,
                      const Schedule& schedule,
                      const int report_step);
 
+
+    // check if well/group bottom is a sub well/group of the group top
+    static bool
+    isInGroupChainTopBot(const std::string& bottom,
+                         const std::string& top,
+                         const Schedule& schedule,
+                         const int report_step);
+
     static std::string
     control_group(const Group& group,
                   const GroupState<Scalar>& group_state,
@@ -305,24 +312,37 @@ public:
     checkGroupConstraintsProd(const std::string& name,
                               const std::string& parent,
                               const Group& group,
-                              const WellState<Scalar>& wellState,
+                              const WellStateType& wellState,
                               const GroupState<Scalar>& group_state,
                               const int reportStepIdx,
                               const GuideRate* guideRate,
                               const Scalar* rates,
-                              const PhaseUsage& pu,
                               const Scalar efficiencyFactor,
                               const Schedule& schedule,
                               const SummaryState& summaryState,
                               const std::vector<Scalar>& resv_coeff,
+                              const bool check_guide_rate,
                               DeferredLogger& deferred_logger);
+    static Scalar
+    getWellGroupTargetProducer(const std::string& name,
+                               const std::string& parent,
+                               const Group& group,
+                               const WellStateType& wellState,
+                               const GroupState<Scalar>& group_state,
+                               const int reportStepIdx,
+                               const GuideRate* guideRate,
+                               const Scalar* rates,
+                               const Scalar efficiencyFactor,
+                               const Schedule& schedule,
+                               const SummaryState& summaryState,
+                               const std::vector<Scalar>& resv_coeff,
+                               DeferredLogger& deferred_logger);
 
     template <class AverageRegionalPressureType>
     static void setRegionAveragePressureCalculator(const Group& group,
                                                    const Schedule& schedule,
                                                    const int reportStepIdx,
                                                    const FieldPropsManager& fp,
-                                                   const PhaseUsage& pu,
                                                    std::map<std::string, std::unique_ptr<AverageRegionalPressureType>>& regionalAveragePressureCalculator);
 };
 
