@@ -20,6 +20,7 @@
 */
 
 #include <config.h>
+
 #include <opm/simulators/wells/WellInterfaceGeneric.hpp>
 
 #include <opm/common/ErrorMacros.hpp>
@@ -33,7 +34,11 @@
 #include <opm/input/eclipse/Schedule/Well/WellPolymerProperties.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 #include <opm/input/eclipse/Schedule/Well/WVFPEXP.hpp>
+
+#include <opm/material/fluidsystems/BlackOilDefaultFluidSystemIndices.hpp>
+
 #include <opm/simulators/utils/DeferredLoggingErrorHelpers.hpp>
+
 #include <opm/simulators/wells/PerforationData.hpp>
 #include <opm/simulators/wells/ParallelWellInfo.hpp>
 #include <opm/simulators/wells/VFPHelpers.hpp>
@@ -45,35 +50,40 @@
 
 #include <fmt/format.h>
 
+#include <algorithm>
 #include <cassert>
 #include <cmath>
 #include <cstddef>
 #include <stdexcept>
+#include <string>
+#include <vector>
 
 namespace Opm {
 
-template<class Scalar>
-WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+WellInterfaceGeneric<Scalar, IndexTraits>::
 WellInterfaceGeneric(const Well& well,
                      const ParallelWellInfo<Scalar>& pw_info,
                      const int time_step,
                      const ModelParameters& param,
                      const int pvtRegionIdx,
-                     const int num_components,
+                     const int num_conservation_quantities,
                      const int num_phases,
                      const int index_of_well,
+                     const PhaseUsageInfo<IndexTraits>& phase_usage,
                      const std::vector<PerforationData<Scalar>>& perf_data)
       : well_ecl_(well)
       , parallel_well_info_(pw_info)
       , current_step_(time_step)
       , param_(param)
       , pvtRegionIdx_(pvtRegionIdx)
-      , num_components_(num_components)
+      , num_conservation_quantities_(num_conservation_quantities)
       , number_of_phases_(num_phases)
       , index_of_well_(index_of_well)
+      , phase_usage_(phase_usage)
       , perf_data_(&perf_data)
-      , ipr_a_(num_components)
-      , ipr_b_(num_components)
+      , ipr_a_(num_conservation_quantities)
+      , ipr_b_(num_conservation_quantities)
 {
     assert(well.name()==pw_info.name());
     assert(std::is_sorted(perf_data.begin(), perf_data.end(),
@@ -88,13 +98,14 @@ WellInterfaceGeneric(const Well& well,
 
     // We do not want to count SHUT perforations here, so
     // it would be wrong to use wells.getConnections().size().
-    number_of_perforations_ = perf_data.size();
+    // This is the number_of_local_perforations_ on this process only!
+    number_of_local_perforations_ = perf_data.size();
 
     // perforations related
     {
-        well_cells_.resize(number_of_perforations_);
-        well_index_.resize(number_of_perforations_);
-        saturation_table_number_.resize(number_of_perforations_);
+        well_cells_.resize(number_of_local_perforations_);
+        well_index_.resize(number_of_local_perforations_);
+        saturation_table_number_.resize(number_of_local_perforations_);
         int perf = 0;
         for (const auto& pd : perf_data) {
             well_cells_[perf] = pd.cell_index;
@@ -125,15 +136,15 @@ WellInterfaceGeneric(const Well& well,
 //  value in VFPPROD record 5 (GFR values) and supplying a dummy input value
 //  for the gas rate to the methods in VFPProdProperties.cpp, we can extend
 //  the VFP calculations to the two-phase oil-water case.
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 adaptRatesForVFP(std::vector<Scalar>& rates) const
 {
     const auto& pu = this->phaseUsage();
-    if (pu.num_phases == 2) {
-        if (    pu.phase_used[BlackoilPhases::Aqua] == 1
-             && pu.phase_used[BlackoilPhases::Liquid] == 1
-             && pu.phase_used[BlackoilPhases::Vapour] == 0)
+    if (pu.numActivePhases() == 2) {
+        if ( pu.phaseIsActive(IndexTraits::waterPhaseIdx)
+             && pu.phaseIsActive(IndexTraits::oilPhaseIdx)
+             && !pu.phaseIsActive(IndexTraits::gasPhaseIdx))
         {
             assert(rates.size() == 2);
             rates.push_back(0.0);  // set gas rate to zero
@@ -145,78 +156,79 @@ adaptRatesForVFP(std::vector<Scalar>& rates) const
     }
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 const std::vector<PerforationData<Scalar>>&
-WellInterfaceGeneric<Scalar>::perforationData() const
+WellInterfaceGeneric<Scalar, IndexTraits>::perforationData() const
 {
     return *perf_data_;
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 const std::string&
-WellInterfaceGeneric<Scalar>::name() const
+WellInterfaceGeneric<Scalar, IndexTraits>::name() const
 {
     return well_ecl_.name();
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::isInjector() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::isInjector() const
 {
     return well_ecl_.isInjector();
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::isProducer() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::isProducer() const
 {
     return well_ecl_.isProducer();
 }
 
-template<class Scalar>
-int WellInterfaceGeneric<Scalar>::indexOfWell() const
+template<typename Scalar, typename IndexTraits>
+int WellInterfaceGeneric<Scalar, IndexTraits>::indexOfWell() const
 {
     return index_of_well_;
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::getAllowCrossFlow() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::getAllowCrossFlow() const
 {
     return well_ecl_.getAllowCrossFlow();
 }
 
-template<class Scalar>
-const Well& WellInterfaceGeneric<Scalar>::wellEcl() const
+template<typename Scalar, typename IndexTraits>
+const Well& WellInterfaceGeneric<Scalar, IndexTraits>::wellEcl() const
 {
     return well_ecl_;
 }
 
-template<class Scalar>
-Well& WellInterfaceGeneric<Scalar>::wellEcl()
+template<typename Scalar, typename IndexTraits>
+Well& WellInterfaceGeneric<Scalar, IndexTraits>::wellEcl()
 {
     return well_ecl_;
 }
 
-template<class Scalar>
-const PhaseUsage& WellInterfaceGeneric<Scalar>::phaseUsage() const
+template<typename Scalar, typename IndexTraits>
+const PhaseUsageInfo<IndexTraits>&
+WellInterfaceGeneric<Scalar, IndexTraits>::phaseUsage() const
 {
-    assert(phase_usage_ != nullptr);
-
-    return *phase_usage_;
+    return phase_usage_;
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::wsolvent() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::wsolvent() const
 {
     return wsolvent_;
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::rsRvInj() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::rsRvInj() const
 {
     return well_ecl_.getInjectionProperties().rsRvInj;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 initInjMult(const std::vector<Scalar>& max_inj_mult)
 {
     // prev_inj_multiplier_ will stay unchanged during the time step
@@ -229,8 +241,8 @@ initInjMult(const std::vector<Scalar>& max_inj_mult)
     this->inj_multiplier_damp_factor_ = std::vector<Scalar>(nperf, 1.);
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 updateInjMult(std::vector<Scalar>& inj_multipliers,
               DeferredLogger& deferred_logger) const
 {
@@ -244,9 +256,10 @@ updateInjMult(std::vector<Scalar>& inj_multipliers,
     inj_multipliers = this->inj_multiplier_;
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::
-getInjMult(const int perf,
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::
+getInjMult(const int local_perf_index,
            const Scalar bhp,
            const Scalar perf_pres,
            DeferredLogger& dlogger) const
@@ -255,7 +268,7 @@ getInjMult(const int perf,
 
     Scalar multiplier = 1.;
 
-    const auto perf_ecl_index = this->perforationData()[perf].ecl_index;
+    const auto perf_ecl_index = this->perforationData()[local_perf_index].ecl_index;
     const bool is_wrev = this->well_ecl_.getInjMultMode() == Well::InjMultMode::WREV;
 
     const bool active_injmult = (is_wrev && this->well_ecl_.aciveWellInjMult()) ||
@@ -307,8 +320,8 @@ getInjMult(const int perf,
     return multiplier;
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::
 wellHasTHPConstraints(const SummaryState& summaryState) const
 {
     // only wells under prediction mode can have THP constraint
@@ -323,9 +336,9 @@ wellHasTHPConstraints(const SummaryState& summaryState) const
     return WellBhpThpCalculator(*this).wellHasTHPConstraints(summaryState);
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
-updateWellTestState(const SingleWellState<Scalar>& ws,
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
+updateWellTestState(const SingleWellState<Scalar, IndexTraits>& ws,
                     const double& simulationTime,
                     const bool& writeMessageToOPMLog,
                     const bool zero_group_target,
@@ -344,8 +357,8 @@ updateWellTestState(const SingleWellState<Scalar>& ws,
     // TODO: well can be shut/closed due to other reasons
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+Scalar WellInterfaceGeneric<Scalar, IndexTraits>::
 getTHPConstraint(const SummaryState& summaryState) const
 {
     if (dynamic_thp_limit_) {
@@ -355,14 +368,14 @@ getTHPConstraint(const SummaryState& summaryState) const
     return WellBhpThpCalculator(*this).getTHPConstraint(summaryState);
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::underPredictionMode() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::underPredictionMode() const
 {
     return well_ecl_.predictionMode();
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::initCompletions()
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::initCompletions()
 {
     assert(completions_.empty() );
 
@@ -389,8 +402,8 @@ void WellInterfaceGeneric<Scalar>::initCompletions()
     assert(my_next_perf == perf_data_->end());
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 closeCompletions(const WellTestState& wellTestState)
 {
     const auto& connections = well_ecl_.getConnections();
@@ -405,17 +418,17 @@ closeCompletions(const WellTestState& wellTestState)
     }
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
-setVFPProperties(const VFPProperties<Scalar>* vfp_properties_arg)
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
+setVFPProperties(const VFPProperties<Scalar, IndexTraits>* vfp_properties_arg)
 {
     vfp_properties_ = vfp_properties_arg;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
-setPrevSurfaceRates(WellState<Scalar>& well_state,
-                    const WellState<Scalar>& prev_well_state) const
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
+setPrevSurfaceRates(WellState<Scalar, IndexTraits>& well_state,
+                    const WellState<Scalar, IndexTraits>& prev_well_state) const
 {
     auto& ws = well_state.well(this->index_of_well_);
     auto& ws_prev = prev_well_state.well(this->index_of_well_);
@@ -438,24 +451,24 @@ setPrevSurfaceRates(WellState<Scalar>& well_state,
     }
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 setGuideRate(const GuideRate* guide_rate_arg)
 {
     guide_rate_ = guide_rate_arg;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 setWellEfficiencyFactor(const Scalar efficiency_factor)
 {
     well_efficiency_factor_ = efficiency_factor;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::setRepRadiusPerfLength()
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::setRepRadiusPerfLength()
 {
-    const int nperf = number_of_perforations_;
+    const int nperf = number_of_local_perforations_;
 
     perf_rep_radius_.clear();
     perf_length_.clear();
@@ -467,7 +480,7 @@ void WellInterfaceGeneric<Scalar>::setRepRadiusPerfLength()
 
     const WellConnections& connections = well_ecl_.getConnections();
     const std::size_t num_conns = connections.size();
-    int num_active_connections = 0;
+    [[maybe_unused]] int num_active_connections = 0;
     auto my_next_perf = perf_data_->begin();
     for (std::size_t c = 0; c < num_conns; ++c) {
         if (my_next_perf == perf_data_->end())
@@ -496,38 +509,45 @@ void WellInterfaceGeneric<Scalar>::setRepRadiusPerfLength()
     assert(num_active_connections == nperf);
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 setWsolvent(const Scalar wsolvent)
 {
     wsolvent_ = wsolvent;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 setDynamicThpLimit(const Scalar thp_limit)
 {
     dynamic_thp_limit_ = thp_limit;
 }
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 std::optional<Scalar>
-WellInterfaceGeneric<Scalar>::getDynamicThpLimit() const
+WellInterfaceGeneric<Scalar, IndexTraits>::getDynamicThpLimit() const
 {
     return dynamic_thp_limit_;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
+setDynamicThpLimit(const std::optional<Scalar> thp_limit)
+{
+    dynamic_thp_limit_ = thp_limit;
+}
+
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 updatePerforatedCell(std::vector<bool>& is_cell_perforated)
 {
-    for (int perf_idx = 0; perf_idx < number_of_perforations_; ++perf_idx) {
+    for (int perf_idx = 0; perf_idx < number_of_local_perforations_; ++perf_idx) {
         is_cell_perforated[well_cells_[perf_idx]] = true;
     }
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::
 isVFPActive(DeferredLogger& deferred_logger) const
 {
     // since the well_controls only handles the VFP number when THP constraint/target is there.
@@ -568,39 +588,40 @@ isVFPActive(DeferredLogger& deferred_logger) const
     }
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::isOperableAndSolvable() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::isOperableAndSolvable() const
 {
     return operability_status_.isOperableAndSolvable();
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::useVfpExplicit() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::useVfpExplicit() const
 {
     const auto& wvfpexp = well_ecl_.getWVFPEXP();
     return (wvfpexp.explicit_lookup() || operability_status_.use_vfpexplicit);
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::thpLimitViolatedButNotSwitched() const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::thpLimitViolatedButNotSwitched() const
 {
     return operability_status_.thp_limit_violated_but_not_switched;
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::
-getALQ(const WellState<Scalar>& well_state) const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::
+getALQ(const WellState<Scalar, IndexTraits>& well_state) const
 {
     // no alq for injectors.
     if (isInjector())
         return 0.0;
 
-    return well_state.getALQ(name());
+    return well_state.well(name()).alq_state.get();
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
-reportWellSwitching(const SingleWellState<Scalar> &ws,
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
+reportWellSwitching(const SingleWellState<Scalar, IndexTraits> &ws,
                     DeferredLogger& deferred_logger) const
 {
     if (well_control_log_.empty())
@@ -620,9 +641,9 @@ reportWellSwitching(const SingleWellState<Scalar> &ws,
     }
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::
-isPressureControlled(const WellState<Scalar>& well_state) const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::
+isPressureControlled(const WellState<Scalar, IndexTraits>& well_state) const
 {
     const auto& ws = well_state.well(this->index_of_well_);
     if (this->isInjector()) {
@@ -636,10 +657,10 @@ isPressureControlled(const WellState<Scalar>& well_state) const
     }
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::
 wellUnderZeroRateTargetIndividual(const SummaryState& summary_state,
-                                  const WellState<Scalar>& well_state) const
+                                  const WellState<Scalar, IndexTraits>& well_state) const
 {
     if (this->isProducer()) { // producers
         const auto prod_controls = this->well_ecl_.productionControls(summary_state);
@@ -652,9 +673,9 @@ wellUnderZeroRateTargetIndividual(const SummaryState& summary_state,
     }
 }
 
-template<class Scalar>
-bool WellInterfaceGeneric<Scalar>::
-wellUnderGroupControl(const SingleWellState<Scalar>& ws) const
+template<typename Scalar, typename IndexTraits>
+bool WellInterfaceGeneric<Scalar, IndexTraits>::
+wellUnderGroupControl(const SingleWellState<Scalar, IndexTraits>& ws) const
 {
     // Check if well is under group control
     const bool isGroupControlled = (this->isInjector() && ws.injection_cmode == Well::InjectorCMode::GRUP) ||
@@ -662,14 +683,45 @@ wellUnderGroupControl(const SingleWellState<Scalar>& ws) const
     return isGroupControlled;
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::resetWellOperability()
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::resetWellOperability()
 {
     this->operability_status_.resetOperability();
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::wmicrobes_() const
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::addPerforations(const std::vector<RuntimePerforation>& perfs)
+{
+    for (const auto& perf : perfs) {
+        auto it = std::find(well_cells_.begin(), well_cells_.end(), perf.cell);
+        if (it != this->well_cells_.end()) {
+            // If perforation to cell already exists, just add contribution.
+            const auto ind = std::distance(this->well_cells_.begin(), it);
+            this->well_index_[ind] += static_cast<Scalar>(perf.ctf);
+        }
+        else {
+            this->well_cells_.push_back(perf.cell);
+            this->well_index_.push_back(static_cast<Scalar>(perf.ctf));
+            this->perf_depth_.push_back(static_cast<Scalar>(perf.depth));
+
+            // Not strictly needed.
+            const double nan = std::nan("1");
+            this->perf_rep_radius_.push_back(nan);
+            this->perf_length_.push_back(nan);
+            this->bore_diameters_.push_back(nan);
+
+            // For now use the saturation table for the first cell.
+            this->saturation_table_number_
+                .push_back(this->saturation_table_number_.front());
+
+            ++this->number_of_local_perforations_;
+        }
+    }
+}
+
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::wmicrobes_() const
 {
     auto injectorType = this->well_ecl_.injectorType();
 
@@ -683,8 +735,9 @@ Scalar WellInterfaceGeneric<Scalar>::wmicrobes_() const
     }
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::wfoam_() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::wfoam_() const
 {
     auto injectorType = this->well_ecl_.injectorType();
 
@@ -697,8 +750,9 @@ Scalar WellInterfaceGeneric<Scalar>::wfoam_() const
     }
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::wsalt_() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::wsalt_() const
 {
     auto injectorType = this->well_ecl_.injectorType();
 
@@ -711,8 +765,9 @@ Scalar WellInterfaceGeneric<Scalar>::wsalt_() const
     }
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::woxygen_() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::woxygen_() const
 {
     auto injectorType = this->well_ecl_.injectorType();
 
@@ -726,8 +781,9 @@ Scalar WellInterfaceGeneric<Scalar>::woxygen_() const
     }
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::wpolymer_() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::wpolymer_() const
 {
     auto injectorType = this->well_ecl_.injectorType();
 
@@ -741,14 +797,15 @@ Scalar WellInterfaceGeneric<Scalar>::wpolymer_() const
     }
 }
 
-template<class Scalar>
-Scalar WellInterfaceGeneric<Scalar>::wurea_() const
+template<typename Scalar, typename IndexTraits>
+Scalar
+WellInterfaceGeneric<Scalar, IndexTraits>::wurea_() const
 {
     auto injectorType = this->well_ecl_.injectorType();
 
     if (injectorType == InjectorType::WATER) {
         WellMICPProperties urea = this->well_ecl_.getMICPProperties();
-        const Scalar urea_injection_concentration = urea.m_ureaConcentration / 10.; //Dividing by scaling factor 10
+        const Scalar urea_injection_concentration = urea.m_ureaConcentration;
         return urea_injection_concentration;
     } else {
         // Not a water injection well => no urea.
@@ -756,28 +813,28 @@ Scalar WellInterfaceGeneric<Scalar>::wurea_() const
     }
 }
 
-template<class Scalar>
-int WellInterfaceGeneric<Scalar>::polymerTable_() const
+template<typename Scalar, typename IndexTraits>
+int WellInterfaceGeneric<Scalar, IndexTraits>::polymerTable_() const
 {
     return this->well_ecl_.getPolymerProperties().m_skprpolytable;
 }
 
-template<class Scalar>
-int WellInterfaceGeneric<Scalar>::polymerWaterTable_() const
+template<typename Scalar, typename IndexTraits>
+int WellInterfaceGeneric<Scalar, IndexTraits>::polymerWaterTable_() const
 {
     return this->well_ecl_.getPolymerProperties().m_skprwattable;
 }
 
-template<class Scalar>
-int WellInterfaceGeneric<Scalar>::polymerInjTable_() const
+template<typename Scalar, typename IndexTraits>
+int WellInterfaceGeneric<Scalar, IndexTraits>::polymerInjTable_() const
 {
     return this->well_ecl_.getPolymerProperties().m_plymwinjtable;
 }
 
-template<class Scalar>
-std::pair<bool,bool> WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+std::pair<bool,bool> WellInterfaceGeneric<Scalar, IndexTraits>::
 computeWellPotentials(std::vector<Scalar>& well_potentials,
-                      const WellState<Scalar>& well_state)
+                      const WellState<Scalar, IndexTraits>& well_state)
 {
     const int np = this->number_of_phases_;
     well_potentials.resize(np, 0.0);
@@ -832,8 +889,8 @@ computeWellPotentials(std::vector<Scalar>& well_potentials,
     return {compute_potential, bhp_controlled_well};
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
 checkNegativeWellPotentials(std::vector<Scalar>& well_potentials,
                             const bool checkOperability,
                             DeferredLogger& deferred_logger)
@@ -853,12 +910,12 @@ checkNegativeWellPotentials(std::vector<Scalar>& well_potentials,
     }
 }
 
-template<class Scalar>
-void WellInterfaceGeneric<Scalar>::
-prepareForPotentialCalculations(const SummaryState& summary_state,
-                                WellState<Scalar>& well_state,
-                                Well::InjectionControls& inj_controls,
-                                Well::ProductionControls& prod_controls) const
+template<typename Scalar, typename IndexTraits>
+void WellInterfaceGeneric<Scalar, IndexTraits>::
+onlyKeepBHPandTHPcontrols(const SummaryState& summary_state,
+                          WellState<Scalar, IndexTraits>& well_state,
+                          Well::InjectionControls& inj_controls,
+                          Well::ProductionControls& prod_controls) const
 {
     const bool has_thp = this->wellHasTHPConstraints(summary_state);
     auto& ws = well_state.well(this->index_of_well_);
@@ -878,7 +935,7 @@ prepareForPotentialCalculations(const SummaryState& summary_state,
                 ws.injection_cmode = Well::InjectorCMode::BHP;
                 ws.bhp = inj_controls.bhp_limit;
             }
-        } 
+        }
     } else { // producer
         prod_controls.clearControls();
         prod_controls.addControl(Well::ProducerCMode::BHP);
@@ -893,14 +950,15 @@ prepareForPotentialCalculations(const SummaryState& summary_state,
                 ws.production_cmode = Well::ProducerCMode::BHP;
                 ws.bhp = prod_controls.bhp_limit;
             }
-        } 
-    }    
+        }
+    }
 }
 
-template class WellInterfaceGeneric<double>;
+
+template class WellInterfaceGeneric<double, BlackOilDefaultFluidSystemIndices>;
 
 #if FLOW_INSTANTIATE_FLOAT
-template class WellInterfaceGeneric<float>;
+template class WellInterfaceGeneric<float, BlackOilDefaultFluidSystemIndices>;
 #endif
 
 } // namespace Opm
