@@ -40,30 +40,35 @@ class DeferredLogger;
 class GuideRate;
 template<class Scalar> class ParallelWellInfo;
 template<class Scalar> struct PerforationData;
-struct PhaseUsage;
 class SummaryState;
-template<class Scalar> class VFPProperties;
+template<typename Scalar, typename IndexTraits> class VFPProperties;
 class WellTestState;
-template<class Scalar> class WellState;
-template<class Scalar> class SingleWellState;
+template<typename Scalar, typename IndexTraits> class WellState;
+template<typename Scalar, typename IndexTraits> class SingleWellState;
 class Group;
 class Schedule;
+template<typename IndexTraits> class PhaseUsageInfo;
 
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 class WellInterfaceGeneric {
 public:
     using ModelParameters = BlackoilModelParameters<Scalar>;
 
+    using WellStateType = WellState<Scalar, IndexTraits>;
+
     WellInterfaceGeneric(const Well& well,
                          const ParallelWellInfo<Scalar>& parallel_well_info,
                          const int time_step,
                          const ModelParameters& param,
                          const int pvtRegionIdx,
-                         const int num_components,
+                         const int num_conservation_quantities,
                          const int num_phases,
                          const int index_of_well,
+                         const PhaseUsageInfo<IndexTraits>& phase_usage,
                          const std::vector<PerforationData<Scalar>>& perf_data);
 
+    virtual ~WellInterfaceGeneric() = default;
+
     /// \brief Get the perforations of the well
     const std::vector<PerforationData<Scalar>>& perforationData() const;
 
@@ -86,7 +91,7 @@ public:
 
     const Well& wellEcl() const;
     Well& wellEcl();
-    const PhaseUsage& phaseUsage() const;
+    const PhaseUsageInfo<IndexTraits>& phaseUsage() const;
 
     /// Returns true if the well is currently in prediction mode (i.e. not history mode).
     bool underPredictionMode() const;
@@ -99,9 +104,9 @@ public:
     void initCompletions();
     void closeCompletions(const WellTestState& wellTestState);
 
-    void setVFPProperties(const VFPProperties<Scalar>* vfp_properties_arg);
-    void setPrevSurfaceRates(WellState<Scalar>& well_state,
-                             const WellState<Scalar>& prev_well_state) const;
+    void setVFPProperties(const VFPProperties<Scalar, IndexTraits>* vfp_properties_arg);
+    void setPrevSurfaceRates(WellStateType& well_state,
+                             const WellStateType& prev_well_state) const;
     void setGuideRate(const GuideRate* guide_rate_arg);
     void setWellEfficiencyFactor(const Scalar efficiency_factor);
     void setRepRadiusPerfLength();
@@ -116,6 +121,7 @@ public:
 
     void stopWell() { this->wellStatus_ = Well::Status::STOP; }
     void openWell() { this->wellStatus_ = Well::Status::OPEN; }
+    Well::Status wellStatus() { return this->wellStatus_;}
 
     bool wellIsStopped() const { return this->wellStatus_ == Well::Status::STOP; }
 
@@ -125,17 +131,17 @@ public:
 
     const GuideRate* guideRate() const { return guide_rate_; }
 
-    int numComponents() const { return num_components_; }
+    int numConservationQuantities() const { return num_conservation_quantities_; }
 
     int numPhases() const { return number_of_phases_; }
 
-    int numPerfs() const { return number_of_local_perforations_; }
+    int numLocalPerfs() const { return number_of_local_perforations_; }
 
     Scalar refDepth() const { return ref_depth_; }
 
     Scalar gravity() const { return gravity_; }
 
-    const VFPProperties<Scalar>* vfpProperties() const { return vfp_properties_; }
+    const VFPProperties<Scalar, IndexTraits>* vfpProperties() const { return vfp_properties_; }
 
     const ParallelWellInfo<Scalar>& parallelWellInfo() const { return parallel_well_info_; }
 
@@ -148,7 +154,7 @@ public:
     const std::map<int,std::vector<int>>& getCompletions() const { return completions_; }
 
     Scalar getTHPConstraint(const SummaryState& summaryState) const;
-    Scalar getALQ(const WellState<Scalar>& well_state) const;
+    Scalar getALQ(const WellStateType& well_state) const;
     Scalar wsolvent() const;
     Scalar rsRvInj() const;
 
@@ -166,19 +172,19 @@ public:
     // whether a well is specified with a non-zero and valid VFP table number
     bool isVFPActive(DeferredLogger& deferred_logger) const;
 
-    void reportWellSwitching(const SingleWellState<Scalar>& ws,
+    void reportWellSwitching(const SingleWellState<Scalar, IndexTraits>& ws,
                              DeferredLogger& deferred_logger) const;
 
     bool changedToOpenThisStep() const { return this->changed_to_open_this_step_; }
 
-    void updateWellTestState(const SingleWellState<Scalar>& ws,
+    void updateWellTestState(const SingleWellState<Scalar, IndexTraits>& ws,
                              const double& simulationTime,
                              const bool& writeMessageToOPMLog,
                              const bool zero_group_target,
                              WellTestState& wellTestState,
                              DeferredLogger& deferred_logger) const;
 
-    bool isPressureControlled(const WellState<Scalar>& well_state) const;
+    bool isPressureControlled(const WellStateType& well_state) const;
 
     Scalar wellEfficiencyFactor() const { return well_efficiency_factor_; }
 
@@ -220,22 +226,23 @@ protected:
     int polymerWaterTable_() const;
 
     bool wellUnderZeroRateTargetIndividual(const SummaryState& summary_state,
-                                           const WellState<Scalar>& well_state) const;
+                                           const WellState<Scalar, IndexTraits>& well_state) const;
 
-    bool wellUnderGroupControl(const SingleWellState<Scalar>& ws) const;
+    bool wellUnderGroupControl(const SingleWellState<Scalar, IndexTraits>& ws) const;
 
     std::pair<bool,bool>
     computeWellPotentials(std::vector<Scalar>& well_potentials,
-                          const WellState<Scalar>& well_state);
+                          const WellStateType& well_state);
 
     void checkNegativeWellPotentials(std::vector<Scalar>& well_potentials,
                                      const bool checkOperability,
                                      DeferredLogger& deferred_logger);
 
-    void prepareForPotentialCalculations(const SummaryState& summary_state,
-                                         WellState<Scalar>& well_state,
-                                         Well::InjectionControls& inj_controls,
-                                         Well::ProductionControls& prod_controls) const;
+    // Remove all other controls than THP and BHP
+    void onlyKeepBHPandTHPcontrols(const SummaryState& summary_state,
+                                   WellStateType& well_state,
+                                   Well::InjectionControls& inj_controls,
+                                   Well::ProductionControls& prod_controls) const;
 
     void resetDampening() {
         std::fill(this->inj_multiplier_damp_factor_.begin(), this->inj_multiplier_damp_factor_.end(), 1.0);
@@ -248,9 +255,9 @@ protected:
         {
             if (!operable_under_only_bhp_limit || !solvable || has_negative_potentials) {
                 return false;
-            } else {
-                return ( (isOperableUnderBHPLimit() || isOperableUnderTHPLimit()) );
             }
+
+            return isOperableUnderBHPLimit() || isOperableUnderTHPLimit();
         }
 
         bool isOperableUnderBHPLimit() const
@@ -304,7 +311,7 @@ protected:
     // We assume a well to not penetrate more than one pvt region.
     const int pvtRegionIdx_;
 
-    const int num_components_;
+    const int num_conservation_quantities_;
 
     // number of phases
     int number_of_phases_;
@@ -312,6 +319,8 @@ protected:
     // the index of well in Wells struct
     int index_of_well_;
 
+    const PhaseUsageInfo<IndexTraits>& phase_usage_;
+
     const std::vector<PerforationData<Scalar>>* perf_data_;
 
     // the vectors used to describe the inflow performance relationship (IPR)
@@ -367,8 +376,6 @@ protected:
 
     Well::Status wellStatus_;
 
-    const PhaseUsage* phase_usage_;
-
     Scalar gravity_;
     Scalar wsolvent_;
     std::optional<Scalar> dynamic_thp_limit_;
@@ -389,7 +396,7 @@ protected:
     std::vector<Scalar> inj_fc_multiplier_;
 
     Scalar well_efficiency_factor_;
-    const VFPProperties<Scalar>* vfp_properties_;
+    const VFPProperties<Scalar, IndexTraits>* vfp_properties_;
     const GuideRate* guide_rate_;
 
     std::vector<std::string> well_control_log_;
@@ -399,4 +406,4 @@ protected:
 
 }
 
-#endif // OPM_WELLINTERFACE_HEADER_INCLUDED
+#endif // OPM_WELLINTERFACE_GENERIC_HEADER_INCLUDED
