@@ -26,20 +26,23 @@
 
 #include <opm/common/ErrorMacros.hpp>
 
+#include <opm/material/fluidsystems/PhaseUsageInfo.hpp>
+
 #include <opm/input/eclipse/Schedule/Events.hpp>
 
 #include <opm/output/data/Wells.hpp>
 
-#include <opm/simulators/wells/ALQState.hpp>
 #include <opm/simulators/wells/GlobalWellInfo.hpp>
 #include <opm/simulators/wells/PerfData.hpp>
 #include <opm/simulators/wells/SegmentState.hpp>
 #include <opm/simulators/wells/SingleWellState.hpp>
 #include <opm/simulators/wells/WellContainer.hpp>
 
-#include <opm/simulators/utils/BlackoilPhases.hpp>
 #include <opm/simulators/utils/ParallelCommunication.hpp>
 
+#include <algorithm>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
 #include <map>
 #include <optional>
@@ -52,26 +55,33 @@ namespace Opm
 
 template<class Scalar> class ParallelWellInfo;
 template<class Scalar> struct PerforationData;
+template<class Scalar> class ConnFracStatistics;
 class Schedule;
-enum class WellStatus;
+enum class WellStatus : std::uint8_t;
 
 /// The state of a set of wells, tailored for use by the fully
 /// implicit blackoil simulator.
-template<class Scalar>
+template<typename Scalar, typename IndexTraits>
 class WellState
 {
 public:
-    static const uint64_t event_mask = ScheduleEvents::WELL_STATUS_CHANGE + ScheduleEvents::PRODUCTION_UPDATE + ScheduleEvents::INJECTION_UPDATE;
+    static const std::uint64_t event_mask = ScheduleEvents::WELL_STATUS_CHANGE
+        | ScheduleEvents::PRODUCTION_UPDATE
+        | ScheduleEvents::INJECTION_UPDATE;
+
     // TODO: same definition with WellInterface, eventually they should go to a common header file.
-    static const int Water = BlackoilPhases::Aqua;
-    static const int Oil = BlackoilPhases::Liquid;
-    static const int Gas = BlackoilPhases::Vapour;
+
+    // TODO: trying to PhasedIdx to make it more explicit as phase index, will evaluate how it goes.
+    // TODO: eventually, we should distinguish phase index and component index
+    static const int waterPhaseIdx = IndexTraits::waterPhaseIdx;
+    static const int oilPhaseIdx = IndexTraits::oilPhaseIdx;
+    static const int gasPhaseIdx = IndexTraits::gasPhaseIdx;
 
     // Only usable for testing purposes
     explicit WellState(const ParallelWellInfo<Scalar>& pinfo);
 
-    explicit WellState(const PhaseUsage& pu)
-        : phase_usage_(pu)
+    explicit WellState(const PhaseUsageInfo<IndexTraits>& pu)
+        : phaseUsageInfo_(pu)
     {}
 
     static WellState serializationTestObject(const ParallelWellInfo<Scalar>& pinfo);
@@ -91,6 +101,12 @@ public:
         return this->size();
     }
 
+    // TODO: temporary naming during development, will change to phaseUsage() later
+    const PhaseUsageInfo<IndexTraits>& phaseUsageInfo() const
+    {
+        return this->phaseUsageInfo_;
+    }
+
     const ParallelWellInfo<Scalar>& parallelWellInfo(std::size_t well_index) const;
 
     /// Allocate and initialize if wells is non-null.  Also tries
@@ -104,7 +120,8 @@ public:
               const int report_step,
               const WellState* prevState,
               const std::vector<std::vector<PerforationData<Scalar>>>& well_perf_data,
-              const SummaryState& summary_state);
+              const SummaryState& summary_state,
+              const bool enableDistributedWells);
 
     void resize(const std::vector<Well>& wells_ecl,
                 const std::vector<std::reference_wrapper<ParallelWellInfo<Scalar>>>& parallel_well_info,
@@ -112,7 +129,8 @@ public:
                 const bool handle_ms_well,
                 const std::size_t numCells,
                 const std::vector<std::vector<PerforationData<Scalar>>>& well_perf_data,
-                const SummaryState& summary_state);
+                const SummaryState& summary_state,
+                const bool enable_distributed_wells);
 
     void setCurrentWellRates(const std::string& wellName,
                              const std::vector<Scalar>& new_rates)
@@ -143,7 +161,6 @@ public:
            const std::function<bool(const int)>& wasDynamicallyClosed) const;
 
     void reportConnections(std::vector<data::Connection>& connections,
-                           const PhaseUsage &pu,
                            std::size_t well_index,
                            const int* globalCellIdxMap) const;
 
@@ -151,8 +168,9 @@ public:
     void initWellStateMSWell(const std::vector<Well>& wells_ecl,
                              const WellState* prev_well_state);
 
-    static void calculateSegmentRates(const std::vector<std::vector<int>>& segment_inlets, const
-                                      std::vector<std::vector<int>>&       segment_perforations,
+    static void calculateSegmentRates(const ParallelWellInfo<Scalar>&      pw_info,
+                                      const std::vector<std::vector<int>>& segment_inlets,
+                                      const std::vector<std::vector<int>>& segment_perforations,
                                       const std::vector<Scalar>&           perforation_rates,
                                       const int                            np,
                                       const int                            segment,
@@ -161,7 +179,10 @@ public:
 
     void communicateGroupRates(const Parallel::Communication& comm);
 
-    void updateGlobalIsGrup(const Parallel::Communication& comm);
+    void updateGlobalIsGrup(const Parallel::Communication& comm,
+                            const std::vector<WellStatus>& well_status);
+    void updateEfficiencyScalingFactor(const std::string& wellName,
+                                       const Scalar value);
 
     bool isInjectionGrup(const std::string& name) const
     {
@@ -178,54 +199,9 @@ public:
         return this->global_well_info.value().is_open(name);
     }
 
-    Scalar getALQ(const std::string& name) const
+    Scalar getGlobalEfficiencyScalingFactor(const std::string& name) const
     {
-        return this->alq_state.get(name);
-    }
-
-    void setALQ(const std::string& name, Scalar value)
-    {
-        this->alq_state.set(name, value);
-    }
-
-    int gliftGetDebugCounter()
-    {
-        return this->alq_state.get_debug_counter();
-    }
-
-    void gliftSetDebugCounter(int value)
-    {
-        return this->alq_state.set_debug_counter(value);
-    }
-
-    int gliftUpdateDebugCounter()
-    {
-        return this->alq_state.update_debug_counter();
-    }
-
-    bool gliftCheckAlqOscillation(const std::string &name) const
-    {
-        return this->alq_state.oscillation(name);
-    }
-
-    int gliftGetAlqDecreaseCount(const std::string &name)
-    {
-        return this->alq_state.get_decrement_count(name);
-    }
-
-    int gliftGetAlqIncreaseCount(const std::string &name)
-    {
-        return this->alq_state.get_increment_count(name);
-    }
-
-    void gliftUpdateAlqIncreaseCount(const std::string &name, bool increase)
-    {
-        this->alq_state.update_count(name, increase);
-    }
-
-    void gliftTimeStepInit()
-    {
-        this->alq_state.reset_count();
+        return this->global_well_info.value().efficiency_scaling_factor(name);
     }
 
     // If the ALQ has changed since the previous time step,
@@ -236,6 +212,14 @@ public:
                               const int report_step,
                               const SummaryState& summary_state);
 
+    void gliftTimeStepInit()
+    {
+        for (size_t i = 0; i < this->size(); ++i) {
+            this->wells_[i].alq_state.reset_count();
+        }
+    }
+
+
     int wellNameToGlobalIdx(const std::string& name)
     {
         return this->global_well_info.value().well_index(name);
@@ -251,6 +235,10 @@ public:
 
     bool wellIsOwned(const std::string& wellName) const;
 
+    bool isRank0() const {
+        return this->global_well_info.value().isRank0();
+    }
+
     void updateStatus(int well_index, WellStatus status);
 
     void openWell(int well_index);
@@ -258,14 +246,9 @@ public:
     void stopWell(int well_index);
 
     /// The number of phases present.
-    int numPhases() const
+    constexpr int numPhases() const
     {
-        return this->phase_usage_.num_phases;
-    }
-
-    const PhaseUsage& phaseUsage() const
-    {
-        return this->phase_usage_;
+        return phaseUsageInfo_.numActivePhases();
     }
 
     /// One rate per well and phase.
@@ -284,42 +267,42 @@ public:
         return this->wells_.well_index(well_name);
     }
 
-    const SingleWellState<Scalar>& operator[](std::size_t well_index) const
+    const SingleWellState<Scalar, IndexTraits>& operator[](std::size_t well_index) const
     {
         return this->wells_[well_index];
     }
 
-    const SingleWellState<Scalar>& operator[](const std::string& well_name) const
+    const SingleWellState<Scalar, IndexTraits>& operator[](const std::string& well_name) const
     {
         return this->wells_[well_name];
     }
 
-    SingleWellState<Scalar>& operator[](std::size_t well_index)
+    SingleWellState<Scalar, IndexTraits>& operator[](std::size_t well_index)
     {
         return this->wells_[well_index];
     }
 
-    SingleWellState<Scalar>& operator[](const std::string& well_name)
+    SingleWellState<Scalar, IndexTraits>& operator[](const std::string& well_name)
     {
         return this->wells_[well_name];
     }
 
-    const SingleWellState<Scalar>& well(std::size_t well_index) const
+    const SingleWellState<Scalar, IndexTraits>& well(std::size_t well_index) const
     {
         return this->operator[](well_index);
     }
 
-    const SingleWellState<Scalar>& well(const std::string& well_name) const
+    const SingleWellState<Scalar, IndexTraits>& well(const std::string& well_name) const
     {
         return this->operator[](well_name);
     }
 
-    SingleWellState<Scalar>& well(std::size_t well_index)
+    SingleWellState<Scalar, IndexTraits>& well(std::size_t well_index)
     {
         return this->operator[](well_index);
     }
 
-    SingleWellState<Scalar>& well(const std::string& well_name)
+    SingleWellState<Scalar, IndexTraits>& well(const std::string& well_name)
     {
         return this->operator[](well_name);
     }
@@ -334,7 +317,6 @@ public:
     template<class Serializer>
     void serializeOp(Serializer& serializer)
     {
-        serializer(alq_state);
         serializer(well_rates);
         if (serializer.isSerializing()) {
             serializer(wells_.size());
@@ -348,30 +330,39 @@ public:
         for (auto& w : wells_) {
             serializer(w);
         }
+        serializer(permanently_inactive_well_names_);
+    }
+
+    bool is_permanently_inactive_well(const std::string& wname) const {
+        return std::find(this->permanently_inactive_well_names_.begin(), this->permanently_inactive_well_names_.end(), wname) != this->permanently_inactive_well_names_.end();
     }
 
 private:
-    PhaseUsage phase_usage_;
+    bool enableDistributedWells_ = false;
+
+    PhaseUsageInfo<IndexTraits> phaseUsageInfo_;
 
     // The wells_ variable is essentially a map of all the wells on the current
     // process. Observe that since a well can be split over several processes a
     // well might appear in the WellContainer on different processes.
-    WellContainer<SingleWellState<Scalar>> wells_;
+    WellContainer<SingleWellState<Scalar, IndexTraits>> wells_;
 
-    // The members alq_state, global_well_info and well_rates are map like
+    // The members global_well_info and well_rates are map like
     // structures which will have entries for *all* the wells in the system.
 
     // Use of std::optional<> here is a technical crutch, the
     // WellStateFullyImplicitBlackoil class should be default constructible,
     // whereas the GlobalWellInfo is not.
-    std::optional<GlobalWellInfo> global_well_info;
-    ALQState<Scalar> alq_state;
+    std::optional<GlobalWellInfo<Scalar>> global_well_info;
 
     // The well_rates variable is defined for all wells on all processors. The
     // bool in the value pair is whether the current process owns the well or
     // not.
     std::map<std::string, std::pair<bool, std::vector<Scalar>>> well_rates;
 
+    // Keep track of permanently inactive well names
+    std::vector<std::string> permanently_inactive_well_names_;
+
     data::Segment
     reportSegmentResults(const int         well_id,
                          const int         seg_ix,
@@ -409,8 +400,29 @@ private:
                             Scalar temperature_first_connection,
                             const std::vector<PerforationData<Scalar>>& well_perf_data,
                             const SummaryState& summary_state);
+
+    static void calculateSegmentRatesBeforeSum(const ParallelWellInfo<Scalar>&      pw_info,
+                                               const std::vector<std::vector<int>>& segment_inlets,
+                                               const std::vector<std::vector<int>>& segment_perforations,
+                                               const std::vector<Scalar>&           perforation_rates,
+                                               const int                            np,
+                                               const int                            segment,
+                                               std::vector<Scalar>&                 segment_rates);
+
+    void reportConnectionFactors(const std::size_t well_index,
+                                 std::vector<data::Connection>& connections) const;
+
+    void reportConnectionPressuresAndRates(const std::size_t well_index,
+                                           std::vector<data::Connection>& connections) const;
+
+    void reportConnectionFilterCake(const std::size_t well_index,
+                                    std::vector<data::Connection>& connections) const;
+
+    void reportFractureStatistics(const std::vector<ConnFracStatistics<Scalar>>& stats,
+                                  std::vector<data::Connection>& connections) const;
 };
 
 } // namespace Opm
 
+// TODO: we are missing pu, while it was not there in the first place
 #endif // OPM_WELLSTATEFULLYIMPLICITBLACKOIL_HEADER_INCLUDED
