@@ -9,4 +9,6 @@
 #  where FoamSimulator and PurePythonUtils does not currently exists,
 #  but could be possible future extensions..
 #
-from .simulators import BlackOilSimulator
+from .BlackOil import BlackOilSimulator
+from .GasWater import GasWaterSimulator
+from .OnePhase import OnePhaseSimulator
