@@ -63,9 +63,9 @@ testSolver(const Opm::PropertyTree& prm, const std::string& matrix_filename, con
     if(prm.get<std::string>("preconditioner.type") == "cprt"){
         transpose = true;
     }
-    auto wc = [&matrix, transpose]()
+    std::function<Vector()> wc = [&matrix, transpose]()
     {
-        return Opm::Amg::getQuasiImpesWeights<Matrix, Vector>(matrix, 1, transpose);
+        return Opm::Amg::getQuasiImpesWeights<Matrix, Vector>(matrix, 1, transpose, false);
     };
 
     using SeqOperatorType = Dune::MatrixAdapter<Matrix, Vector, Vector>;
